-----------------------------------------------------------------------------
-- |
-- Module      :  Camila.PartialFunc
-- Copyright   :  (c) Alexandra Mendes, Joo Ferreira
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Partial functions
-- 
-----------------------------------------------------------------------------


module Camila.PartialFunc (module Camila.PartialFunc) where

import Camila.Error

-- First alternative

class Partial a where
    pre :: a -> Bool
    pre a = True
    pre' :: a -> Error a
    pre' a = if (pre a) then Ok a else Err "Pre-condition violated!"

instance (Partial a, Partial b) => Partial (a,b) where
    pre(a,b) = (pre a) && (pre b)

instance (Partial a, Partial b) => Partial (Either a b) where
    pre (Left a) = pre a
    pre (Right b) = pre b

--- Second alternative -- looks better!
--- Good improvement: it would be nice to show the precondition, when it's violated!
--- (How? Show for functions?)

data Pf a b = Pf (a -> b) (a -> Bool)

precond f pre = precheck $ Pf f pre

precheck (Pf function precondition) l | precondition l = Ok (function l)
                                      | otherwise = Err "Pre-condition failed!"


