-----------------------------------------------------------------------------
-- |
-- Module      :  Data.FiniteMapExtrasQuickCheck
-- Copyright   :  (c) Joo Ferreira, Alexandra Mendes
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Properties of Data.FiniteMapExtras implementation
-- (based on <http://www.di.uminho.pt/~jno/ps/mfp.ps.gz>, page 389)
-----------------------------------------------------------------------------


module Data.FiniteMapExtrasQuickCheck where

import Camila.Prelude
import Data.Relation.SetOfPairs
import Debug.QuickCheck
import Maybe

-----------------------------------------------------------------------------


instance (Ord a, Ord b, Arbitrary a, Arbitrary b) => Arbitrary (FiniteMap a b) where
  arbitrary = do xs <- arbitrary
                 return (listToFM xs)


instance (Ord a, Arbitrary a) => Arbitrary (Set a) where
  arbitrary = do xs <- arbitrary
                 return (mkSet xs)


prop_MUnion f g = (munion f g) == (munion g f)
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int)

prop_Override f g = 
   isEmptySet ( intersect (domFM f) (domFM g)) ==>
	override f g == override g f
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int)

prop_Munion1 f = (munionRel f f) == mkr f
 where types = f::FiniteMap Int Int

prop_Override1 f = f +++ f == f
 where types = f::FiniteMap Int Int

prop_A35 f g h = 
	override f (override g h) == override (override f g) h
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int, h::FiniteMap Int Int)

prop_A36 f = 
	Just f == munion f emptyFM
 where types = f::FiniteMap Int Int

prop_A37 f = 
	Just f == munion emptyFM f
 where types = f::FiniteMap Int Int

prop_A38 f = 	
	f == override f emptyFM
 where types = f::FiniteMap Int Int

prop_A39 f = 	
	f == override emptyFM f
 where types = f::FiniteMap Int Int

prop_A40 f g =
	dom (munionRel f g) == union (domFM f) (domFM g)
 where types = (f::FiniteMap Int Int,g::FiniteMap Int Int)

prop_A41 f g =
	domFM (override f g) == union (domFM f) (domFM g)
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int)

prop_A43 f g =
	rng (munionRel f g) == union (rngFM f) (rngFM g)
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int)

prop_A45 f g s =
	s <-: (override f g) == override (s <-: f) (s <-: g)
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int, s::Set Int)

prop_A46 f s r =
	r <-: (s <-: f)== (union s r) <-: f
 where types = (f::FiniteMap Int Int, r::Set Int, s::Set Int)

prop_48 f s r =
	mkr ((union s r) <: f) == munionRel (s <: f) (r <: f)
 where types = (f::FiniteMap Int Int, r::Set Int, s::Set Int)

prop_49 f s r = 
	s <: (r <: f) == r <: (s <: f)
 where types = (f::FiniteMap Int Int, r::Set Int, s::Set Int)

prop_50 f r =
	r <-: (r <: f) == emptySet <: f
 where types = (f::FiniteMap Int Int, r::Set Int)

prop_51 f s =
	(listToFM.setToList) (munionRel (s <-: f) (s <: f)) == f
 where types = (f::FiniteMap Int Int, s::Set Int)

prop_52 f s r =
	s <: (r <-: f) == (minusSet s r) <: f
 where types = (f::FiniteMap Int Int, r::Set Int, s::Set Int)

prop_53 f s =
	isEmptySet (intersect s (domFM f)) ==> 
		s <-: f == f
 where types = (f::FiniteMap Int Int, s::Set Int)

prop_54 f s =
	domFM (s <-: f) == minusSet (domFM f) s
 where types = (f::FiniteMap Int Int, s::Set Int)

prop_55 f s =
	domFM (s <: f) == intersect (domFM f) s
 where types = (f::FiniteMap Int Int, s::Set Int)

prop_57 f g =
	mkr (override f g) == munionRel ((domFM g) <-: f) g
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int)

prop_58 f =
	(domFM f) <: f == f
 where types = (f::FiniteMap Int Int)

prop_59 f g s =
	s <: (override f g) == override (s <: f) (s <: g)
 where types = (f::FiniteMap Int Int, g::FiniteMap Int Int, s::Set Int)

prop_60 f s =
	Just f == munion (s <-: f) (s <: f)
 where types = (f::FiniteMap Int Int,s::Set Int)

prop_61 f =
	(domFM f) <-: f == emptyFM
 where types = (f::FiniteMap Int Int)

prop_62 f s r = 
	mkr ((intersect s r) <-: f) == munionRel (s <-: f) (r <-: f)
 where types = (f::FiniteMap Int Int, s::Set Int, r::Set Int)
