-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.SetOfPairsHUnit
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Tests for the implementation of relations as sets of pairs.
--
-----------------------------------------------------------------------------

module Data.Relation.SetOfPairsHUnit where

import Data.Set
import Data.Relation.SetOfPairs
import HUnit

-----------------------------------------------------------------------------

main
  = runTestTT . TestList $ testList
  
labeledTest l t
  = TestLabel l (TestCase t)

testList
  = [ labeledTest "testEmpty" (
        do let rel = emptySet :: Set (Int,Int)
           assertEqual "Domain should be empty" emptySet (dom rel)
           assertEqual "Range should be empty" emptySet (rng rel)
           assertEqual "Carrier should be empty" emptySet (ent rel)
      ),
      labeledTest "testSingleton" (
        do let rel = mkRel [(True,False)]
           assertEqual "Domain" (mkSet [True]) (dom rel)
           assertEqual "Range" (mkSet [False]) (rng rel)
           assertEqual "Carrier" (mkSet [True,False]) (ent rel)
      ),
      labeledTest "testSliceDiamond" (
        do let diamondRel = mkRel [("a","b1"),("a","b2"),("b1","c"),("b2","c")]
           assertEqual "slice from top" 
             diamondRel 
             (slice (mkSet ["a"]) diamondRel) 
           assertEqual "slice from corner" 
             (mkRel [("b1","c")])
             (slice (mkSet ["b1"]) diamondRel) 
      ),
      labeledTest "testSliceDiamondCross" (
        do let rel = mkRel [("a","b1"),("a","b2"),("b1","c"),("b2","c"),
                            ("c","d1"),("c","d2")]
           assertEqual "slice from top" 
             rel 
             (slice (mkSet ["a"]) rel) 
           assertEqual "slice from corner" 
             (mkRel [("b1","c"),("c","d1"),("c","d2")])
             (slice (mkSet ["b1"]) rel) 
      ),
      labeledTest "testChopDiamondCross" (
        do let rel = mkRel [("a","b1"),("a","b2"),("b1","c"),("b2","c"),
                            ("c","d1"),("c","d2")]
           assertEqual "chop top and sinks" 
             rel 
             (chop (mkSet ["a"]) (mkSet ["d1","d2"]) rel) 
           assertEqual "chop between corner and sink" 
             (mkRel [("b1","c"),("c","d1")])
             (chop (mkSet ["b1"]) (mkSet ["d1"]) rel) 
      )
    ]


-----------------------------------------------------------------------------

