-----------------------------------------------------------------------------
-- |
-- Module      :  Data.Relation.TypeInferenceTest
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Tests for "Data.Relation.TypeInferenceTest".
--
-----------------------------------------------------------------------------

module Data.Relation.TypeInferenceTest where

import Data.Relation.TypeInference

import Data.Relation.SetOfPairs

-----------------------------------------------------------------------------
-- * Test

-- | Test data.
testVariableGraph :: VariableGraph String String
testVariableGraph
  = (expression,arrayIndex,assign)
    where
      expression = mkSet [("v1","v2"),("v2","v3"),("v4","v5"),("i1","i2")]
      arrayIndex = mkSet [("i1","A1"),("i2","A2"),("i3","A1")]
      assign     = mkSet [("v7","v1"),("v1","v7"),("v1","v6")]

-- | Test run.
testDot :: IO ()
testDot
 = toDot (typeInference (testVariableGraph))
      

-----------------------------------------------------------------------------
-- * Reusable

-- | Print a type graph in the dot format.
toDot :: TypeGraph String
      -> IO ()
toDot (typeEquiv,subTypeOf)
 = do let ts = types typeEquiv
      let edges = mapSet (\(x,y) -> (typeRepOf x ts,typeRepOf y ts)) subTypeOf
      putStrLn "digraph G {"
      mapM_ putStrLn (map (mkNode.concat.qsort.setToList) ts)
      mapM_ putStrLn (setToList (mapSet mkEdge edges))     
      putStrLn "}"
   where
      mkNode x = show x
      mkEdge (x,y) = (mkNode x)++" -> "++(mkNode y)
      
-- | Print a type graph in the dot format.
toDotString :: TypeGraph String -> String
toDotString (typeEquiv,subTypeOf)
 = let ts = types typeEquiv
       edges = mapSet (\(x,y) -> (typeRepOf x ts,typeRepOf y ts)) subTypeOf
   in unlines $
         ["digraph G {"] ++
         (map (mkNode.concat.qsort.setToList) ts) ++
         (setToList (mapSet mkEdge edges)) ++
         ["}"]
   where
      mkNode x = show x
      mkEdge (x,y) = (mkNode x)++" -> "++(mkNode y)

-- | Obtain the types (groups of variables) of which a given variable is 
--   an element.
typesOf :: (Ord a) => a -> [Set a] -> [Set a]
typesOf x types 
  = [t |  t <- types, x `elementOf` t]

-- | Obtain a type (group of variables) of which the given variable is an
--   element.
typeOf :: (Ord a) => a -> [Set a] -> Set a
typeOf x types
  = case (typesOf x types) of
      []     -> mkSet [x]
      (t:ts) -> t  

-- | Obtain a canonical representation for a type.
typeRepOf :: String -> [Set String] -> String
typeRepOf x types
  = concat $ qsort (setToList (typeOf x types))

-- | Quick sort (used to create canonical representations of groups
--   of variables in the dot format).
qsort        :: Ord a => [a] -> [a]
qsort []     = []
qsort (x:xs) = qsort elts_lt_x ++ [x] ++ qsort elts_greq_x
                 where
                   elts_lt_x   = [y | y <- xs, y < x]
                   elts_greq_x = [y | y <- xs, y >= x]

-----------------------------------------------------------------------------

