-----------------------------------------------------------------------------
-- |
-- Module      :  Data.SetExtras
-- Copyright   :  (c) Alexandra Mendes, Joo Ferreira
-- License     :  GPL
--
-- Maintainer  :  Joo Ferreira, Alexandra Mendes
-- Stability   :  experimental
-- Portability :  experimental
--
-- Extra functions to use with Sets
-- 
-----------------------------------------------------------------------------


module Data.SetExtras (
-- * Sets' basic functions
   filterSet,
   dunion,

-- *  File IO
   readFile_Set,
   interact_Set
) where


import Data.Relation.SetOfPairs
import Data.Set


-- | Given a predicate p and a set, yields a set whose elements validate p.
filterSet :: Ord a => (a -> Bool) -> Set a -> Set a
filterSet p s = mkSet $ filter (\key -> p key) $ setToList s

-- | Given a set of sets ss, the resulting set is the union of all the elements
--   (these are sets themselves) of ss, i.e. it contains all the elements of all the 
--   sets of ss.
dunion :: Ord a => Set (Set a) -> Set a
dunion s = foldr (\s1 s2 -> union s1 s2) emptySet $ setToList s

-- | Applies a given function to a set read from a given file.  
readFile_Set :: (Read a, Ord a, Show c) => FilePath -> ((Set a) -> c) -> IO c
readFile_Set file f = readFile file >>= return.f.read


-- | Applies readFile_Set and writes the result in a given file.
interact_Set :: (Read a, Ord a, Show c) =>
         FilePath -> FilePath -> ((Set a) -> c) -> IO ()
interact_Set inn out f = readFile_Set inn f >>= writeFile out.show
