
\documentclass[12pt]{article}

\usepackage[portuges]{babel}
\usepackage[latin1]{inputenc}

\usepackage{a4wide}
\usepackage{alltt}
\usepackage{graphics}

\title{(Generalised) LR: Examples}
\author{Jo\~ao Saraiva}


\newenvironment{code}
{\vspace{0.3cm}
\textbf{Haskell code} \hspace{1cm} \hrulefill \\
\smallskip
\begin{center}
\begin{minipage}{.80\textwidth}
\begin{alltt}\small}
{\end{alltt}
\end{minipage}
\end{center}
\hrule\smallskip
}


\begin{document}

\maketitle


\begin{code}

module Language.ContextFree.Examples where

import Language.HaLex.Dfa
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
-- import LALR
import Language.ContextFree.LR
import Language.ContextFree.G_LR

import qualified Language.ContextFree.G_SLR

-- import MemoInc_G_LR
--import MyG_LR
--import XmlPretty

import PrettyPrint.Tables.Combinators

\end{code}



\section{Examples Included in the AFP'04 Paper}

\begin{code}

joost = Cfg [T 'a',T 'b'] 
	    [NT 'S',NT 'A',NT 'C',NT 'T'] 
 	    (NT 'S')
	    [("P0",NT 'S' |-> [NT 'T']),
	     ("P1",NT 'S' |-> [NT 'T', NT 'S']),
	     ("P2",NT 'T' |-> [NT 'A', T 'b', NT 'C']),
             ("P3",NT 'A' |-> [T 'a']),
	     ("P4",NT 'C' |-> [NT 'A']),
	     ("P5",NT 'C' |-> [NT 'A', NT 'C'])]

exM = Cfg [T 'a',T 'b']
          [NT 'A',NT 'B']
          (NT 'A')
          [ ('A' |-> "-> BBA", NT 'A' |-> [NT 'B',NT 'B',NT 'A'])
          , ('A' |-> "-> a",   NT 'A' |-> [T 'a'])
          , ('B' |-> "-> b",   NT 'B' |-> [T 'b'])
          , ('B' |-> "-> Bb",  NT 'B' |-> [NT 'B', T 'b'])
          ]

aj = Cfg [T  'b']
         [NT 'S']
         (NT 'S')
         [ ("tresS", NT 'S' |-> [NT 'S',NT 'S', NT 'S'])
         , ("doisS", NT 'S' |-> [NT 'S', NT 'S'])
         , ("umb",   NT 'S' |-> [T 'b'])
         ]

aj2 = Cfg [T 'b']
          [NT 'S',NT 'T',NT 'A']
          (NT 'S')
          [ ("start",NT 'S' |-> [NT 'T'])
          , ("t1",NT 'T' |-> [NT 'A', T 'b'])
          , ("t2",NT 'T' |-> [NT 'T', NT 'T', NT 'T'])
          , ("a1",NT 'A' |-> [NT 'T', T 'b', NT 'A', NT 'A', NT 'A'])
          , ("a2",NT 'A' |-> [NT 'T', NT 'T', T 'b'])
          , ("a3",NT 'A' |-> [])
          ]

myG = Cfg [T 'a', T 'b']
	  [NT 'S',NT 'B']
	  (NT 'S')
	  [('S' |-> "-> BBBB", NT 'S' |-> [NT 'B',NT 'B',NT 'B',NT 'B']),
	   ('B' |-> "-> b",    NT 'B' |-> [T 'b']),
	   ('B' |-> "-> a",    NT 'B' |-> [T 'a'])]

expr = Cfg [T 'i',T '+',T '*']
           [NT 'E']
           (NT 'E')
           [ ("add",NT 'E' |-> [NT 'E',T '+', NT 'E'])
           , ("mul",NT 'E' |-> [NT 'E',T '*', NT 'E'])
           , ("val",NT 'E' |-> [T 'i'])
           ]

\end{code}


\section{Examples taken from lecture notes}


Examples taken from the lecture notes "Especifica\c{c}\~ao e Processamento de Linguagens".

\begin{code}

livro_pag_100 = Cfg [T 'a',T 'b']
                    [NT 'S',NT 'A']
                    (NT 'S')
                    [ ("1",NT 'S' |-> [NT 'A', T 'a'])
                    , ("2",NT 'S' |-> [T 'b'])
                    , ("3",NT 'A' |-> [NT 'A',T 'a'])
                    , ("4",NT 'A' |-> [])
                    ]


{-
pag_102_ndfa = cfg2LR_0Dfa livro_pag_100
pag_104_dfa  = cfg2LR_0Dfa livro_pag_100
pag_105_at   = slr_at      livro_pag_100
-}
\end{code}


We generate the non-deterministic LR0 automaton as follows:

\begin{verbatim}
Examples> saveLR_0Ndfa livro_pag_100
\end{verbatim}

Next we show the graphical representation:

\begin{center}
\includegraphics*{Pag100Ndfa.ps}
\end{center}


The deterministic LR0 automaton is obtained  as follows:

\begin{verbatim}
Examples> saveLR_0Ddfa livro_pag_100
\end{verbatim}


the graphical representation is:

\begin{center}
\includegraphics*{Pag100Dfa.ps}
\end{center}


\begin{verbatim}
Examples> asciiT $ pp_at pag_105_at
\end{verbatim}


\footnotesize
%\tiny
\begin{tabular}{ |l|l|l|l| }
\hline 
[([S','S',$],1),(['A'],1),(['A','A','a'],1),(['S','A','a'],1),(['S','b'],1)] & Reduce ['A'] & Shift & Error\\ 
\hline
[] & Error & Error & Error\\
\hline
[(['S','b'],2)] & Error & Error & Reduce ['S','b']\\
\hline
[([S','S',$],2)] & Error & Error & Accept\\
\hline
[(['A','A','a'],2),(['S','A','a'],2)] & Shift & Error & Error\\
\hline
[([S','S',$],3)] & Error & Error & Error\\
\hline
[(['A','A','a'],3),(['S','A','a'],3)] & Reduce ['A','A','a'] & Error & Reduce ['S','A','a']\\
\hline
\hline 
\end{tabular}
\normalsize




\begin{code}

livro_pag_110 = Cfg [T 'a',T 'b']
                    [NT 'S',NT 'A',NT 'B']
                    (NT 'S')
                    [ ("P1",NT 'S' |-> [NT 'A'])
                    , ("P2",NT 'S' |-> [NT 'B',T 'b'])
                    , ("P3",NT 'A' |-> [NT 'A',T 'a'])
                    , ("P4",NT 'A' |-> [])
                    , ("P5",NT 'B' |-> [NT 'B',T 'a'])
                    , ("P6",NT 'B' |-> [T 'b'])
                    , ("P7",NT 'B' |-> [])
                    ]


{-
livro_pag_110_LR0 = cfg2LR_0Dfa livro_pag_110


livro_pag_110_tables = slrTable2File livro_pag_110 "xx"

livro_pag_110_gen_tables = G_SLR.slrTable2File livro_pag_110 "gg"


--lr_livro_pag_110  = lraccept livro_pag_110 
glr_livro_pag_110 = inc_glraccept livro_pag_110 
--glr_livro_pag_110 = glraccept livro_pag_110 
-}
\end{code}



We prove easily that "aab" is a sentence of the language defined by cfg livro\_pag\_100


\[
S => Bb => Bab => Baab => aab
\]

However, using the lr acceptance parser we get:

\begin{verbatim}
Examples> lrcaccept livro_pag_110 "aab"
False
\end{verbatim}

while, the generalised one accepts the sentence:

\begin{verbatim}
Examples> glrcaccept livro_pag_110 "aab"
True
\end{verbatim}


\subsection{The Expression Grammar}


\begin{code}

exprC= Cfg (map T ['+','-','/','*','(',')']++map T ['0'..'9'])
           [NT 'E',NT 'I',NT 'L']
           (NT 'E')
           [ ("P1",NT 'E' |-> [NT 'E',T '+',NT 'E'])
           , ("P2",NT 'E' |-> [NT 'E',T '-',NT 'E'])
           , ("P3",NT 'E' |-> [NT 'E',T '*',NT 'E'])
           , ("P4",NT 'E' |-> [NT 'E',T '/',NT 'E'])
           , ("P5",NT 'E' |-> [NT 'L'])
	   , ("P6",NT 'L' |-> [NT 'I'])
	   , ("P7",NT 'L' |-> [NT 'I',NT 'L'])
	   , ("P8",NT 'I' |-> [T '0'])
	   , ("P9",NT 'I' |-> [T '1'])
	   , ("P10",NT 'I' |-> [T '2'])
	   , ("P11",NT 'I' |-> [T '3'])
	   , ("P12",NT 'I' |-> [T '4'])
	   , ("P13",NT 'I' |-> [T '5'])
	   , ("P14",NT 'I' |-> [T '6'])
	   , ("P15",NT 'I' |-> [T '7'])
	   , ("P16",NT 'I' |-> [T '8'])
	   , ("P17",NT 'I' |-> [T '9'])
           , ("P18",NT 'E' |-> [T '(',NT 'E', T ')'])
           ]

\end{code}

We generate the non-deterministic LR0 automaton as follows:

\begin{verbatim}
Examples> saveLR_0Ndfa expr
\end{verbatim}

Next we show the graphical representation:

\begin{center}
\includegraphics*{ExprNdfa.ps}
\end{center}


The deterministic LR0 automaton is obtained  as follows:

\begin{verbatim}
Examples> saveLR_0Ddfa expr
\end{verbatim}


the graphical representation is:

\begin{center}
\includegraphics*{ExprDfa.ps}
\end{center}



\end{document}
