module Language.ContextFree.Examples.LittleLambda where

import Language.ContextFree.CfgM
import Control.Monad.State

-- * Abstract syntax

data Type = TVar Identifier
          | Arrow Type Type
          deriving (Show,Eq)
data Expr = Var Identifier
          | Apply Expr Expr
          | Lambda Identifier Type Expr
          | Let [Decl] Expr
          deriving (Show,Eq)
data Decl = Decl Identifier Expr
          deriving (Show,Eq)
          
type Identifier = String

-- * Context-free grammar

littleLambdaCfg :: MonadState [ATerm] m => Cfg (m ()) String
littleLambdaCfg
  = Cfg ["->","\\","::"]
        ["Expr","Type","Identifier"]
        "Expr"
        [ prod "Type" ["Identifier"] "TVar",
          prod "Type" ["Type","->","Type"] "Arrow",
          prod "Expr" ["Identifier"] "Var",
          prod "Expr" ["Expr","Expr"] "Apply",
          prod "Expr" ["\\","Identifier","::","Type","->","Expr"] "Lambda",
          prod "Expr" ["let","{", "Decls", "}","in","Expr"] "Let",
          prod "Decl" ["Identifier","=","Expr"] "Decl",
          prod "Decls" ["Decl"] "#One",
          prod "Decls" ["Decl",";","Decls"] "#MoreSep"
        ]

prod :: MonadState [ATerm] m => a -> [a] -> String -> [(a, m ())]
prod nt rhs "#One"
  = (nt <$$> \ts -> [AList ts]) |-> map (<$$> id) rhs 
prod nt rhs "#MoreSep"
  = (nt <$$> \[t,sep,AList ts] -> [AList (t:ts)]) |-> map (<$$> id) rhs 
prod nt rhs tag
  = (nt <$$> \ts -> [AAppl tag ts]) |-> map (<$$> id) rhs 

-- | The abstract syntax of ATerms.

data ATerm = AAppl String [ATerm]	-- ^ Application
           | AList [ATerm]		-- ^ Lists
           | AInt Integer		-- ^ Integers
           deriving (Read,Show,Eq,Ord)

