-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Saraiva - jas@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Generalised LR Acceptance Function in Haskell
--
-----------------------------------------------------------------------------

module 	Language.ContextFree.G_LR
	( glraccept                       -- Generalised LR acceptance function
	 ,glrLazy
	 ,glrStrict
         ,glr
	) where

import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR
-- import LALR

-- | Generalised LR grammar interpreter
{-glraccept :: (Eq t, Ord t, Ord nt) =>
             Cfg t nt -> [t] -> [Bool]
-}

glraccept g inp =  glr lookupTT_g lookupAT_g [s] inp 
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        daf@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_gslr_at  eg     -- The Action Table

        lookupAT_g          = glookupAT t at    -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT

glrLazy   g inp = or     $ glraccept g inp
glrStrict g inp = length $ glraccept g inp



-- | LR Acceptance function

glr l_tt l_at sk [] = [ res
                      | ac <- acs
                      , res <- glr' l_tt l_at sk [] ac
                      ]
   where st  = top sk 
         acs = l_at st Dollar

glr l_tt l_at sk inp@(h:t) = [ res
                             | ac <- acs
                             , res <- glr' l_tt l_at sk inp ac   -- We have to Memoize this fun. call
                             ]
  where  st  = top sk
         acs = l_at st (T h)  -- It returns a list of actions


glr' l_tt l_at sk inp (Reduce pr) = glr l_tt l_at sk'' inp 
  where sk'   = popN (sizeProd pr) sk       -- reducing the stack   
        st    = top sk'                     -- acessing the state on its top 
        st'   = l_tt st (lhs_prod pr)       -- moving to a new state
        sk''  = push st' sk'                -- pushing the new state

glr' l_tt l_at sk (h:t) (Shift st) = glr l_tt l_at sk' t
  where sk'   = push st sk

glr' l_tt l_at sk inp Accept = [True]

glr' l_tt l_at sk inp Error = [False]



