-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Fernandes
-- Stability   :  experimental
-- Portability :  portable
--
-- Incremental Generalised LR Acceptance Function in Haskell
--
-----------------------------------------------------------------------------

module 	Language.ContextFree.G_LR_Inc 
	( inc_glraccept
	, inc_glrLazy
	, inc_glrStrict
	, glr_inc
	) where

import Char
import List
import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR
-- import LALR

import IOExts
-- import XmlPretty (element)

--trace x y = y 


inc_glraccept g inp =  glr_inc lookupTT_g lookupAT_g [s] inp mt  
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_gslr_at  eg      -- The Action Table

        lookupAT_g          = glookupAT t at     -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT

        mt                  = []                -- Memoisation Table

inc_glrLazy   g inp = or.snd     $ inc_glraccept g inp
inc_glrStrict g inp = length.snd $ inc_glraccept g inp


aux (a,b) = (concat a,concat b)

glr_inc l_tt l_at sk [] mt   =   aux.unzip $ map (\ac -> glr' l_tt l_at sk [] ac mt) acs
  where st  = top sk 
        acs = l_at st Dollar

glr_inc l_tt l_at sk inp@(h:t) mt = incrGlr' l_tt l_at sk inp acs mt 
  where st  = top sk 
        acs = l_at st (T h)

incrGlr' _ _ _ _ [] mt = (mt,[])
incrGlr' l_tt l_at s inp (h:t) mt = let 
				     	(mt',r) = incrGlr' l_tt l_at s inp t mt 
    				     	x       = search (s,inp,h) mt'
    				    in case x of 
        				Nothing -> let (mt'',r') = glr' l_tt l_at s inp h mt'
		   				   in trace "CACHE MISS\n" $ ((s,inp,h,r'):mt'', r'++ r)
					Just y ->  trace "CACHE HIT\n" $ (mt', y ++ r)

search _ [] = Nothing
search (s, inp,h) ((s',inp',h',r):rs) | s==s' && inp == inp' && h == h' = Just r
				      | otherwise = search (s, inp,h) rs 	

glr' l_tt l_at sk inp (Reduce pr) mt  = glr_inc l_tt l_at sk'' inp mt
  where sk'  = popN (sizeProd pr) sk       -- reducing the stack   
        st   = top sk'                     -- acessing the state on its top 
        st'  = l_tt st (lhs_prod pr)  -- moving to a new state
        sk'' = push st' sk'                -- pushing the new state


glr' l_tt l_at sk (h:t) (Shift st) mt = glr_inc l_tt l_at sk' t mt
  where sk'  = push st sk
	  
glr' l_tt l_at sk inp Accept mt = (mt,[True])

glr' l_tt l_at sk inp Error  mt = (mt,[False])



