-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Fernandes
-- Stability   :  experimental
-- Portability :  portable
--
-- Incremental generalised Acceptance Function in Haskell 
-- with semantic action of ATerm trees and counting its nodes.
--
-----------------------------------------------------------------------------

-- This module uses GHC library module Memo and the function memo that
-- memoises a function call (it uses a hash table to implement the memo
-- table).


module 	Language.ContextFree.G_LR_Inc_Semantics 
	( glr_inc_aterm
	, glr_inc_xml
	, glr_inc_sem
	, xml
	, aterm
	, memo
	) where

-----------------------------------------------------------------------------

import Char
import List
import Language.HaLex.Dfa
import Language.ContextFree.Semantics
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR

-- import LALR

import IOExts
import Text.XML.HaXml.Types
import Text.XML.HaXml.Pretty (element)
import Memo (memo)

import PrettyPrint.XML.Xml2Dot
import PrettyPrint.XML.Dot


-----------------------------------------------------------------------------

glr_inc_xml :: Cfg Char Char
           -> [Char] -> [(Bool, PrettyPrint.XML.Dot.Element)]
glr_inc_xml   = semantic_inc_glraccept xml 

glr_inc_aterm :: Cfg Char Char -> [Char] -> [(Bool, ATerm)]	
glr_inc_aterm = semantic_inc_glraccept aterm 


-- | Incremental Generalised LR grammar interpreter that builds an AST
{-semantic_inc_glraccept :: (Eq t,  
                        Ord t, Ord nt) =>
                       (Bool -> ([([Char], [Symb t nt])], Cfg t nt) -> [sem])
                       -> Cfg t nt 
		       -> [t] 
		       -> [sem]
-}
semantic_inc_glraccept f g inp = glr_inc_sem f sem lookupTT_g lookupAT_g [s] inp  
  where eg@(Cfg t nt r p)   = expand_cfg g        -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg     -- The LR_0 DFA
        at                  = e_gslr_at  eg       -- The Action Table
        lookupAT_g          = glookupAT t at      -- The lookup fun. for the AT
        lookupTT_g          = d                   -- The lookup fun. for the TT
	sem 		    = ([],g)		  -- The information needed to build the XML Tree

-- | Incremental LR Acceptance function
glr_inc_sem f semantic l_tt l_at sk [] = [ res
                                     | ac <- acs
                                     , res <- glr' f semantic l_tt l_at sk [] ac
                                     ]
  where st   = top sk 
        acs  = l_at st Dollar

glr_inc_sem f semantic l_tt l_at sk inp@(h:t) 
  = [ res
    | ac <- acs
    , res <- memo (glr' f semantic l_tt l_at) sk inp ac ]
  where  st = top sk 
         acs = l_at st (T h)    -- It returns a list of actions


glr' f (semantic, g) l_tt l_at sk inp (Reduce pr)  
  = glr_inc_sem f ((prName,pr):semantic,g) l_tt l_at sk'' inp   
  where sk'     = popN (sizeProd pr) sk        -- reducing the stack   
        st      = top sk'                      -- acessing the state on its top 
        st'     = l_tt st (lhs_prod pr)   -- moving to a new state
        sk''    = push st' sk'                 -- pushing the new state
	prName  = prodName pr (prods g)


glr' f semantic l_tt l_at sk (h:t) (Shift st) = glr_inc_sem f semantic l_tt l_at sk' t
  where sk'      = push st sk
	  
glr' f (semantic,g) l_tt l_at s inp Accept = f True  (semantic,g)  
glr' f (semantic,g) l_tt l_at s inp Error  = f False (semantic,g) 

aterm True  (semantic, g) = [(True,  toAterm g semantic)]
aterm False (semantic, g) = [(False, AAppl "null" [])]

xml True    (semantic, g) = [(True,  castElement (xmlTree g semantic))]
xml False   (semantic, g) = [(False, castElement (Elem "null" [] []))]




