-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Fernandes
-- Stability   :  experimental
-- Portability :  portable
--
-- Generalised LR parsing + construction of a XML and, or ATerm AST
--
-----------------------------------------------------------------------------

module 	Language.ContextFree.G_LR_Semantics 
	( glr_xml 
	, glr_aterm
	, glr_sem
	, element2dot
	) where

-----------------------------------------------------------------------------

import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR
import Language.ContextFree.Semantics
-- import LALR

import Text.XML.HaXml.Types
import PrettyPrint.XML.Xml2Dot
import PrettyPrint.XML.Dot

-----------------------------------------------------------------------------

element2dot :: PrettyPrint.XML.Dot.Element -> Dot
element2dot ele = lrcEval (C_CDocument_1 (C_CProlog_1 C_NoXmlDecl_1 C_NoMisc_1) ele C_NoMisc_1)

glr_xml :: (Ord t, Ord nt, Show t,Show nt) =>  Cfg t nt
           -> [t] -> [(Bool, PrettyPrint.XML.Dot.Element)]
glr_xml   = semantics_glraccept xml 

glr_aterm :: (Ord t,Ord nt,Show t,Show nt) => Cfg t nt -> [t] -> [(Bool, ATerm)]
glr_aterm = semantics_glraccept aterm 


-- | Generalised LR grammar interpreter that builds an AST
{-semantics_glraccept :: (Eq t,  
                        Ord t, Ord nt) =>
                       (Bool -> ([([Char], [Symb t nt])], Cfg t nt) -> [sem])
                       -> Cfg t nt 
		       -> [t] 
		       -> [sem]
-}
semantics_glraccept f g inp =  glr_sem f semantics lookupTT_g lookupAT_g [s] inp 
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_gslr_at  eg     -- The Action Table

        lookupAT_g          = glookupAT t at    -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT
	semantics           = ([],g)




glr_sem f semantics l_tt l_at sk [] = [ res
		                  | ac <- acs
                                  , res <- glr' f semantics l_tt l_at sk [] ac
                                  ]
   where st   = top sk 
         acs  = l_at st Dollar

glr_sem f semantics l_tt l_at sk inp@(h:t) = [ res
                                         | ac <- acs
                                         , res <- glr' f semantics l_tt l_at sk inp ac   
                                         ]
  where  st   = top sk
         acs  = l_at st (T h)    -- It returns a list of actions


glr' f (semantics,g) l_tt l_at sk inp (Reduce pr) 
  = glr_sem f ((prName,pr):semantics,g)  l_tt l_at sk'' inp 
  where sk'     = popN (sizeProd pr) sk         -- reducing the stack   
        st      = top sk'                       -- acessing the state on its top 
        st'     = l_tt st (lhs_prod pr)         -- moving to a new state
        sk''    = push st' sk'                  -- pushing the new state
	prName  = prodName pr (prods g)

glr' f semantics l_tt l_at sk (h:t) (Shift st) = glr_sem f semantics l_tt l_at sk' t
  where sk'     = push st sk

glr' f (semantics,g) l_tt l_at sk inp Accept = f True  (semantics,g)
glr' f (semantics,g) l_tt l_at sk inp Error  = f False (semantics,g)

aterm True  (semantic, g) = [(True,  toAterm g semantic)]
aterm False (semantic, g) = [(False, AAppl "null" [])]

xml :: (Eq nt, Eq t, Show nt, Show t) => Bool -> ([(String,[Symb t nt])], Cfg t nt) -> [(Bool,PrettyPrint.XML.Dot.Element)]
xml True    (semantic, g) = [(True,  castElement (xmlTree g semantic))]
xml False   (semantic, g) = [(False, castElement (Elem "null" [] []))]


