-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Saraiva - jas@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Classical LR acceptance algorithm 
--
-----------------------------------------------------------------------------

module 	Language.ContextFree.LR 
	( lraccept                           -- LR acceptance function
        , lr
	) where

------------------------------------------------------------------------------

import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR

-- import LALR

------------------------------------------------------------------------------

-- | LR Grammar Interpreter


lraccept :: (Ord t,Ord nt) 
         => Cfg t nt      -- ^ Grammar
         -> [t]           -- ^ Input sentence
         -> Bool

lraccept g inp =  lr lookupTT_g lookupAT_g [s] inp 
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_slr_at  eg      -- The Action Table

        lookupAT_g          = lookupAT t at     -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT


-- | LR Acceptance function

lr :: (st -> Symb t nt -> st)
   -> (st -> Symb t nt -> Action st [Symb t nt])
   -> [st] 
   -> [t] 
   -> Bool
lr l_tt l_at sk []      = lr' l_tt l_at sk [] ac
   where st = top sk 
         ac = l_at st Dollar



lr l_tt l_at sk inp@(h:t) = lr' l_tt l_at sk inp ac
  where  st = top sk 
         ac = l_at st (T h)

lr' l_tt l_at sk inp (Reduce pr) = lr l_tt l_at sk'' inp 
  where sk'  = popN (sizeProd pr) sk            -- reducing the stack   
        st   = top sk'                          -- acessing the state on its top 
        st'  = l_tt st (lhs_prod pr)            -- moving to a new state
        sk'' = push st' sk'                     -- pushing the new state

lr' l_tt l_at sk inp@(h:t) (Shift st) = lr l_tt l_at sk' t
  where sk'  = push st sk 

lr' l_tt l_at s inp Accept = True

lr' l_tt l_at s inp Error = False

