-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Fernandes
-- Stability   :  experimental
-- Portability :  portable
--
-- Generalised LR parsing + construction of a XML and, or ATerm AST
--
-----------------------------------------------------------------------------

module 	Language.ContextFree.LR_Semantics 
	( lr_xml 
	, lr_aterm
	, lr_sem
	, xml
	, aterm
	) where

-----------------------------------------------------------------------------

import Language.HaLex.Dfa
import Language.ContextFree.Stack
import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.Semantics
-- import LALR

import Text.XML.HaXml.Types
import PrettyPrint.XML.Xml2Dot
import PrettyPrint.XML.Dot

-----------------------------------------------------------------------------



lr_xml :: (Ord t, Ord nt, Show t,Show nt) =>  Cfg t nt
           -> [t] -> (Bool, PrettyPrint.XML.Dot.Element)
lr_xml   = semantics_lraccept xml 

lr_aterm :: (Ord t,Ord nt,Show t,Show nt) => Cfg t nt -> [t] -> (Bool, ATerm)
lr_aterm = semantics_lraccept aterm 



semantics_lraccept f g inp =  lr_sem f semantics lookupTT_g lookupAT_g [s] inp 
  where eg@(Cfg t nt r p)   = expand_cfg g      -- The expanded grammar
        dfa@(Dfa v q s z d) = ecfg2LR_0Dfa eg   -- The LR_0 DFA

        at                  = e_slr_at  eg     -- The Action Table

        lookupAT_g          = lookupAT t at    -- The lookup fun. for the AT
        lookupTT_g          = d                 -- The lookup fun. for the TT
	semantics           = ([],g)




lr_sem f semantics l_tt l_at sk [] = lr' f semantics l_tt l_at sk [] ac
   where st   = top sk 
         ac  = l_at st Dollar

lr_sem f semantics l_tt l_at sk inp@(h:t) = lr' f semantics l_tt l_at sk inp ac
  where  st   = top sk
         ac  = l_at st (T h)    


lr' f (semantics,g) l_tt l_at sk inp (Reduce pr) 
  = lr_sem f ((prName,pr):semantics,g)  l_tt l_at sk'' inp 
  where sk'     = popN (sizeProd pr) sk         -- reducing the stack   
        st      = top sk'                       -- acessing the state on its top 
        st'     = l_tt st (lhs_prod pr)         -- moving to a new state
        sk''    = push st' sk'                  -- pushing the new state
	prName  = prodName pr (prods g)

lr' f semantics l_tt l_at sk (h:t) (Shift st) = lr_sem f semantics l_tt l_at sk' t
  where sk'     = push st sk

lr' f (semantics,g) l_tt l_at sk inp Accept = f True  (semantics,g)
lr' f (semantics,g) l_tt l_at sk inp Error  = f False (semantics,g)

aterm True  (semantic, g) = (True,  toAterm g semantic)
aterm False (semantic, g) = (False, AAppl "null" [])

xml :: (Eq nt, Eq t, Show nt, Show t) => Bool -> ([(String,[Symb t nt])], Cfg t nt) -> (Bool,PrettyPrint.XML.Dot.Element)
xml True    (semantic, g) = (True,  castElement (xmlTree g semantic))
xml False   (semantic, g) = (False, castElement (Elem "null" [] []))


