-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Universidade do Minho, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  Joo Fernandes
-- Stability   :  experimental
-- Portability :  portable
--
-- Construction of the Generalised (SLR1) Action Tables
--
-----------------------------------------------------------------------------


module Language.ContextFree.Semantics where


import Char
import List

import Language.ContextFree.Cfg
import Language.ContextFree.LR_0

import Text.XML.HaXml.Types


data ATerm 	= AAppl AFun [ATerm] deriving (Eq,Show)
type AFun	= String


toAterm g ((n,p):ps) = reverseATerm $  AAppl n (fst (toAterm' g ((n, reverse (tail p)):ps)))
	--sts                 = map fst at

toAterm' g ((_,[]):ps) = ([],ps)
toAterm' g ((n,h:t):ps) | is_terminal h g = let (r',ps') =  toAterm' g ((n,t):ps)
					    in ([AAppl (showSymb h) []]++r',ps')
			| otherwise       = let ((pName, pr),prs)     = prodNt h ps
		  			        rhs_p                 = tail pr
					        (r',prs')             = toAterm' g ((pName,rhs_p):prs)
						(r'',prs'')           = toAterm' g ((n,t):prs')
					    in ([AAppl pName r']++r'',prs'') 
					      
prodNt nt [] = ((show nt,[]),[])
prodNt nt ((n,h):t) | head h == nt = ((n,h),t)
		    | otherwise = let ((n',h'),t') = prodNt nt t
			          in  ((n',h'),(n,h):t')

reverseATerm :: ATerm -> ATerm
reverseATerm (AAppl x []) = AAppl x []
reverseATerm (AAppl x  l) = AAppl x (reverse (map reverseATerm l))


func :: (Show t, Show nt) => [(String, [Symb t nt])] -> [(ProdName, [String])]
func = map (\(n,y) -> (n,map func_aux y))

func_aux :: (Show t,Show nt) => Symb t nt -> String
func_aux (T x)  = show x
func_aux (NT x) = show x

prodName pr [] = ""
prodName pr ((name, prod):t) | pr == prod = name
			     | otherwise = prodName pr t



count_Nodes (AAppl afun []) = 1
count_Nodes (AAppl afun l ) = 1 + sum (map count_Nodes l)

--xmlTree :: (Eq nt, Eq t, Show nt, Show t) => Cfg t nt -> [(String,[Symb t nt])] -> 
xmlTree g ((n,p):ps) = reverseContents $  Elem n [] (fst (xmlTree' g ((n, reverse (tail p)):ps)))

xmlTree' g ((_,[]):ps) = ([],ps)
xmlTree' g ((n,h:t):ps) | is_terminal h g = let (r',ps') =  xmlTree' g ((n,t):ps)
					    in ([CString True (showSymb h)]++r',ps')
			| otherwise       = let ((pName, pr),prs)     = prodNt h ps
		  			        rhs_p                 = tail pr
					        (r',prs')             = xmlTree' g ((pName,rhs_p):prs)
						(r'',prs'')           = xmlTree' g ((n,t):prs')
					    in ([CElem (Elem pName [] r')]++r'',prs'') 


reverseContents :: Element -> Element
reverseContents (Elem n l lc) = let lc' = reverseContent lc
			        in (Elem n l (reverse lc'))

reverseContent [] = []
reverseContent ((CString b str):t) = (CString b str): reverseContent t
reverseContent ((CElem e):t)       = (CElem (reverseContents e)):reverseContent t



