{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Parse Gnumeric spreadsheets.
--
-----------------------------------------------------------------------------

module Language.Gnumeric.Parser where

import Language.Gnumeric.Syntax
import Language.Gnumeric.SyntaxTermInstances
import Language.Gnumeric.Formula
import Language.Gnumeric.FormulaTermInstances

import Text.XML.HaXml.Xml2Haskell
import Text.XML.HaXml.Escape
import Text.XML.HaXml.Types
import Text.XML.HaXml.Parse  (xmlParse)

import System
import StrategyLib
import Control.Monad.Identity
import Debug.Trace

-----------------------------------------------------------------------------

-- | Read, parse, and show a Gnumeric file.
testGnumeric :: IO ()
testGnumeric = do
  [fileName] <- getArgs
  (workbook::Gmr'Cells) <- readGnumeric fileName
  putStrLn $ showXml workbook
  
-- | Read and parse a Gnumeric file.  
readGnumeric :: (Term a, XmlContent a) => FilePath -> IO a
readGnumeric fileName = do
  let tmpFile = "/tmp/TestGnumericTEMPFILE"
  let systemCmd = "zcat \""++fileName++"\" | egrep \"(Cell)|version\" > "++tmpFile
  --putStrLn systemCmd
  system systemCmd
  content <- readFile tmpFile 
  --putStrLn content
  let parseResult = parseFormulas $ readXml' content
  case (parseResult) of
    (Just workbook) -> return workbook
    Nothing -> error "NO PARSE"

-- | Alternative to 'readXml' that invokes 'xmlUnEscape' to transform XML
--   character references back to the charaters they represent.
readXml' :: XmlContent a => String -> Maybe a
readXml' s =
    let (Document _ _ y) = xmlParse "string input" s in
    fst (fromElem [CElem (xmlUnEscape stdXmlEscaper y)])

-- | Parse the formulas in a Gnumeric spreadsheet or a fragment of a spreadsheet.
--   The parse results are stored in the AST attribute of each cell.
parseFormulas :: Term a => a -> a
parseFormulas a
  = runIdentity $ applyTP (full_tdTP (idTP `adhocTP` worker)) a
    where
      worker :: Gmr'Cell -> Identity Gmr'Cell
      worker c@(Gmr'Cell _ "") = return c
      worker (Gmr'Cell attrs content) = do
        let f = parseFormula (mkCellId attrs) content
        let attrs' = attrs{ gmr'CellAst = Just f }
        return $
          trace ("Parsed formula: "++content
                 ++" at "++(mkCellId attrs)
                 ++" as: "++(show f))
            (Gmr'Cell attrs' content)

-- | Create the letter-number identifier for a cell, based on the row and column
--   numbers stored in the cell abstract syntax.
mkCellId :: Gmr'Cell_Attrs -> String
mkCellId attrs
  = col++row
    where
      rowNr = gmr'CellRow attrs
      colNr = gmr'CellCol attrs
      col = fromColNr (read colNr)
      row = fromRowNr (read rowNr)

-----------------------------------------------------------------------------

