-----------------------------------------------------------------------------
-- |
-- Module      :  Language.HaLex
-- Copyright   :  (c) Joo Saraiva 2001,2002,2003,2004
-- License     :  LGPL
-- 
-- Maintainer  :  jas@di.uminho.pt
-- Stability   :  provisional
-- Portability :  portable
--
-- The HaLex library contains support for lexical analysis in Haskell, 
-- including regular expressions, finite automata, and their visualization
-- as graphs. This module simply re-exports the various modules of the
-- the library.
--
-----------------------------------------------------------------------------

module Language.Halex (
  -- * Regular expressions 
  module Language.HaLex.RegExp,
  module Language.HaLex.RegExp2Fa,
  module Language.HaLex.RegExpAsDiGraph,
  
  -- * Deterministic Finite Automata
  module Language.HaLex.Dfa,
  --module Language.HaLex.DfaMonad,
  module Language.HaLex.Dfa2MDfa,
  
  -- * Non-Deterministic Finite Automata
  module Language.HaLex.Ndfa,
  
  -- * Finita Automata
  module Language.HaLex.FaClasses,
  module Language.HaLex.FaOperations,
  module Language.HaLex.FaAsDiGraph,
  
  -- * Miscelaneous
  module Language.HaLex.Minimize,
  --module Language.HaLex.Mini,
  
  module Language.HaLex.Equivalence,
  
) where

import Language.HaLex.RegExp
import Language.HaLex.RegExp2Fa
import Language.HaLex.RegExpAsDiGraph
  
import Language.HaLex.Dfa
import Language.HaLex.DfaMonad
import Language.HaLex.Dfa2MDfa
  
import Language.HaLex.Ndfa
  
import Language.HaLex.Minimize
import Language.HaLex.Mini
  
import Language.HaLex.Equivalence
  
import Language.HaLex.FaClasses
import Language.HaLex.FaOperations
import Language.HaLex.FaAsDiGraph
