--
-- Equivalence of Regular Expressions 
-- 
--
-- Code Included in the Lecture Notes on
-- 
--      Language Processing (with a functional flavour)             
--
--
-- copyright Joo Saraiva
--           Department of Computer Science,
--           University of Minho, 
--           Braga, Portugal
--           jas@di.uminho.pt
--           2001
--


module Language.HaLex.Equivalence ( 
                     equivDfa
                   , equivNdfa 
                   , equivRE
                   --, equivRegExp
                   , equivREs
                   ) where

import Language.HaLex.RegExp
import Language.HaLex.RegExp2Fa
--import Language.HaLex.RegExpParser
import Language.HaLex.Dfa
import Language.HaLex.Ndfa
import Language.HaLex.FaOperations
import Language.HaLex.Minimize

equivDfa :: (Ord st, Ord sy) => Dfa st sy -> Dfa st sy -> Bool
equivDfa dfa1 dfa2 = tdfa_1 == tdfa_2
 where canonicalDfa1 = beautifyDfa $ minimizeDfa dfa1
       canonicalDfa2 = beautifyDfa $ minimizeDfa dfa2
       tdfa_1        = dfa2tdfa canonicalDfa1
       tdfa_2        = dfa2tdfa canonicalDfa2

equivNdfa :: (Ord st, Ord sy) => Ndfa st sy -> Ndfa st sy -> Bool
equivNdfa ndfa1 ndfa2 = equivDfa (ndfa2dfa ndfa1) (ndfa2dfa ndfa2)

equivRE :: RegExp -> RegExp -> Bool
equivRE re1 re2 = equivNdfa (regExp2Ndfa re1) (regExp2Ndfa re2)

equivREs :: [RegExp] -> Bool
equivREs re = or (map (equivRE reFst) (tail re))
  where reFst = head re

{- Uses parser
equivRegExp :: String -> String -> Bool
equivRegExp re1 re2 = equiv_re abs_re1 abs_re2
  where abs_re1 = parseRegExp re1
        abs_re2 = parseRegExp re2

        equiv_re (Just p) (Just q) = equivRE p q        
        equiv_re _ _               = False
-}











