-----------------------------------------------------------------------------
-- |
-- Module      :  Language.HaLex.RegExpAsDiGraph
-- Copyright   :  (c) Joo Saraiva 2001,2002,2003,2004
-- License     :  LGPL
-- 
-- Maintainer  :  jas@di.uminho.pt
-- Stability   :  provisional
-- Portability :  portable
--
-- Regular Expressions as Directed Graphs (in GraphViz)
-- Code Included in the Lecture Notes on 
--      Language Processing (with a functional flavour).   
--
-----------------------------------------------------------------------------

module Language.HaLex.RegExpAsDiGraph ( 
                         re2graphviz
                       , re2DiGraph
                       , re2DiGraph'
                       , re2DiGraph''
                       , re2DiGraph'''
                       , re2DiGraphIO
                       , absRe2DiGraph_File
                       --, re2DiGraph_File 
                       ) where

import Language.HaLex.RegExp
import Language.HaLex.RegExp2Fa
--import Language.HaLex.RegExpParser
import Language.HaLex.Dfa
import Language.HaLex.Ndfa
import Language.HaLex.FaOperations
import Language.HaLex.Minimize
import Language.HaLex.FaAsDiGraph

-- | Print a 'RegExp' in GraphViz
re2graphviz :: RegExp     -- ^ Regular Expression (Abstract)
            -> [Char]     -- ^ Graph's Name
            -> Bool       -- ^ True: Deterministic ; False: Non-Deterministic
            -> Bool       -- ^ Minimized?
            -> Bool       -- ^ Beautified?   (states as numbers)
            -> Bool       -- ^ Sync State?
            -> [Char]

re2graphviz re n d m b s 
  | not d                        = re2DiGraphNdfa re n
  | d && not m          && s     = re2DiGraph    re n
  | d && not m && b     && not s = re2DiGraph'   re n
  | d && m     && b     && s     = re2DiGraph''' re n
  | d && m     && b     && not s = re2DiGraph''  re n
  | otherwise                    = re2DiGraph''' re n



re2DiGraph :: RegExp -> [Char] -> [Char]
re2DiGraph re name = dfa2graphviz dfa name 
  where dfa = (ndfa2dfa . regExp2Ndfa) re

re2DiGraph' re name = dfa2graphviz dfa name 
  where dfa = (beautifyDfa .ndfa2dfa . regExp2Ndfa) re

re2DiGraph'' re name = dfa2graphviz dfa name 
  where dfa = (beautifyDfa . minimizeDfa . ndfa2dfa . regExp2Ndfa) re

re2DiGraph''' re name = dfa2DiGraphWithNoSyncSt dfa name 
  where dfa = (beautifyDfaWithSyncSt . minimizeDfa . ndfa2dfa . regExp2Ndfa) re

re2DiGraphNdfa re name = ndfa2graphviz ndfa name
  where ndfa = regExp2Ndfa re  


---------------------------

re2DiGraphIO :: RegExp -> [Char] -> IO ()
re2DiGraphIO er name = dfa2graphviz2file dfa name 
  where dfa = (beautifyDfa . ndfa2dfa . regExp2Ndfa) er


absRe2DiGraph_File er name = 
        dfaDiGraphWithNoSyncStIO dfa name (name++".dot") 
  where dfa = (beautifyDfaWithSyncSt . minimizeDfa . ndfa2dfa . regExp2Ndfa) er

{- Uses parser
re2DiGraph_File :: [Char] -> [Char] -> IO ()
re2DiGraph_File c_re name = faux abst_re name 
  where abst_re = parseRegExp c_re

faux (Just p) name = absRe2DiGraph_File p name 
faux _        name = absRe2DiGraph_File Epsilon name 
-}







