-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module implements dead code elimination for Haskell.
-- Under construction!!!
-- We are not yet faithfully dealing with qualified vs. unqualified names.
-- Same holds for module level analysis.
--
-----------------------------------------------------------------------------

module Language.Haskell.DeadCode where

-------------------------------------------------------------------------------

import StrategyLib
import Monad
import List

import Language.Haskell.Syntax
import Language.Haskell.SyntaxTermInstances
import Language.Haskell.ModuleCollection
import Language.Haskell.FreeNames

-------------------------------------------------------------------------------
-- * Dead code elimination

-- | This function removes unused local declarations
hsElimDeadCode :: (Term t, MonadPlus m) => t -> m t
hsElimDeadCode = applyTP (full_tdTP worker)
  where
    worker = idTP `adhocTP` match

    match (HsMatch sl fun pats rhs {-where-} decls)
      = do (pf,pd) <- hsFreeAndDeclared pats
           (rf,rd) <- hsFreeAndDeclared rhs
           (df,dd) <- hsFreeAndDeclaredList decls
           decls'  <- filterM (hsTestDecl ((df `union` rf) \\ pd)) decls
           return (HsMatch sl fun pats rhs decls')

-- | Test wether the names declared by a given declaration are among
--   a given list of free names.
hsTestDecl :: MonadPlus m => [HsQName] -> HsDecl -> m Bool
hsTestDecl freenames decl
  | not (isValueDef decl)
  = return True
hsTestDecl freenames (HsTypeSig _ ns _)
  = do let declnames = map UnQual ns
       return $ any (`elem` freenames) declnames
hsTestDecl freenames decl
  = do (_,declnames) <- hsFreeAndDeclared decl
       return $ any (`elem` freenames) declnames

-- | Test whether a given declaration defines a value (vs a type).
isValueDef :: HsDecl -> Bool
isValueDef (HsTypeSig _ _ _) = True
isValueDef (HsFunBind _) = True
isValueDef (HsPatBind _ _ _ _) = True
isValueDef (HsInfixDecl _ _ _ _) = True
isValueDef _ = False

-------------------------------------------------------------------------------
-- * Application extraction

-- | This function removes unused top declarations from
--   a list of modules, until it reaches a fixpoint. The first
--   module is assumed to be the top modules, from which no names
--   are supposed to be deleted.
hsExtrAppl :: MonadPlus m
         => [(ModuleName,[ModuleName],HsModule)] 
         -> m [(ModuleName,[ModuleName],HsModule)]

hsExtrAppl l@(h:t)
  = do l' <- mapM worker t >>= return . (:) h
       if l==l' then return l else hsExtrAppl l'
  where
    worker (n,i,m@(HsModule sl n' i' e' ds))
      = do clients <- return $ filter (\e@(_,i',_) -> n `elem` i') l
           (imp,_) <- hsFreeAndDeclared clients
           (lcl,_) <- hsFreeAndDeclaredList ds
           ds'     <- filterM (hsTestDecl (imp++lcl)) ds
           return (n,i,HsModule sl n' i' e' ds')

-------------------------------------------------------------------------------
-- * Instances

-- | Missing instance generated by DrIFT
instance Eq HsModule where
    (HsModule aa ab ac ad ae) == (HsModule aa' ab' ac' ad' ae') 
           = aa == aa' && ab == ab' && ac == ac' && ad == ad' && ae == ae'
    _ == _ = False

-------------------------------------------------------------------------------
