-----------------------------------------------------------------------------
-- |
-- Copyright   :  (c) Joost Visser 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- This module implements some metrics on Haskell sources.
--
-----------------------------------------------------------------------------

module Language.Haskell.Metrics where

import Language.Haskell.Syntax
import Language.Haskell.ModuleGraph

import Data.Relation.SetOfPairs

-----------------------------------------------------------------------------

haskellMetricReport :: [HsModule] -> [(String,String)]
haskellMetricReport ms 
  = [nOm,cIs]
    where
      nOm = ("Number of modules",show $ length ms)
      modGraph = buildModuleGraph ms
      cIs = ("Circular import groups", 
            show $ strongNonSingletonComponentSet modGraph)

-----------------------------------------------------------------------------


{-

On module graph:

- How many modules?
- Depth and width of module graph?
- How many imports / exports per module?
- Cycles?

Per module:

- How many top-level declarations
  - How many functions?
  - How many types?
  - How many classes?
  - How many instances?
  
- Depth and width of each function?
- How many recursive / mutally recursive functions / types?
- Functions without type?

- How many instances per class?
- How many "undefined", "error", "fail" ?
- How many "quickCheck" properties?

Per function definition

- How many nested function declarations?
- How many nested applications?


-}