-------------------------------------------------------------------------------
--
-- | 
-- Module      :  Language.VDM_SL.AuxFuncLib
-- Copyright   :  (c) Tiago Alves, Paulo Silva, 2004
-- License     :  LGPL
-- 
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Auxiliary functions.
--
-------------------------------------------------------------------------------

module Language.VDM_SL.AuxFuncLib where

-- |
dom :: [(String, String)] -> [String]
dom = filterEquals . map (fst)

-- |
rng :: [(String, String)] -> [String]
rng = filterEquals . map (snd)

-- |
filterEquals :: (Eq a) => [a] -> [a]
filterEquals = filterDuplicates (==)

-- |
filterDuplicates :: (Eq a) => (a -> a -> Bool) -> [a] -> [a]
filterDuplicates _ [] = []
filterDuplicates f (x:xs) = x : (filterDuplicates f (filter (\e -> not (f e x)) xs))

-- |
union :: (Eq a) => [a] -> [a] -> [a]
union l1 l2= filterEquals $ l1 ++ l2

-- |
composition :: [(String, String)] -> [(String, String)] -> [(String, String)]
composition ori tc = 
	filterEquals [(a,b) | (a,x) <- tc, (z,b) <- ori, x == z ] 

-- | Apply in a iterative way a function to an argument while
-- the result of the function is different from the previous result
applyIter :: (Eq a) => (a -> a) -> a -> a
applyIter f arg
   | arg == (f arg) = arg
   | otherwise = applyIter f (f arg)

