{-# OPTIONS -fglasgow-exts #-}

module Pointless.Examples.Examples where

import Pointless.Combinators
import Pointless.Combinators.Uncurried
import Pointless.Functors
import Pointless.Isomorphisms
import Pointless.RecursionPatterns


import qualified List


approx n (x::a) = ((cata (_L :: Int) ((_L!) \/ (inn .) . (. out) . pmap (_L::a)) n) x)::a

-- 1 + Id

length' :: [a] -> Int
length' = ana (_L::Int) f
    where f = (id -|- snd) . out

factPara :: Int -> Int
factPara = para (_L::Int) f
    where f = one \/ (mult . (id >< succ))

plus' :: (Int,Int) -> Int
plus' = accum (_L::Int) t f
    where t = (fst -|- id >< succ) . distl
	  f = (snd \/ fst) . distl

plus'' :: (Int,Int) -> Int
plus'' = ana (_L::Int) h
    where h = (bang -|- (id >< zero \/ (zero >< id \/ succ >< id))) . coassocr . (distl -|- distl) . distr . (out >< out)
--    where h = (bang -|- ((id >< (0!)) \/ (((0!) >< id) \/ (succ >< id)))) . coassocr . (distl -|- distl) . distr . (out >< out)

plus''' :: (Int,Int) -> Int
plus''' = apo (_L::Int) h
    where h = (id -|- coswap) . coassocr . (fst -|- inn >< id) . distr . (out >< out)

ack :: Int -> Int -> Int
ack = cata (_L::Int) f
    where f = (succ!) \/ (curry (accum (_L::Int) t g . swap))
	  t = (fst -|- id) . distl
	  g = ((app . (id >< (1!))) \/ app) . distr . swap

factZygo :: Int -> Int
factZygo = zygo (_L::Int) inn f
    where f = (1!) \/ (mult . (id >< succ))


-- A * Id

type Stream a = Mu (Const a :*: Id)

generate :: Int -> Stream Int
generate = ana (_L::Stream Int) (id /\ succ)

idStream :: Stream a -> Stream a
idStream = ana (_L::Stream a) out

accumMalcolm :: ((b, a) -> a) -> (Stream b, a) -> Stream a
accumMalcolm o = ana (_L::Stream a) g
    where g = snd /\ swap . (o >< id) . assocl . (id >< swap) . assocr . (out >< id)

inits :: Stream a -> Stream [a]
inits = accumMalcolm cons . (id /\ nil . bang)

mapStream :: (a -> b) -> Stream a -> Stream b
mapStream f = ana (_L::Stream b) g 
    where g = (f >< id) . out

malcolm :: ((b, a) -> a) -> a -> Stream b -> Stream a
malcolm o e = mapStream (cata (_L::[b]) ((e!) \/ o)) . accumMalcolm cons . (id /\ nil . bang)

fmalcolm :: ((b, a) -> a) -> a -> Stream b -> Stream a
fmalcolm o e = accumMalcolm o . (id /\ (e!))

-- 1 + A * Id

wrap :: a -> [a]
wrap = cons . (id /\ nil . bang)

length :: [a] -> Int
length = cata (_L) f
    where f = zero \/ succ . snd


len' :: [a] -> Int
len' = (zero . bang \/ succ . len' . tail) . (null?)

repeat' :: a -> [a]
repeat' = ana (_L::[a]) (inr . (id /\ id))

replicate' :: (Int,a) -> [a]
replicate' = ana (_L::[a]) h
    where h = (bang -|- snd /\ id) . distl . (out >< id)

rev :: [a] -> [a]
rev = cata (_L::[a]) f 
    where f = nil \/ (cat . swap . (wrap >< id))

frev :: [a] -> [a] -> [a]
frev = cata (_L::[a]) f
    where f = pnt id \/ (comp . swap . ((curry cons) >< id))

frev' :: ([a],[a]) -> [a]
frev' = accum (_L::[a]) t f
    where t = (fst -|- assocr . (fst /\ cons . (fst >< id))) . distl
	  f = (snd \/ (snd . fst)) . distl

cauda :: [a] -> [a]
cauda = para (_L::[a]) f
    where f = ([]!) \/ snd . snd

--cauda' :: [a] -> [a]
cauda' = fst . (cata (_L::[a]) (f /\ inn . (id -|- id >< snd)))
    where f = ([]!) \/ snd . snd
	      
sum :: [Int] -> Int
sum = cata (_L::[Int]) f
    where f = (0!) \/ plus

prod :: [Int] -> Int
prod = cata (_L) f
    where f = (1!) \/ mult

fprod :: [Int] -> Int -> Int
fprod = cata (_L::[Int]) f
    where f = (id!) \/ (comp . swap . ((curry mult) >< id))

downto :: Int -> [Int]
downto = ana (_L) f
    where f = (bang -|- (id /\ pred)) . ((==0) ?)

myzip :: ([a],[b]) -> [(a,b)]
myzip = ana (_L::[(a,b)]) g
    where g = (bang -|- ((fst >< fst) /\ (snd >< snd))) . coassocl . (distl -|- distl) . distr . (out >< out)

take' :: (Int,[a]) -> [a]
take' = ana (_L::[a]) h
    where h = (bang -|- assocr . (swap >< id) . assocl) . coassocl . (distl -|- distl) . distr . (out >< out)

mult :: (Int,Int) -> Int
mult = hylo (_L::[Int]) f g
    where g = (snd -|- fst /\ id) . distr . (id >< out)
	  f = zero \/ plus

one = suck . zero

fact :: Int -> Int
fact = hylo (_L :: [Int]) f g
    where g = (id -|- succ /\ id) . out
	  f = one \/ mult

conc :: [[a]] -> [a]
conc = cata (_L :: [[a]]) g
    where g = ([]!) \/ cat

iszero = (==0)

pffact :: Int -> Int
pffact = ((1!) \/ mult) .
         (id -|- id >< pffact) . 
         (id -|- id /\ pred) . (iszero?)

part :: (Ord a) => (a,[a]) -> ([a],[a])  
part = hylo (_L::[(a,a)]) f g
    where g = (snd -|- ((id >< fst) /\ (id >< snd))) . distr . (id >< out)
	  f = (nil /\ nil) \/ (((cons >< id) . assocl . (snd >< id) \/ (id >< cons) . ((fst . snd) /\ (id >< snd)) . (snd >< id)) . (((uncurry (>)) . fst)?))

bubble :: (Ord a) => [a] -> Either One (a,[a])
bubble = cata (_L::[a]) f
    where f = id -|- ((id >< ([]!)) \/ ((id >< cons) . assocr . (id \/ (swap >< id)) . ((uncurry (<) . fst) ?) . assocl)) . distr

bsort :: (Ord a) => [a] -> [a]
bsort = ana (_L::[a]) bubble

insert :: Ord a => (a, [a]) -> [a]
insert = apo (_L::[a]) f
    where f = inr. undistr . (inr \/ (inr \/ inl)) . ((id >< nil) -|- ((id >< cons) . assocr -|- assocr . (swap >< id)) . distl . ((le?) >< id) . assocl) . distr . (id >< out)
	  le = uncurry (<=)

insert' :: Ord a => a -> [a] -> [a]
insert' x = apo (_L::[a]) f
    where f = inr . undistr . coswap . ((id \/ id) -|- id) . coassocl . (((x!) /\ nil) -|- (((x!) /\ cons) -|- id) . (((x <=) . fst)?)) . out

insert'' :: Ord a => a -> [a] -> [a]
insert'' x = para (_L::[a]) f
    where f = wrap . (x!) \/ ((x:) . cons . (id >< snd) \/ cons . (id >< fst)) . (((x <=) . fst)?)

isort :: (Ord a) => [a] -> [a]
isort = cata (_L::[a]) (nil \/ insert)

snoc' :: (a,[a]) -> [a]
snoc' = apo (_L::[a]) h
    where h = inr . undistr . coswap . (id >< nil  -|-  assocr . (swap >< id) . assocl) . distr . (id >< out)

sorted :: (Ord a) => [a] -> Bool
sorted = para (_L::[a]) f
    where f = true \/ (uncurry (&&)) . ((true . bang \/ (uncurry (<=)) . (id >< head)) . ((null . snd)?) >< id) . assocl . (id >< swap)

isumsOp (l,x) = map (x+) l

isums :: [Int] -> [Int]
isums = cata (_L::[Int]) f
    where f = nil \/ isumsOp . swap . (id >< cons . (zero . bang /\ id))

fisums :: [Int] -> Int -> [Int]
fisums = cata (_L::[Int]) f
    where f = pnt (nil . bang) \/ comp . swap . (curry plus >< (cons .) . split . (pnt id . bang /\ id))

aux (l,m) = m ++ map ((last m) +) l

i :: [Int] -> [Int]
i = cata (_L::[Int]) f
    where f = nil \/ aux . swap . (wrap >< id)

fi :: [Int] -> [Int] -> [Int]
fi = cata (_L::[Int]) f
    where f = (id!) \/ comp . swap . ((curry aux) . wrap >< id)


subsOp (r,l) = map (l++) r

subs :: Eq a => [a] -> [[a]]
subs = cata (_L::[a]) f
    where f = cons . (nil /\ nil) \/ (uncurry List.union) . (snd /\ subsOp . swap . (wrap >< id))

subsOp' (r,x) = map (x:) r

subs' :: Eq a => [a] -> [[a]]
subs' = cata (_L::[a]) f
    where f = cons . (nil /\ nil) \/ (uncurry List.union) . (snd /\ subsOp' . swap)

fsubs :: Eq a => [a] -> [a] -> [[a]]
fsubs = cata (_L::[a]) f
    where f = (wrap!) \/ ((uncurry List.union).) . split . (snd /\ (curry subsOp) . app . swap . (wrap >< id))

fsubs' :: Eq a => [a] -> [a] -> [[a]]
fsubs' = cata (_L::[a]) f
    where f = (wrap!) \/ ((uncurry List.union).) . split . (snd /\ comp . swap . ((curry snoc) >< id))

fsubs'' :: Eq a => [a] -> a -> [[a]]
fsubs'' = cata (_L::[a]) f
    where f = ((cons . (wrap /\ nil . bang))!) \/ ((uncurry List.union).) . split . (snd /\ (curry subsOp') . app . swap)

mymap :: [a] -> (a -> b) -> [b]
mymap = cata (_L::[a]) f
    where f = (([]!)!) \/ (curry (cons . (app . swap >< app) . ((fst >< id) /\ (snd >< id))))

fisums' :: ([Int],Int) -> [Int]
fisums' = accum (_L::[Int]) t f
    where t = (fst -|- ((fst . fst) /\ ((snd . fst) /\ (plus . (fst >< id))))) . distl
	  f = ((wrap . snd) \/ (cons . swap . (snd >< id))) . distl

msplit :: [a] -> ([a],[a])
msplit = cata (_L::[a]) f
    where f = (nil /\ nil) \/ (swap . (cons >< id) . assocl)

hsplit :: (Ord a) => [a] -> (a,([a],[a]))
hsplit [x] = (x,([],[]))
hsplit (h:t) | h < m     = (h,(m:l,r))
	     | otherwise = (m,(h:r,l))
	     where (m,(l,r)) = hsplit t

cat' :: [a] -> [a] -> [a]
cat' = cata (_L::[a]) f
    where f = (id!) \/ (comp . (curry cons >< id))

accumMalcolm' :: ((b, a) -> a) -> ([b], a) -> [a]
accumMalcolm' o = ana (_L::[a]) g
    where g = (fst -|- (snd /\ (id >< o) . assocr . (swap >< id))) . distl . (out >< id)

malcolm' :: ((b, a) -> a) -> a -> [b] -> [a]
malcolm' o e = map (cata (_L::[b]) ((e!) \/ o)) . accumMalcolm' cons . (id /\ nil . bang)

fmalcolm' :: ((b, a) -> a) -> a -> [b] -> [a]
fmalcolm' o e = accumMalcolm' o . (id /\ (e!))


-- A + Id

type From a = Mu (Const a :+: Id)

plus :: (Int,Int) -> Int
plus = hylo (_L::From Int) f g
    where g = (snd -|- id) . distl . (out >< id)
	  f = id \/ succ

gt :: (Int,Int) -> Bool
gt = hylo (_L :: From Bool) f g
    where g = ((((False!) \/ (True!)) \/ (False!)) -|- id) . coassocl . (distl -|- distl) . distr . (out >< out)
	  f = id \/ id

filter' :: (a -> Bool) -> [a] -> [a]
filter' p = cata (_L::[a]) f
    where f = (nil \/ (cons \/ snd)) . (id -|- ((p . fst)?))

-- A + B*Id

type NeList a b = Mu (Const a :+: (Const b :*: Id))

cat :: ([a],[a]) -> [a]
cat = hylo (_L::NeList [a] a) f g
    where g = (snd -|- assocr) . distl . (out >< id)
	  f = id \/ cons

snoc :: (a,[a]) -> [a]
snoc = hylo (_L::NeList a a) f g
    where g = (fst -|- assocr . (swap >< id) . assocl) . distr . (id >< out)
	  f = wrap \/ cons

merge :: (Ord a) => ([a],[a]) -> [a]
merge = hylo (_L::NeList [a] a) f g
    where g = ((id \/ id) -|- ((id \/ id) . (assocr -|- (assocr . (swap >< id) . assocl)) . (id >< cons -|- cons >< id) . (((uncurry (<)) . (fst >< fst))?) )) . coassocl . (snd -|- (((cons . fst) -|- id) . distr . (id >< out))) . distl . (out >< id)
	  f = id \/ cons

-- 1 + A * (Id * Id)

data Tree a = Empty | Node a (Tree a) (Tree a) deriving Show

instance FunctorOf (Const One :+: (Const a :*: (Id :*: Id))) (Tree a)
    where inn' (Inl (Const _)) = Empty
	  inn' (Inr (Const x :*: (Id l :*: Id r))) = Node x l r
	  out' Empty = Inl (Const _L)
	  out' (Node x l r) = Inr (Const x :*: (Id l :*: Id r))

nleaves :: Tree a -> Int
nleaves = cata (_L::Tree a) f
    where f = (1!) \/ (plus . snd)

nnodes :: Tree a -> Int
nnodes = cata (_L::Tree a) f
    where f = (0!) \/ (succ . plus . snd)

genTree :: Int -> Tree Int
genTree = ana (_L::Tree Int) f
    where f = (bang -|- (id /\ (pred /\ pred))) . ((==0)?)

qsort :: (Ord a) => [a] -> [a]
qsort = hylo (_L::Mu (Const One :+: (Const a :*: (Id :*: Id)))) f g
    where g = (id -|- (fst /\ part)) . out
	  f = nil \/ (cat . (id >< cons) . assocr . (swap >< id) . assocl)

preTree :: Tree a -> [a]
preTree = cata (_L::Tree a) f
    where f = ([]!) \/ (cons . (id >< cat))

postTree :: Tree a -> [a]
postTree = cata (_L::Tree a) f
    where f = ([]!) \/ (cat . swap . (wrap >< cat))

fpostTree :: Tree a -> [a] -> [a]
fpostTree = cata (_L::Tree a) f
    where f = (id!) \/ (comp . swap . (curry cons >< comp))

-- A + Id * Id

data LTree a = Leaf a | Branch (LTree a) (LTree a)

instance FunctorOf (Const a :+: (Id :*: Id)) (LTree a)
    where inn' (Inl (Const x)) = Leaf x
	  inn' (Inr (Id l :*: Id r)) = Branch l r
	  out' (Leaf x) = Inl (Const x)
	  out' (Branch l r) = Inr (Id l :*: Id r)

leaves :: LTree a -> [a]
leaves = cata (_L::LTree a) f
    where f = wrap \/ cat

fleaves :: LTree a -> [a] -> [a]
fleaves = cata (_L::LTree a) f
    where f = (curry cons) \/ comp

genLTree :: Int -> LTree Int
genLTree = ana (_L::LTree Int) f
    where f = ((0!) -|- (id /\ id)) . out

height :: LTree a -> Int
height = cata (_L::LTree a) f
    where f = (0!) \/ (succ . (uncurry max)) 

fheight'' :: LTree a -> (Int,Int) -> Int
fheight'' = cata (_L::LTree a) f
    where f = ((uncurry max)!) \/ comp . (id >< curry ((fst . snd /\ app) . (id >< (succ >< id))))

fheight' :: LTree a -> Int -> Int
fheight' = cata (_L::LTree a) f
    where f = (id!) \/ (curry (app . (id >< succ))) . (((uncurry max).) . split)

fheight :: LTree a -> Int -> Int -> Int
fheight = cata (_L::LTree a) f
    where f = (max!) \/ (.succ) . (comp.) . split

-- 1 + Id * Id

fib :: Int -> Int
fib = hylo (_L :: Mu (Const One :+: Id :*: Id)) f g
    where g = (bang -|- pred /\ pred . pred) . ((<=1)?)
	  f = one \/ plus

-- A + Id * Id

msort :: (Ord a) => [a] -> [a]
msort = hylo (_L::Mu ((Const One :+: Const a) :+: (Id :*: Id))) f g
    where g = coassocl . (id -|- (fst -|- msplit . cons) . ((null . snd)?)) . out 
	  f = (([]!) \/ wrap) \/ merge

-- A + B * (Id * Id)

hsort :: (Ord a) => [a] -> [a]
hsort = hylo f g h
    where f = _L :: Mu ((Const One :+: Const a) :+: (Const a :*: (Id :*: Id))) 
	  h = coassocl . (id -|- (fst -|- hsplit . cons) . ((null . snd)?)) . out
	  g = (([]!) \/ wrap) \/ cons . (id >< merge)

-- A * [Id]

data Rose a = Forest a [Rose a]

instance FunctorOf (Const a :*: ([] :@: Id)) (Rose a)
    where inn' (Const x :*: Comp l) = Forest x (map unId l)
	  out' (Forest x l)         = Const x :*: Comp (map Id l)

preorder :: Rose a -> [a]
preorder = cata (_L::Rose a) (cons . (id >< flatten))

flatten :: [[a]] -> [a]
flatten = cata (_L::[[a]]) (nil \/ cat)

genRose :: Int -> Rose Int
genRose = ana (_L::Rose Int) f
    where f = ((id /\ ([]!)) \/ (id /\ downto . pred)) . ((==0)?)

postRose :: Rose a -> [a]
postRose = cata (_L::Rose a) f
    where f = cat . swap . (wrap >< cata (_L::[[a]]) (nil \/ cat))

fpostRose :: Rose a -> [a] -> [a]
fpostRose = cata (_L::Rose a) f
    where f = comp . swap . (curry cons >< cata (_L::[[a]->[a]]) ((id!) \/ comp))
