{-# OPTIONS -fglasgow-exts #-}

module Pointless.Examples.Pw2pf where

import Pointless.Combinators
import Pointless.Functors
import Pointless.RecursionPatterns
import Pointless.Isomorphisms hiding (coswap)

-- Some macros...

unpnt f = app . (f . bang /\ id)

casesum e l r = app . (curry ((l \/ r) . distr) /\ e)

cases e l r = app . (eithr . (l /\ r) /\ e)

eithr :: (a -> c, b -> c) -> Either a b -> c
eithr = curry ((app \/ app) . (fst >< id -|- snd >< id) . distr)

-- Negation

nott :: Bool -> Bool
nott True  = False
nott False = True

nott_pw :: Bool -> Bool
nott_pw = \x -> case (out x) of Left  _ -> inn (inr _L)
				Right _ -> inn (inl _L)

nott_pf :: Bool -> Bool
nott_pf = unpnt (curry (casesum (out . snd) (inn . inr . bang) (inn . inl . bang)))


-- Coswap

coswap :: Either a b -> Either b a
coswap = \x -> case x of Left  y -> inr y
			 Right z -> inl z

coswap_pf :: Either a b -> Either b a
coswap_pf = unpnt (curry (cases snd (curry (inr . snd)) (curry (inl . snd))))

-- Testing for empty list

nul :: [a] -> Bool
nul []     = True
nul (x:xs) = False

nul_pw :: [a] -> Bool
nul_pw = \x -> case (out x) of Left  _ -> inn (inl _L)
			       Right _ -> inn (inr _L)

nul_pf :: [a] -> Bool
nul_pf = unpnt (curry (casesum (out . snd) (inn . inl . bang) (inn . inr . bang)))

-- Recursive identity over naturals

idnat :: Int -> Int 
idnat = mu (\f -> \x -> case (out x) of Left  y -> inn (inl _L)
	                                Right z -> inn (inr (f z))
	   )

idnat_pf :: Int -> Int
idnat_pf = unpnt (mu_pf . curry (curry (cases (out . snd) (curry (inn . inl . bang)) (curry (inn . inr . app . (snd . fst . fst /\ snd))))))

-- Recursive identity over strems

type Stream a = Mu (Const a :*: Id)

generate :: Int -> Stream Int
generate = ana (_L::Stream Int) (id /\ succ)

headrec :: Stream a -> a
headrec = mu (\f -> \x -> fst (out x))

headrec_pf :: Stream a -> a
--headrec_pf = unpnt (mu_pf . curry (curry (fst . out . snd)))
--headrec_pf = unpnt (hylo (_L :: Mu (Const a :*: Id)) app (curry (curry (fst . out . snd)) /\ id))
--headrec_pf = unpnt (hylo (_L :: Mu (Const a :*: Id)) (curry (fst . out . snd)) (id /\ id))
headrec_pf = unpnt (curry ((hylo (_L :: Mu (Const a :*: Id)) fst out) . snd)) 


headrec_hylo :: Stream a -> a
headrec_hylo = hylo (_L::Stream a) fst out

idstr :: Stream a -> Stream a
idstr = mu (\f -> \x -> inn (fst (out x), f (snd (out x))))

idstr_pf :: Stream a -> Stream a
idstr_pf = unpnt (mu_pf . (curry (curry (inn . (fst . out . snd /\ app . (snd . fst /\ snd . out . snd))))))

-- Length of a list

len :: [a] -> Int
len []     = 0
len (x:xs) = succ (len xs)

len_pw :: [a] -> Int
len_pw = \x -> case (out x) of Left  z -> inn (inl _L)
			       Right y -> inn (inr (len_pw (snd y)))

len_pf :: [a] -> Int
len_pf = unpnt (curry (casesum (out . snd) (inn . inl . bang) (inn . inr . len_pf . snd . snd)))

len_pf' :: [a] -> Int
len_pf' = inn . (id -|- len_pf' . snd) . out

hlen :: [a] -> Int
hlen = hylo (_L :: Int) g h
    where g (Left _)   = 0
	  g (Right v1) = succ v1
	  h []     = Left _L
	  h (x:xs) = Right xs
				    
hlen_pw :: [a] -> Int
hlen_pw = hylo (_L :: Int) g h
    where g = \x -> case x of Left  y -> inn (inl _L)
			      Right z -> inn (inr z)
	  h = \x -> case (out x) of Left  y -> inl y
				    Right z -> inr (snd z)

hlen_pf :: [a] -> Int
hlen_pf = hylo (_L :: Int) g h
    where g = unpnt (curry (casesum (snd) (inn . inl . bang) (inn . inr . snd)))
	  h = unpnt (curry (casesum (out . snd) (inl . bang) (inr . snd . snd)))

hlen_pf' :: [a] -> Int
hlen_pf' = hylo (_L :: Int) inn ((id -|- snd) . out)


mu :: (a -> a) -> a
mu f = f (mu f)

len_core :: [a] -> Int
len_core = mu (\f -> \x -> case (out x) of Left  _ -> inn (inl _L)
	                                   Right y -> inn (inr (f (snd y)))
	      )

fix = mu

len_pcf :: [a] -> Int
len_pcf = fix (\f -> \l -> if (null l) then 0
	                               else succ (f (tail l)))  

mu_pf :: (a -> a) -> a
mu_pf = hylo (_L :: Mu (Const (a -> a) :*: Id)) app (id /\ id)

len_pfm :: [a] -> Int
--len_pfm = unpnt (mu_pf . curry (curry (casesum (out . snd) (inn . inl . bang) (inn . inr . app . (snd . fst . fst /\ snd . snd)))))
len_pfm = unpnt (mu_pf . curry (curry (cases (out . snd) (curry (inn . inl . bang)) (curry (inn . inr . app . (snd . fst . fst /\ snd . snd))))))


-- Factorial

mult = uncurry (*)

fact :: Int -> Int
fact 0     = 1
fact (n+1) = mult (n+1, fact n)

fact_pw :: Int -> Int
fact_pw = \x -> case (out x) of Left  z -> inn (inr (inn (inl _L)))
				Right y -> mult (inn (inr y), fact_pw y)

hfact_pw :: Int -> Int
hfact_pw = hylo (_L :: [Int]) g h
    where g = \x -> case x of Left  z -> inn (inr (inn (inl _L)))
			      Right y -> mult (inn (inr (fst y)), snd y)
	  h = \x -> case (out x) of Left  z -> inl z
				    Right y -> inr (y, y)

-- Fibonacci 

plus = uncurry (+)

fib :: Int -> Int
fib 0     = 1
fib 1     = 1
fib (n+2) = plus (fib n, fib (n+1))

fib_pw :: Int -> Int
fib_pw = \x -> case (out x) 
               of Left  y -> inn (inr (inn (inl _L))) 
		  Right z -> case (out z) 
			     of Left  v -> inn (inr (inn (inl _L)))
				Right w -> plus (fib_pw w, fib_pw (inn (inr w)))

hfib_pw :: Int -> Int
hfib_pw = hylo (_L :: Mu (Const One :+: (Const One :+: (Id :*: Id)))) g h
    where g = \x -> case x 
                    of Left  y -> inn (inr (inn (inl _L)))
		       Right z -> case z
				  of Left  v -> inn (inr (inn (inl _L)))
				     Right w -> plus (fst w, snd w)
	  h = \x -> case (out x) 
                    of Left  y -> inl y 
		       Right z -> case (out z) 
				  of Left  v -> inr (inl v)
				     Right w -> inr (inr (w, inn (inr w)))
