{-# OPTIONS -fglasgow-exts #-}

module Pointless.Examples.WordCount where

import Pointless.Combinators
import Pointless.Functors
import Pointless.RecursionPatterns
import Pointless.Isomorphisms

import Data.Char

-- Using accumulations

wca :: (String, Bool) -> Int
wca = accum (_L::String) t g
    where t = (fst -|- assocr . (fst /\ fst . (isSpace . fst >< id))) . distl
	  g = (((0!) \/ (1!)) . (snd?) \/ (succ . snd . fst \/ snd . fst) . ((land . (isSpace >< not) . (fst >< id))?)) . distl

-- Using standard prelude definitions

wc :: String -> Int
wc = len . wrds

len :: [a] -> Int
len = cata (_L::[a]) g
    where g = zero \/ succ . snd

wrds :: String -> [String]
wrds = ana (_L::[String]) h
    where h = (bang -|- brk) . (null?) . drp

drp :: String -> String
drp = hylo (_L::Mu (Const [Char] :+: Id)) g h
    where g = id \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((not . isSpace . fst)?)) . out

brk :: String -> (String, String)
brk = hylo (_L::Mu (Const [Char] :+: (Const Char :*: Id))) g h
    where g = (nil . bang /\ id) \/ (cons >< id) . assocl
	  h = (inn -|- id) . coassocl . (id -|- ((isSpace . fst)?)) . out

-- After hylo-split and hylo-shift

wc1 :: String -> Int
wc1 = ana (_L::Int) h
    where h = (bang -|- snd . brk1) . (null?) . drp1

drp1 :: String -> String
drp1 = hylo (_L::Mu (Const [Char] :+: Id)) g h
    where g = id \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((not . isSpace . fst)?)) . out

brk1 :: String -> (String, String)
brk1 = hylo (_L::Mu (Const [Char] :+: (Const Char :*: Id))) g h
    where g = (nil . bang /\ id) \/ (cons >< id) . assocl
	  h = (inn -|- id) . coassocl . (id -|- ((isSpace . fst)?)) . out

-- After hylo-fusion (snd . brk) and hylo-shift

wc2 :: String -> Int
wc2 = ana (_L::Int) h
    where h = (bang -|- brk2) . (null?) . drp2 -- (id -|- brk2) . (id -|- snd) . out . drp2 

_wc2 :: String -> Int
_wc2 l = case (drp2 l) 
	 of []     -> 0
	    (x:xs) -> 1 + _wc2 (brk2 (x:xs)) -- 1 + _wc2 (brk2 xs)

wc2_t :: String -> Int
wc2_t = hylo (_L::[Char]) g h
    where h = (id -|- ((fst /\ brk2 . cons) \/ id) . ((not . isSpace . fst)?)) . out
	  g = (inn . (id -|- snd) \/ snd) . coassocl . (id -|- ((not . isSpace . fst)?))

_wc2_t :: String -> Int
_wc2_t []                       = 0
_wc2_t (x:xs) | not (isSpace x) = 1 + _wc2_t (brk2 (x:xs)) -- 1 + _wc2_t (brk2 xs)
              | otherwise       = _wc2_t xs

drp2 :: String -> String
drp2 = hylo (_L::Mu (Const [Char] :+: Id)) g h
    where g = id \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((not . isSpace . fst)?)) . out

_drp2 :: String -> String
_drp2 []                       = []
_drp2 (x:xs) | not (isSpace x) = x:xs
	     | otherwise       = _drp2 xs

brk2 :: String -> String
brk2 = hylo (_L::Mu (Const [Char] :+: Id)) g h
    where g = id \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((isSpace . fst)?)) . out

_brk2 :: String -> String
_brk2 []                 = []
_brk2 (x:xs) | isSpace x = x:xs
	     | otherwise = _brk2 xs

-- (id -|- id >< brk2) . out == out . brk2'

wc2' :: String -> Int
wc2' = ana (_L::Int) h
    where h = (id -|- snd) . out . brk2' . drp2

_wc2' :: String -> Int
_wc2' l = case (_brk2' . drp2) l 
	  of []     -> 0
	     (x:xs) -> 1 + _wc2' xs

brk2' :: String -> String
brk2' = hylo (_L::Mu (Const String :+: Id)) g h
    where g = id \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((lor . (isSpace >< null))?)) . out

_brk2' :: String -> String
_brk2' []                 = []
_brk2' (x:[])             = x:[]
_brk2' (x:xs) | isSpace x = x:xs
	      | otherwise = _brk2' xs

wc2'' :: String -> Int
wc2'' = ana (_L::Int) h
    where h = brk2'' . drp2

brk2'' :: String -> Either One String
brk2'' = hylo (_L::Mu (Const String :+: Id)) g h
    where g = (id -|- snd) . out \/ id
	  h = (inn -|- snd) . coassocl . (id -|- ((lor . (isSpace >< null))?)) . out


-- After inwards fusion

wc_t :: String -> Int
wc_t = para (_L::[Char]) g
    where g = zero \/ ((succ . fst . snd \/ fst . snd) . ((land . ((not . isSpace) >< (aux . snd)))?))
	  aux = (true \/ isSpace . fst) . out

_wc_t :: String -> Int
_wc_t []                                      = 0
_wc_t (x:[])   | not (isSpace x)              = 1 + _wc_t []
	       | otherwise                    = _wc_t []
_wc_t (x:y:xs) | not (isSpace x) && isSpace y = 1 + _wc_t (y:xs)
	       | otherwise                    = _wc_t (y:xs)

_wc_t' :: String -> Int
_wc_t' []                                      = 0
_wc_t' (x:[])                                  = 0
_wc_t' (x:y:xs) | not (isSpace x) && isSpace y = 1 + _wc_t' (y:xs)
	        | otherwise                    = _wc_t' (y:xs)

-- Auxiliary definitions

land = uncurry (&&)
lor = uncurry (||)

