module Pointless.Normalization.Curien where

import Pointless.Syntax.Types
import Pointless.Syntax.Pointwise as Pointwise
import Pointless.Syntax.Pointfree as Pointfree
import Data.Maybe
import Data.FiniteMap
import Control.Monad.Reader

toPw :: Pointfree.Term -> Pointwise.Term
toPw (ID a)         = Lam 1 a (Var 1)
toPw (f :.: g)      = let a :-> _ = fromJust (Pointfree.infer g)
			  in Lam 1 a (toPw f :@: (toPw g :@: Var 1))
toPw (FST a)        = Lam 1 a (Fst (Var 1))
toPw (SND a)        = Lam 1 a (Snd (Var 1))
toPw (f :/\: g)     = let a :-> _ = fromJust (Pointfree.infer f)
			  in Lam 1 a ((toPw f :@: Var 1) :&: (toPw g :@: Var 1))
toPw (AP (a :-> b)) = Lam 1 ((a :-> b) :*: a) (Fst (Var 1) :@: Snd (Var 1))
toPw (Curry f)      = let (a :*: b) :-> _ = fromJust (Pointfree.infer f)
			  in Lam 1 a (Lam 2 b (toPw f :@: (Var 1 :&: Var 2)))
toPw (NIL a)        = Lam 1 a Unit


-- Translation based on an evironment as nested pairs

type Ctx = (Pointwise.Term, Type)

toEnv :: Ctx -> Env
toEnv (Unit, One)              = emptyFM
toEnv (t :&: (Var x), a :*: b) = addToFM (toEnv (t, a)) x b

path :: Ctx -> Name -> Pointfree.Term
path (t :&: (Var x), a :*: b) y | x==y      = SND (a :*: b)
				| otherwise = path (t, a) y :.: FST (a :*: b)

toPf :: Pointwise.Term -> Reader Ctx Pointfree.Term
toPf (Var v)     = do ctx <- ask
		      return (path ctx v)
toPf Unit        = do a <- asks snd
		      return (NIL a)
toPf (r :&: s)   = do f <- toPf r
		      g <- toPf s
		      return (f :/\: g)
toPf (Fst t)     = do f <- toPf t
		      ctx <- ask
		      a <- return (fromJust (runReaderT (Pointwise.infer t) (toEnv ctx)))
		      return (FST a :.: f)
toPf (Snd t)     = do f <- toPf t
		      ctx <- ask
		      a <- return (fromJust (runReaderT (Pointwise.infer t) (toEnv ctx)))
		      return (SND a :.: f)
toPf (Lam v a t) = do f <- local (\(r, b) -> (r :&: (Var v), b :*: a)) (toPf t)
		      return (Curry f)
toPf (f :@: t)   = do g <- toPf f
		      h <- toPf t
		      ctx <- ask
		      a <- return (fromJust (runReaderT (Pointwise.infer f) (toEnv ctx)))
		      return (AP a :.: (g :/\: h))
