module Pointless.Normalization.Evaluation where

import Pointless.Syntax.Types
import Pointless.Syntax.Pointwise as Pointwise
import Pointless.Syntax.Pointfree as Pointfree
import Data.Maybe
import Data.FiniteMap
import Control.Monad.Reader
import Pointless.Normalization.Curien

type TERM = Int -> Pointwise.Term

data Value = UNIT | VAR TERM | PAIR (Value, Value) | LAM (Value -> Value)

type Valuation = FiniteMap Name Value

eval :: Pointwise.Term -> Valuation -> Value
eval Unit        v = UNIT
eval (Var i)     v = fromJust (lookupFM v i)
eval (x :&: y)   v = PAIR (eval x v, eval y v)
eval (Fst x)     v = case (eval x v)
		     of PAIR y -> fst y
eval (Snd x)     v = case (eval x v)
		     of PAIR y -> snd y
eval (Lam i _ x) v = LAM (\a -> eval x (addToFM v i a))
eval (f :@: x)   v = case (eval f v) 
		     of LAM g -> g (eval x v)

pfeval :: Pointfree.Term -> Value
pfeval (ID _)     = LAM id
pfeval (f :.: g)  = let LAM h = pfeval f
			LAM i = pfeval g
			in LAM (h . i)
pfeval (FST _)    = LAM (\x -> case x of PAIR v -> fst v)
pfeval (SND _)    = LAM (\x -> case x of PAIR v -> snd v)
pfeval (f :/\: g) = let LAM h = pfeval f
			LAM i = pfeval g
			in LAM (\x -> PAIR (h x, i x))
pfeval (AP _)     = LAM (\x -> case x of PAIR (LAM f, y) -> f y)
pfeval (Curry f)  = let LAM g = pfeval f
			in LAM (\x -> LAM (\y -> g (PAIR (x,y))))
pfeval (NIL _)    = LAM (\x -> UNIT)

reify :: Type -> Value -> TERM
reify One       UNIT     = const Unit
reify (Base _)  (VAR t)  = t
reify (a :*: b) (PAIR t) = \i -> reify a (fst t) i :&: reify b (snd t) i
reify (a :-> b) (LAM f)  = \i -> Lam i a (reify b (f (reflect a (const (Var i)))) (i+1))

reflect :: Type -> TERM -> Value
reflect One       t = UNIT
reflect (Base _)  t = VAR t
reflect (a :*: b) t = PAIR (reflect a (\i -> Fst (t i)), reflect b (\i -> Snd (t i)))
reflect (a :-> b) t = LAM (\x -> reflect b (\i -> t i :@: reify a x i))

nbe :: Pointwise.Term -> Pointwise.Term
nbe t = reify a (eval t emptyFM) 1
    where a = fromJust (runReaderT (Pointwise.infer t) emptyFM)

pfnbe :: Pointfree.Term -> Pointwise.Term
pfnbe f = reify a (pfeval f) 1
    where a = fromJust (Pointfree.infer f)