{-# OPTIONS -fglasgow-exts -fth #-}

module Pointless.Normalization.Rewrite where

import Control.Monad.Reader hiding (lift)
import Data.Generics hiding (Unit,(:*:))
import Pointless.Syntax.Types
import Pointless.Syntax.Pointwise
--import Pointless.Syntax.Pointfree
import Data.Maybe
import Data.List
import Data.FiniteMap

-- Pointfree rewriting
{-
type PFM = ReaderT [PF] Maybe

once :: MonadPlus m => GenericM m -> GenericM m
once f x = f x `mplus` gmapMo (once f) x

many :: (MonadPlus m) => GenericM m -> GenericM m
many f x = ((once f) x >>= (many f)) `mplus` (return x) 

pfbeta :: PF -> PFM PF
pfbeta a = do b <- once (mkMp beta) a
	      many (mkMp subst) b
    where beta (Comp (AP _) (Split (Curry x) y)) = let Just (F c _) = pf2tp y
						       in return (Comp x (Split (ID c) y))
	  beta _ = fail "Beta not applicable!"
	  subst (Comp (Comp x y) z) = return (Comp x (Comp y z))
	  subst (Comp (ID _) x) = return x
	  subst (Comp x (ID _)) = return x
	  subst (Comp (FST _) (Split x y)) = return x
	  subst (Comp (SND _) (Split x y)) = return y
	  subst (Comp (Split x y) z) = return (Split (Comp x z) (Comp y z))
	  subst (Split (FST a) (SND b)) = return (ID a)
	  subst (Split (Comp (FST _) x) (Comp (SND _) y)) | x==y = return x
	  subst (Comp (Curry x) y) = let Just (F d _) = pf2tp y
					 Just (F (P _ b) _) = pf2tp x 
					 in return (Curry (Comp x (Split (Comp y (FST (P d b))) (SND (P d b)))))
	  subst _ = fail "Subst not applicable!"

pfpair :: PF -> PFM PF
pfpair = once (mkMp aux)
    where aux (Comp (FST _) (Split x y)) = return x
	  aux (Comp (SND _) (Split x y)) = return y
	  aux _                          = fail "Pair cancelation not applicable!"

-- Variables must be taken as indivisible entities
pfonce :: (PF -> PFM PF) -> PF -> PFM PF
pfonce f x | isVar x = f x
	   where isVar (SND _)          = True
		 isVar (Comp x (FST _)) = isVar x
		 isVar _                = False
pfonce f x = f x `mplus` local (x:) (gmapMo (mkMp (pfonce f)) x)

-- Application must be taken as an indivisible entities. It suffices to not expand AP.
pfunit :: PF -> PFM PF
pfunit = pfonce (mkMp aux)
    where aux (NIL _) = fail "Cannot unit expand nil!"
	  aux (AP _)  = fail "Cannot expand AP!"
	  aux e       = case pf2tp e of Just (F a U) -> return (NIL a)
					otherwise    -> fail "Unit expansion not applicable!"

pfsp :: PF -> PFM PF
pfsp = pfonce aux
    where aux (Split _ _) = fail "Cannot pair expand a split!"
	  aux (AP _)      = fail "Cannot expand AP!"
	  aux e           = do p <- ask
			       when (not (null p) && projected (head p)) (fail "Cannot pair expand a projected term!")
			       case pf2tp e of Just (F _ (P a b)) -> return (Split (Comp (FST (P a b)) e) (Comp (SND (P a b)) e))
					       otherwise          -> fail "Pair expansion not applicable!"
	  projected (Comp (FST _) _) = True
	  projected (Comp (SND _) _) = True
	  projected _                = False

pfeta :: PF -> PFM PF
pfeta = pfonce aux
    where aux (Curry _) = fail "Cannot eta expand a curry!"
	  aux (AP _)    = fail "Cannot expand AP!"    
	  aux e         = do p <- ask
			     when (length p > 1 && applied e (p!!1)) (fail "Cannot eta expand an applied term!")
			     case pf2tp e of Just (F a (F b c)) -> return (Curry (Comp (AP (F b c)) (Split (Comp e (FST (P a b))) (SND (P a b)))))
		       			     otherwise          -> fail "Eta expansion not applicable!"
	  applied x (Comp (AP _) (Split y _)) | x==y = True
	  applied _ _                                = False

pf2pf :: PF -> PF
pf2pf e = fromJust (runReaderT (many (pfbeta `plus` pfpair `plus` pfeta `plus` pfsp `plus` pfunit) e) [])
    where plus f g x = (f x) `mplus` (g x)
	  many f x = (f x >>= (many f)) `mplus` (return x) 

-}

--
-- Pointwise
--


data Context = Context {env :: Env, parent :: Maybe Term}

emptyCt = Context {env = emptyFM, parent = Nothing}


type Rule = Term -> ReaderT Context Maybe Term

once :: Rule -> Rule
once r e@(Lam v t x) = r e `mplus` 
    local (\c -> c {env = addToFM (env c) v t, parent = Just e}) 
	  (gmapMo (mkMp (once r)) e)
once r e = r e `mplus`
    local (\c -> c {parent = Just e}) (gmapMo (mkMp (once r)) e)

many :: Rule -> Rule
many r x = (once r x >>= many r) `mplus` (return x) 

plus :: Rule -> Rule -> Rule
plus f g x = (f x) `mplus` (g x)


isElim :: Maybe Term -> Bool
isElim Nothing          = True
isElim (Just (Fst _))   = True
isElim (Just (Snd _))   = True
isElim (Just (_ :@: _)) = True
isElim _                = False

eta_unit :: Rule
eta_unit Unit = fail "Cannot expand unit!"
eta_unit e = 
    do env <- asks env
       case (runReaderT (infer e) env) 
	    of Just One  -> return Unit
	       otherwise -> fail "Unit expansion not applicable!"

beta_pair :: Rule
beta_pair (Fst (x :&: y)) = return x
beta_pair (Snd (x :&: y)) = return y
beta_pair _               = fail "Product cancelation not applicable!"

eta_pair :: Rule
eta_pair (_ :&: _) = fail "Cannot expand a pair!"
eta_pair e = 
    do env    <- asks env
       parent <- asks parent
       when (isElim parent) (fail "Cannot expand a projected term!")
       case (runReaderT (infer e) env)
	    of Just (_ :*: _) -> return (Fst e :&: Snd e)
	       otherwise      -> fail "Pair expansion not applicable!"

beta_ap :: Rule
beta_ap (Lam v t x :@: y) = 
    return (runReader (replace x) (unitFM v y))
beta_ap _ = fail "Beta reduction not applicable!"

eta_func :: Rule
eta_func (Lam _ _ _) = fail "Cannot expand an abstraction!"
eta_func e = 
    do env    <- asks env
       parent <- asks parent
       when (isElim parent) (fail "Cannot expand an applied term!")
       case (runReaderT (infer e) env)
	    of Just (a :-> _) -> 
		   do let new = succ (maximum (free e))
		      return (Lam new a (e :@: (Var new)))
	       otherwise      -> fail "Eta expansion not applicable!"

ccc :: Rule
ccc = many (once (beta_ap  `plus` beta_pair `plus` 
		  eta_unit `plus` eta_pair  `plus` eta_func))

nbr :: Term -> Term
nbr t = fromJust (runReaderT (ccc t) emptyCt)


