{-# OPTIONS -fglasgow-exts -fth #-}

module Pointless.Pointwise where

import Pointless.Combinators
import Language.Haskell.THSyntax
import Data.List
import Data.Maybe
import Control.Monad
import Control.Monad.Reader hiding (lift)
import Data.Generics hiding (Unit)

type ID = Int

data TP = U
	| B ID
	| P TP TP
	| F TP TP
	  deriving (Eq, Typeable, Data)

instance Show TP
    where show U       = "()"
	  show (B t)   = "a"++(show t)
	  show (P t u) = "("++(show t)++" , "++(show u)++")"
	  show (F t u) = "("++(show t)++" -> "++(show u)++")"

data PF = ID TP
	| Comp PF PF
	| FST TP
	| SND TP
	| Split PF PF
	| AP TP
	| Curry PF
	| NIL TP
	| CONST String TP
	  deriving (Eq, Typeable, Data)

nil :: a -> ()
nil = (()!)

{-
instance Show PF
    where show (ID t)            = "(id :: "++(show (F t t))++")"
	  show (Comp f g)        = (show f) ++ " . " ++ (show g)
	  show (FST (P a b))     = "(fst :: "++(show (F (P a b) a))++")"
	  show (SND (P a b))     = "(snd :: "++(show (F (P a b) b))++")"
	  show (Split f g)       = "("++(show f)++" /\\ "++(show g)++")"
	  show (AP (F a b))      = "(app :: "++(show (F (P (F a b) a) b))++")"
	  show (Curry f)         = "curry ("++(show f)++")"
	  show (NIL t)           = "(nil :: "++(show (F t U))++")"
	  show (CONST v (F a b)) = "("++v++"! :: "++(show (F a b))++")"
-}

instance Show PF
    where show f = (aux f)++" :: "++(show (fromJust (pf2tp f)))
	      where aux (ID _)       = "id"
		    aux (Comp f g)  = (aux f) ++ " . " ++ (aux g)
		    aux (FST _)     = "fst" 
		    aux (SND _)     = "snd"
		    aux (Split f g) = "("++(aux f)++" /\\ "++(aux g)++")"
		    aux (AP _)      = "app"
		    aux (Curry f)   = "curry ("++(aux f)++")"
		    aux (NIL _)     = "nil"
		    aux (CONST v _) = "("++v++"!)"

-- The type annotations disappear because we don't have scoped type variables in the syntax
instance Lift PF
    where lift (ID _)      = varE "id"
	  lift (Comp f g)  = infixApp (lift f) (varE ".") (lift g)
	  lift (FST _)     = varE "fst"
	  lift (SND _)     = varE "snd"
	  lift (Split f g) = infixApp (lift f) (varE "/\\") (lift g)
	  lift (AP _)      = varE "app"
	  lift (Curry f)   = appE (varE "curry") (lift f)
	  lift (NIL _)     = varE "nil"
	  lift (CONST v _) = varE ("("++v++"!)")

pf2tp :: PF -> Maybe TP
pf2tp (ID t)            = return (F t t)
pf2tp (Comp f g)        = do (F a b) <- pf2tp f
			     (F c d) <- pf2tp g
			     unless (a == d) (fail "Type error!!")
			     return (F c b)
pf2tp (FST (P a b))     = return (F (P a b) a)
pf2tp (SND (P a b))     = return (F (P a b) b)
pf2tp (Split f g)       = do (F a b) <- pf2tp f
			     (F c d) <- pf2tp g
			     unless (a == c) (fail "Type error!!")
			     return (F a (P b d))
pf2tp (AP (F a b))      = return (F (P (F a b) a) b)
pf2tp (Curry f)         = do (F a b) <- pf2tp f
			     case a of (P c d) -> return (F c (F d b))
				       otherwise -> fail "Type error!!"
pf2tp (NIL t)           = return (F t U)
pf2tp (CONST _ (F a b)) = return (F a b)

-- Pointfree rewriting

type PFM = ReaderT [PF] Maybe

once :: MonadPlus m => GenericM m -> GenericM m
once f x = f x `mplus` gmapMo (once f) x

many :: (MonadPlus m) => GenericM m -> GenericM m
many f x = ((once f) x >>= (many f)) `mplus` (return x) 

pfbeta :: PF -> PFM PF
pfbeta a = do b <- once (mkMp beta) a
	      many (mkMp subst) b
    where beta (Comp (AP _) (Split (Curry x) y)) = let Just (F c _) = pf2tp y
						       in return (Comp x (Split (ID c) y))
	  beta _ = fail "Beta not applicable!"
	  subst (Comp (Comp x y) z) = return (Comp x (Comp y z))
	  subst (Comp (ID _) x) = return x
	  subst (Comp x (ID _)) = return x
	  subst (Comp (FST _) (Split x y)) = return x
	  subst (Comp (SND _) (Split x y)) = return y
	  subst (Comp (Split x y) z) = return (Split (Comp x z) (Comp y z))
	  subst (Split (FST a) (SND b)) = return (ID a)
	  subst (Split (Comp (FST _) x) (Comp (SND _) y)) | x==y = return x
	  subst (Comp (Curry x) y) = let Just (F d _) = pf2tp y
					 Just (F (P _ b) _) = pf2tp x 
					 in return (Curry (Comp x (Split (Comp y (FST (P d b))) (SND (P d b)))))
	  subst _ = fail "Subst not applicable!"

pfpair :: PF -> PFM PF
pfpair = once (mkMp aux)
    where aux (Comp (FST _) (Split x y)) = return x
	  aux (Comp (SND _) (Split x y)) = return y
	  aux _                          = fail "Pair cancelation not applicable!"

-- Variables must be taken as indivisible entities
pfonce :: (PF -> PFM PF) -> PF -> PFM PF
pfonce f x | isVar x = f x
	   where isVar (SND _)          = True
		 isVar (Comp x (FST _)) = isVar x
		 isVar _                = False
pfonce f x = f x `mplus` local (x:) (gmapMo (mkMp (pfonce f)) x)

-- Application must be taken as an indivisible entities. It suffices to not expand AP.
pfunit :: PF -> PFM PF
pfunit = pfonce (mkMp aux)
    where aux (NIL _) = fail "Cannot unit expand nil!"
	  aux (AP _)  = fail "Cannot expand AP!"
	  aux e       = case pf2tp e of Just (F a U) -> return (NIL a)
					otherwise    -> fail "Unit expansion not applicable!"

pfsp :: PF -> PFM PF
pfsp = pfonce aux
    where aux (Split _ _) = fail "Cannot pair expand a split!"
	  aux (AP _)      = fail "Cannot expand AP!"
	  aux e           = do p <- ask
			       when (not (null p) && projected (head p)) (fail "Cannot pair expand a projected term!")
			       case pf2tp e of Just (F _ (P a b)) -> return (Split (Comp (FST (P a b)) e) (Comp (SND (P a b)) e))
					       otherwise          -> fail "Pair expansion not applicable!"
	  projected (Comp (FST _) _) = True
	  projected (Comp (SND _) _) = True
	  projected _                = False

pfeta :: PF -> PFM PF
pfeta = pfonce aux
    where aux (Curry _) = fail "Cannot eta expand a curry!"
	  aux (AP _)    = fail "Cannot expand AP!"    
	  aux e         = do p <- ask
			     when (length p > 1 && applied e (p!!1)) (fail "Cannot eta expand an applied term!")
			     case pf2tp e of Just (F a (F b c)) -> return (Curry (Comp (AP (F b c)) (Split (Comp e (FST (P a b))) (SND (P a b)))))
		       			     otherwise          -> fail "Eta expansion not applicable!"
	  applied x (Comp (AP _) (Split y _)) | x==y = True
	  applied _ _                                = False

pf2pf :: PF -> PF
pf2pf e = fromJust (runReaderT (many (pfbeta `plus` pfpair `plus` pfeta `plus` pfsp `plus` pfunit) e) [])
    where plus f g x = (f x) `mplus` (g x)
	  many f x = (f x >>= (many f)) `mplus` (return x) 


--
-- Pointwise
--

data PW = Var ID
	| Lambda ID TP PW
	| Unit
	| Ap PW PW
	| Pair PW PW
	| Fst PW
	| Snd PW
	| Const String TP
	  deriving (Eq, Typeable, Data)

{-
instance Show PW
    where show (Var v)        = "v"++(show v)
	  show (Lambda v t e) = "(\\ v" ++ (show v) ++ " :: " ++ (show t) ++ " -> " ++ (show e) ++ ")"
	  show Unit           = "()"
	  show (Ap f e)       = (show f) ++ " (" ++ (show e) ++ ")"
	  show (Pair l r)     = "(" ++ (show l) ++ " , " ++ (show r) ++ ")"
	  show (Fst e)        = "fst (" ++ (show e) ++ ")"
	  show (Snd e)        = "snd (" ++ (show e) ++ ")"
	  show (Const x t)    = x ++ " :: " ++ (show t)
-}

instance Show PW
    where show e = (aux e)++" :: "++(show (fromJust (pw2tp [] e)))
	      where aux (Var v)        = "v"++(show v)
		    aux (Lambda v _ e) = "(\\v"++(show v)++" -> "++(aux e)++")"
		    aux Unit           = "()"
		    aux (Ap f e)       = (aux f)++" ("++(aux e)++")"
		    aux (Pair l r)     = "("++(aux l)++" , "++(aux r)++")"
		    aux (Fst e)        = "fst ("++(aux e)++")"
		    aux (Snd e)        = "snd ("++(aux e)++")"
		    aux (Const x _)    = x

instance Lift PW
    where lift (Var v)        = varE ("v"++(show v))
	  lift (Lambda v _ e) = lam1E (varP ("v"++(show v))) (lift e)
	  lift Unit           = conE "()"
	  lift (Ap f e)       = appE (lift f) (lift e)
	  lift (Pair l r)     = tupE [lift l, lift r]
	  lift (Fst e)        = appE (varE "fst") (lift e)
	  lift (Snd e)        = appE (varE "snd") (lift e)
	  lift (Const x _)    = conE x

-- Typing

type Context = [(ID,TP)]

pw2tp :: Context -> PW -> Maybe TP
pw2tp c (Var v)        = lookup v c
pw2tp c (Lambda v t e) = do te <- pw2tp ((v,t):c) e
			    return (F t te)
pw2tp c Unit           = return U
pw2tp c (Ap f e)       = do te <- pw2tp c e
			    tf <- pw2tp c f 
			    case tf of (F a b)   -> if (a == te) 
						    then return b
						    else fail "Type error!!"
				       otherwise -> fail "Type error!!"
pw2tp c (Pair x y)     = do tx <- pw2tp c x
			    ty <- pw2tp c y
			    return (P tx ty)
pw2tp c (Fst x)        = do tx <- pw2tp c x
			    case tx of (P a b) -> return a
				       otherwise -> fail "Type error!!"
pw2tp c (Snd x)        = do tx <- pw2tp c x
			    case tx of (P a b) -> return b
				       otherwise -> fail "Type error!!"
pw2tp c (Const x t)    = return t

-- Free vars manipulation using boilerplate technique

freeVars :: PW -> [ID]
freeVars = aux
    where aux :: Data a => a -> [ID]
	  aux x = ((([] `mkQ` refsFun) x) `union` (nub . concat . gmapQ aux) x) \\ ([] `mkQ` decsFun) x
	  refsFun (Var v) = [v]
	  refsFun _       = []
	  decsFun (Lambda v _ _) = [v]
	  decsFun _              = []
	  
-- Pointwise rewrite rules

type PWM = ReaderT (Context,[PW]) Maybe

pwonce :: (PW -> PWM PW) -> PW -> PWM PW
pwonce f x@(Lambda v t _) = f x `mplus` local (\(c,p) -> ((v,t):c,x:p)) (gmapMo (mkMp (pwonce f)) x)
pwonce f x                = f x `mplus` local (\(c,p) -> (c,x:p)) (gmapMo (mkMp (pwonce f)) x)

pwbeta :: PW -> PWM PW
pwbeta = pwonce aux
    where aux (Ap (Lambda v t a) b) = return (subst v a b)
	  aux _                     = fail "Beta reduction not applicable!"
	  subst v (Var x) e | x == v = e
			    | otherwise = Var x
	  subst v (Lambda x t a) e | x == v = Lambda x t a
				   | x `elem` (freeVars e) = let u = succ (maximum (freeVars e ++ freeVars a))
								 in subst u (Lambda u t (subst v a (Var u))) e
				   | otherwise = Lambda x t (subst v a e)
	  subst v Unit e = Unit
	  subst v (Ap f x) e = Ap (subst v f e) (subst v x e)
	  subst v (Pair x y) e = Pair (subst v x e) (subst v y e)
	  subst v (Fst x) e = Fst (subst v x e)
	  subst v (Snd x) e = Snd (subst v x e)
	  subst v (Const x t) e = Const x t

pweta :: PW -> PWM PW
pweta = pwonce aux
    where aux (Lambda _ _ _) = fail "Cannot eta expand an abstraction!"
	  aux e              = do c <- asks fst
				  p <- asks snd
				  when (not (null p) && applied e (head p)) (fail "Cannot eta expand an applied term!")
				  case pw2tp c e of Just (F a _) -> do let x = succ (maximum (freeVars e))
								       return (Lambda x a (Ap e (Var x)))
						    otherwise    -> fail "Eta expansion not applicable!"
	  applied x (Ap y _) | x==y = True
	  applied _ _               = False

pwpair :: PW -> PWM PW
pwpair = pwonce aux
    where aux (Fst (Pair x y)) = return x
	  aux (Snd (Pair x y)) = return y
	  aux _                = fail "Pair cancelation not applicable!"

pwsp :: PW -> PWM PW
pwsp = pwonce aux
    where aux (Pair x y) = fail "Cannot pair expand a pair!"
	  aux e          = do c <- asks fst
			      p <- asks snd
			      when (not (null p) && projected (head p)) (fail "Cannot pair expand a projected term!")
			      case pw2tp c e of Just (P _ _) -> return (Pair (Fst e) (Snd e))
						otherwise    -> fail "Pair expansion not applicable!"
	  projected (Fst _) = True
	  projected (Snd _) = True
	  projected _       = False
						       
pwunit :: PW -> PWM PW
pwunit = pwonce aux
    where aux Unit = fail "Cannot unit expand unit!"
	  aux e    = do c <- asks fst
			case pw2tp c e of Just U    -> return Unit
					  otherwise -> fail "Unit expansion not applicable!"

pw2pw :: PW -> PW
pw2pw e = fromJust (runReaderT (many (pwbeta `plus` pwpair `plus` pweta `plus` pwsp `plus` pwunit) e) ([],[]))
    where plus f g x = (f x) `mplus` (g x)
	  many f x = (f x >>= (many f)) `mplus` (return x) 

-- Translation from pw to pf
-- I do not know if the translation of the constants is correct

pw2pf :: Context -> PW -> PF
pw2pf l (Var v)        = let (Just i) = elemIndex v (map fst l)
			     aux = foldr (\x y -> P y x) U
			     tps = reverse (take (i+1) (map aux (tails (map snd l))))
       			     path = zipWith ($) (SND : (replicate i FST)) tps
			     in foldr1 Comp path
pw2pf l (Lambda v t e) = Curry (pw2pf ((v,t):l) e)
pw2pf l Unit           = NIL (foldr (\x y -> P y x) U (map snd l)) 
pw2pf l (Ap f e)       = Comp (AP (fromJust (pw2tp l f))) (Split (pw2pf l f) (pw2pf l e))
pw2pf l (Pair f e)     = Split (pw2pf l f) (pw2pf l e)
pw2pf l (Fst e)        = Comp (FST (fromJust (pw2tp l e))) (pw2pf l e)
pw2pf l (Snd e)        = Comp (SND (fromJust (pw2tp l e))) (pw2pf l e)
pw2pf l (Const x t)    = (CONST x (F (foldr (\x y -> P y x) U (map snd l)) t))


-- Since we can obtain A -> B from 1 -> (A -> B) we can translate functions mor effectively. The correctness of this translation can be proved unsig the "quote rules" of Curien.
-- The same cannot be done in general, since there is no way to recover A from 1 -> A in pf because it can only express morphisms

lambda2pf :: PW -> PF
lambda2pf f = case (pw2tp [] f) 
	      of Just (F a b) -> Comp (AP (F a b)) (Split (Comp (pw2pf [] f) (NIL a)) (ID a))

-- Translation from pf to pw
				 
pf2pw :: PF -> PW
pf2pw (ID t)            = Lambda 1 t (Var 1)
pf2pw (Comp f g)        = let (F a b) = fromJust (pf2tp g)
			      in Lambda 1 a (Ap (pf2pw f) (Ap (pf2pw g) (Var 1)))
pf2pw (FST t)           = Lambda 1 t (Fst (Var 1))
pf2pw (SND t)           = Lambda 1 t (Snd (Var 1))
pf2pw (Split f g)       = let (F a b) = fromJust (pf2tp f)
			      in Lambda 1 a (Pair (Ap (pf2pw f) (Var 1)) (Ap (pf2pw g) (Var 1)))
pf2pw (AP (F a b))      = Lambda 1 (P (F a b) a) (Ap (Fst (Var 1)) (Snd (Var 1)))
pf2pw (Curry f)         = let (F (P a b) c) = fromJust (pf2tp f)
			      in Lambda 1 a (Lambda 2 b (Ap (pf2pw f) (Pair (Var 1) (Var 2))))
pf2pw (NIL t)           = Lambda 1 t Unit
pf2pw (CONST x (F a b)) = Lambda 1 a (Const x b)

-- Examples

pf_iso_swap1 = Comp (Split (SND (P (B 2) (B 1))) (FST (P (B 2) (B 1)))) (Split (SND (P (B 1) (B 2))) (FST (P (B 1) (B 2))))
pf_iso_swap2 = ID (P (B 2) (B 1))

-- Example of a term that breaks confluence with contractions

ex1 = Lambda 1 (P U (B 1)) (Pair (Fst (Var 1)) (Snd (Var 1)))

-- By extensionality of unit we get

ex1_cp1 = Lambda 1 (P U (B 1)) (Pair Unit (Snd (Var 1)))

-- And by surjective pairing

ex1_cp2 = Lambda 1 (P U (B 1)) (Var 1)

-- Another example

ex2 = Lambda 1 (F (B 1) U) (Lambda 2 (B 1) (Ap (Var 1) (Var 2)))

ex2_cp1 = Lambda 1 (F (B 1) U) (Lambda 2 (B 1) Unit)

ex2_cp2 = Lambda 1 (F (B 1) U) (Var 1)




ex3 = Lambda 1 (P (B 1) (B 2)) (Fst (Ap (Lambda 1 (P (B 1) (B 2)) (Var 1)) (Var 1)))

ex4 = Lambda 1 (P (B 1) (B 2)) (Fst (Var 1))

ex5 = Lambda 1 (F (F (B 1) (B 2)) (B 3)) (Lambda 2 (F (B 1) (B 2)) (Ap (Var 1) (Var 2)))
