{-# OPTIONS -fglasgow-exts #-}

module Pointless.Syntax.Pointfree where

import Pointless.Syntax.Types
import Data.Generics hiding ((:*:))
import Control.Monad
import Data.Maybe

data Term = ID Type
	  | Term :.: Term
	  | FST Type
	  | SND Type
	  | Term :/\: Term
	  | AP Type
	  | Curry Term
	  | NIL Type
	    deriving (Eq, Typeable, Data)

instance Show Term
    where show f = (aux f) -- ++ " :: "++(show (fromJust (infer f)))
	      where aux (ID _)       = "id"
		    aux (f :.: g)  = (aux f) ++ " . " ++ (aux g)
		    aux (FST _)     = "fst" 
		    aux (SND _)     = "snd"
		    aux (f :/\: g) = "("++(aux f)++" /\\ "++(aux g)++")"
		    aux (AP _)      = "app"
		    aux (Curry f)   = "curry ("++(aux f)++")"
		    aux (NIL _)     = "nil"

{-
-- The type annotations disappear because we don't have scoped type variables in the syntax
instance Lift PF
    where lift (ID _)      = varE "id"
	  lift (Comp f g)  = infixApp (lift f) (varE ".") (lift g)
	  lift (FST _)     = varE "fst"
	  lift (SND _)     = varE "snd"
	  lift (Split f g) = infixApp (lift f) (varE "/\\") (lift g)
	  lift (AP _)      = varE "app"
	  lift (Curry f)   = appE (varE "curry") (lift f)
	  lift (NIL _)     = varE "nil"
	  lift (CONST v _) = varE ("("++v++"!)")
-}

infer :: Term -> Maybe Type
infer (ID t)          = return (t :-> t)
infer (f :.: g)       = do (a :-> b) <- infer f
			   (c :-> d) <- infer g
			   guard (a == d)
			   return (c :-> b)
infer (FST (a :*: b)) = return ((a :*: b) :-> a)
infer (SND (a :*: b)) = return ((a :*: b) :-> b)
infer (f :/\: g)      = do (a :-> b) <- infer f
			   (c :-> d) <- infer g
			   guard (a == c)
			   return (a :-> (b :*: d))
infer (AP (a :-> b))  = return (((a :-> b) :*: a) :-> b)
infer (Curry f)       = do ((a :*: b) :-> c) <- infer f
			   return (a :-> (b :-> c))
infer (NIL t)         = return (t :-> One)



swapiso = swap1 :.: swap2
    where swap2 = SND (Base 1 :*: Base 2) :/\: FST (Base 1 :*: Base 2)
	  swap1 = SND (Base 2 :*: Base 1) :/\: FST (Base 2 :*: Base 1)

idpair = ID (Base 1 :*: Base 2)

idfunc = ID (Base 1 :-> Base 2)