{-# OPTIONS -fglasgow-exts #-}

module Pointless.Syntax.Pointwise where

import Pointless.Syntax.Types
import Data.Generics hiding (Unit,(:*:))
import Data.Maybe
import Data.List
import Data.FiniteMap
import Control.Monad.Reader

data Term = Var Name
	  | Unit
	  | Term :&: Term
	  | Fst Term
	  | Snd Term
	  | Lam Name Type Term
	  | Term :@: Term
	    deriving (Eq, Typeable, Data)


instance Show Term
    where show e = (aux e)++" :: "++(show (fromJust (runReaderT (infer e) emptyFM)))
	      where aux (Var v)        = "v"++(show v)
		    aux (Lam v _ e) = "(\\v"++(show v)++" -> "++(aux e)++")"
		    aux Unit           = "()"
		    aux (f :@: e)       = (aux f)++" ("++(aux e)++")"
		    aux (l :&: r)     = "("++(aux l)++" , "++(aux r)++")"
		    aux (Fst e)        = "fst ("++(aux e)++")"
		    aux (Snd e)        = "snd ("++(aux e)++")"


{-
instance Lift PW
    where lift (Var v)        = varE ("v"++(show v))
	  lift (Lambda v _ e) = lam1E (varP ("v"++(show v))) (lift e)
	  lift Unit           = conE "()"
	  lift (Ap f e)       = appE (lift f) (lift e)
	  lift (Pair l r)     = tupE [lift l, lift r]
	  lift (Fst e)        = appE (varE "fst") (lift e)
	  lift (Snd e)        = appE (varE "snd") (lift e)
	  lift (Const x _)    = conE x
-}
-- Typing

type Env = FiniteMap Name Type

infer :: Term -> ReaderT Env Maybe Type
infer (Var v)     = do env <- ask
		       guard (v `elemFM` env)
		       return (fromJust (lookupFM env v))
infer Unit        = return One
infer (x :&: y)   = do a <- infer x
		       b <- infer y
		       return (a :*: b)
infer (Fst x)     = do (a :*: b) <- infer x
		       return a
infer (Snd x)     = do (a :*: b) <- infer x
		       return b
infer (Lam v a x) = do b <- local (\e -> addToFM e v a) (infer x)
		       return (a :-> b)
infer (f :@: e)   = do (a :-> b) <- infer f
		       c <- infer e
		       guard (a == c)
		       return b

free :: Term -> [Name]
free (Var v)     = [v]
free (Lam v _ x) = delete v (free x)
free e           = nub (concat (gmapQ (mkQ [] free) e))


type Subst = FiniteMap Name Term

replace :: Term -> Reader Subst Term
replace (Var v) = 
    do subst <- ask
       return (lookupWithDefaultFM subst (Var v) v)
replace (Lam v t x) = 
    do subst <- ask
       let new = succ (maximum (concat (map free (x:eltsFM subst))))
       y <- local (\s -> addToFM s v (Var new)) (replace x)
       return (Lam new t y)
replace e = gmapM (mkM replace) e



ex1     = Lam 1 (One :*: (Base 1)) (Fst (Var 1) :&: Snd (Var 1))
ex1_cp1 = Lam 1 (One :*: (Base 1)) (Unit :&: Snd (Var 1))
ex1_cp2 = Lam 1 (One :*: (Base 1)) (Var 1)

ex2     = Lam 1 (Base 1 :-> One) (Lam 2 (Base 1) (Var 1 :@: Var 2))
ex2_cp1 = Lam 1 (Base 1 :-> One) (Lam 2 (Base 1) Unit)
ex2_cp2 = Lam 1 (Base 1 :-> One) (Var 1)

swapiso = Lam 1 (Base 1 :*: Base 2) (swap2 :@: (swap1 :@: (Var 1)))
    where swap1 = Lam 1 (Base 1 :*: Base 2) (Snd (Var 1) :&: Fst (Var 1))
	  swap2 = Lam 1 (Base 2 :*: Base 1) (Snd (Var 1) :&: Fst (Var 1))

nswaps i = Lam 1 (Base 1 :*: Base 2) (foldr (\a b -> a b) (Var 1) aux)
    where aux = replicate i (\x -> swap2 :@: (swap1 :@: x)) 
	  swap1 = Lam 1 (Base 1 :*: Base 2) (Snd (Var 1) :&: Fst (Var 1))
	  swap2 = Lam 1 (Base 2 :*: Base 1) (Snd (Var 1) :&: Fst (Var 1))

idpair = Lam 1 (Base 1 :*: Base 2) (Var 1)

