
module PrettyPrint.XML.Dot where

import PrettyPrint.XML.LrcPrelude
import List

--
--      Abstract Syntax
--
--
data Document
	= C_CDocument_1 !Prolog !Element !Miscs 
	deriving (Show , Eq , Ord)

data Prolog
	= C_CProlog_1 !XMLDecl !Miscs 
	deriving (Show , Eq , Ord)

data XMLDecl
	= C_CXmlDecl_1 !VersionInfo !EncodingDecl !SDDecl 
	| C_NoXmlDecl_1 
	deriving (Show , Eq , Ord)

data VersionInfo
	= C_CVerInfo_1 !Name 
	| C_NoVerInfo_1 
	deriving (Show , Eq , Ord)

data Name
	= C_Ident_1 !STR 
	deriving (Show , Eq , Ord)

data EncodingDecl
	= C_CEncodingDecl_1 !STR 
	| C_NoEncodingDecl_1 
	deriving (Show , Eq , Ord)

data SDDecl
	= C_CSDDecl_1 !BOOL 
	| C_NoSDDecl_1 
	deriving (Show , Eq , Ord)

data Miscs
	= C_LstMisc_1 !Misc !Miscs 
	| C_NoMisc_1 
	deriving (Show , Eq , Ord)

data Misc
	= C_CPI_1 !STR 
	| C_Comment_1 !Name 
	deriving (Show , Eq , Ord)

data Element
	= C_CElement_1 !Name !LstAttribute !LstContent !Name 
	| C_CElementError_1 !Name !LstAttribute !LstContent !Name 
	| C_CEmptyElem_1 !Name !LstAttribute 
	| C_CSelectedElem_1 !Name !LstAttribute !LstContent !Name 
	| C_NoElement_1 
	deriving (Show , Eq , Ord)

data LstAttribute
	= C_CLstAttribute_1 !Attribute !LstAttribute 
	| C_NoLstAttribute_1 
	deriving (Show , Eq , Ord)

data Attribute
	= C_CAttribute_1 !Name !AttValue 
	deriving (Show , Eq , Ord)

data AttValue
	= C_CAttValueRef_1 !Reference 
	| C_CAttValueString_1 !STR 
	deriving (Show , Eq , Ord)

data Reference
	= C_CRefChar_1 !STR 
	| C_CRefEntity_1 !STR 
	deriving (Show , Eq , Ord)

type LstContent = [Content]

data Content
	= C_CElem_1 !Element 
	| C_CMisc_1 !Misc 
	| C_CRef_1 !Reference 
	| C_CString_1 !BOOL !CharData 
	deriving (Show , Eq , Ord)

data CharData
	= C_CCharData_1 !STR 
	deriving (Show , Eq , Ord)


--
--      Type of Attributes
--
--
data AttDef
	= C_CAttDef_1 !Name !AttType !DefaultDecl 
	deriving (Show , Eq , Ord)

data AttListDecl
	= C_CAttListDecl_1 !Name !LstAttDef 
	deriving (Show , Eq , Ord)

data AttType
	= C_CEnumeratedType_1 !EnumeratedType 
	| C_CStringType_1 
	| C_CTokenizedType_1 !TokenizedType 
	deriving (Show , Eq , Ord)

data Attr
	= C_CAttr_1 !IdA !IdA 
	deriving (Eq , Ord)

instance Show Attr where
	show (C_CAttr_1 (C_CId_1 "label") y) =  "label = \"" ++ (show y) ++ "\""
	show (C_CAttr_1 x y) = (show x) ++ "=" ++ (show y) 


data AttrStmt
	= C_CEdge_1 !Attrs 
	| C_CGraph_1 !Attrs 
	| C_CNode_1 !Attrs 
	deriving (Show , Eq , Ord)

type Attrs = [Attr]

data CDSect
	= C_CCDSetc_1 !CharData 
	deriving (Show , Eq , Ord)

data CP
	= C_CChoice_1 !LstCP !Modifier 
	| C_CSeq_1 !LstCP !Modifier 
	| C_CTagName_1 !Name !Modifier 
	deriving (Show , Eq , Ord)

data ConditionalSect
	= C_CIgnoreSect_1 !IgnoreSect 
	| C_CIncludeSect_1 !IncludeSect 
	deriving (Show , Eq , Ord)

data ContentSpec
	= C_CANY_1 
	| C_CContentSpec_1 !CP 
	| C_CEMPTY_1 
	| C_CMixed_1 !Mixed 
	deriving (Show , Eq , Ord)

data DefaultDecl
	= C_CDefault_1 !AttValue !FIXED 
	| C_CIMPLIED_1 
	| C_CREQUIRED_1 
	deriving (Show , Eq , Ord)

data DocTypeDecl
	= C_CDTD_1 !Name !ExternalID !LstMarkupDecl 
	deriving (Show , Eq , Ord)

data Dot
	= C_CDot_1 !DotBody 
	deriving (Eq , Ord)

instance Show Dot where
	show (C_CDot_1 d) = show d


data DotBody
	= C_CBodyDiGraph_1 !IdA !Stmts 
	| C_CBodyGraph_1 !IdA !Stmts 
	deriving (Eq , Ord)

instance Show DotBody where
	show (C_CBodyDiGraph_1 n sts) = "digraph " ++ (show n) ++ "{\n" ++ (concat $ map show sts) ++ "}"
	show (C_CBodyGraph_1 n sts)   = "graph "   ++ (show n) ++ "{\n" ++ (concat $ map show sts) ++ "}"

data EV
	= C_CEVRef_1 !Reference 
	| C_CEVString_1 !STR 
	deriving (Show , Eq , Ord)

data EdgeOp
	= C_CEdgeOp_1 !STR 
	deriving (Eq , Ord)

instance Show EdgeOp where
	show (C_CEdgeOp_1 s) = " "++ s ++" "

type EdgeRHS = [RHSExp]

data EdgeStmt
	= C_CEdgeStmtNode_1 !NodeId !EdgeRHS !Attrs 
	| C_CEdgeStmtSubG_1 !SubG !EdgeRHS !Attrs 
	deriving (Eq , Ord)


instance Show EdgeStmt where
	show (C_CEdgeStmtNode_1 nd ed attrs) = let attrs' = concat $ ((intersperse [',']).(map show)) attrs
						   ed'    = concat $ map show ed
					in (show nd) ++ " " ++ ed' ++ " [" ++ attrs' ++ "];\n"

data EitherExtPubID
	= C_CEitherExternID_1 !ExternalID 
	| C_CEitherPublicID_1 !PublicID 
	deriving (Show , Eq , Ord)

data ElemTag
	= C_CElemTag_1 !Name !LstAttribute 
	deriving (Show , Eq , Ord)

data ElementDecl
	= C_CElementDecl_1 !Name !ContentSpec 
	deriving (Show , Eq , Ord)

data EntityDecl
	= C_CEntityGEDecl_1 !GEDecl 
	| C_CEntityPEDecl_1 !PEDecl 
	deriving (Show , Eq , Ord)

data EntityDef
	= C_CDefEntityValue_1 !EntityValue 
	| C_CDefExternalID_1 !ExternalID !NDataDecl 
	deriving (Show , Eq , Ord)

data EntityRef
	= C_CEntityRef_1 !Name 
	deriving (Show , Eq , Ord)

data EntityValue
	= C_CEntityValue_1 !LstEV 
	deriving (Show , Eq , Ord)

data EnumeratedType
	= C_CEnumeration_1 !Enumeration 
	| C_CNotationType_1 !NotationType 
	deriving (Show , Eq , Ord)

data Enumeration
	= C_CEnumeration2_1 !NmTokens 
	deriving (Show , Eq , Ord)

data ExtPE
	= C_CExtPE_1 !TextDecl !ExtSubsetDecl 
	deriving (Show , Eq , Ord)

data ExtParsedEnt
	= C_CExtParsedEnt_1 !TextDecl !Content 
	deriving (Show , Eq , Ord)

data ExtSubsetDecl
	= C_CExtConditionalSect_1 !ConditionalSect 
	| C_CExtMarkupDecl_1 !MarkupDecl 
	deriving (Show , Eq , Ord)

data ExternalID
	= C_CPUBLIC_1 !PubidLiteral !SystemLiteral 
	| C_CSYSTEM_1 !SystemLiteral 
	deriving (Show , Eq , Ord)

data FIXED
	= C_CFIXED_1 
	| C_NoFixed_1 
	deriving (Show , Eq , Ord)

data GEDecl
	= C_CGEDecl_1 !Name !EntityDef 
	deriving (Show , Eq , Ord)

data IdA
	= C_CId_1 !STR 
	deriving (Eq , Ord)

instance Show IdA where
	show (C_CId_1 x) = x

data Ignore
	= C_CIgnore_1 !STR 
	deriving (Show , Eq , Ord)

data IgnoreSect
	= C_CIgnoreSect2_1 !LstIgnoreSectContents 
	deriving (Show , Eq , Ord)

data IgnoreSectContents
	= C_CIgnoreSectContents_1 !Ignore !LstParIgnore 
	deriving (Show , Eq , Ord)

data IncludeSect
	= C_CIncludeSect2_1 !ExtSubsetDecl 
	deriving (Show , Eq , Ord)

data LstAttDef
	= C_CLstAttDef_1 !AttDef !LstAttDef 
	| C_NoLstAttDef_1 
	deriving (Show , Eq , Ord)

data LstCP
	= C_CLstCP_1 !CP !LstCP 
	| C_NoLstCP_1 
	deriving (Show , Eq , Ord)

data LstEV
	= C_CLstEV_1 !EV !LstEV 
	| C_NoEV_1 
	deriving (Show , Eq , Ord)

data LstExtSubsetDecl
	= C_CLstExtSubsetDecl_1 !ExtSubsetDecl !LstExtSubsetDecl 
	| C_NoLstExtSubsetDecl_1 
	deriving (Show , Eq , Ord)

data LstIgnoreSectContents
	= C_CIgnSectCont_1 !IgnoreSectContents !LstIgnoreSectContents 
	| C_NoIgnSectCont_1 
	deriving (Show , Eq , Ord)

data LstMarkupDecl
	= C_CLstMarkupDecl_1 !MarkupDecl !LstMarkupDecl 
	| C_NoLstMarkupDecl_1 
	deriving (Show , Eq , Ord)

data LstParIgnore
	= C_CLstParIgnore_1 !ParIgnore !LstParIgnore 
	| C_NoParIgnore_1 
	deriving (Show , Eq , Ord)

data MarkupDecl
	= C_CAttList_1 !AttListDecl 
	| C_CEntity_1 !EntityDecl 
	| C_CMarkupElem_1 !ElementDecl 
	| C_CMarkupMisc_1 !Misc 
	| C_CNotation_1 !NotationDecl 
	deriving (Show , Eq , Ord)

data Mixed
	= C_CPCDATA_1 
	| C_CPCDATAplus_1 !Names 
	deriving (Show , Eq , Ord)

data Modifier
	= C_CNone_1 
	| C_Plus_1 
	| C_Query_1 
	| C_Star_1 
	deriving (Show , Eq , Ord)

data NDataDecl
	= C_NCDATA_1 !Name 
	| C_NoDATA_1 
	deriving (Show , Eq , Ord)

data Names
	= C_CNames_1 !Name !Name 
	| C_NoName_1 
	deriving (Show , Eq , Ord)

data NmToken
	= C_CNmToken_1 !STR 
	deriving (Show , Eq , Ord)

data NmTokens
	= C_LNmTokens_1 !NmToken !NmTokens 
	| C_NoNmTokens_1 
	deriving (Show , Eq , Ord)

data NodeId
	= C_CNodeId_1 !IdA 
	| C_CNodeIdPort_1 !IdA !IdA 
	deriving (Eq , Ord)

instance Show NodeId where
	show (C_CNodeId_1 s)         = show s
	show (C_CNodeIdPort_1 s1 s2) = "\"" ++ show s1 ++ "\":" ++ show s2

data NodeStmt
	= C_CNodeStmt_1 !NodeId !Attrs 
	deriving (Eq , Ord)

instance Show NodeStmt where
	show (C_CNodeStmt_1 nd attrs) = let attrs' = concat $ ((intersperse [',']).(map show)) attrs
					in (show nd) ++ " [" ++ attrs' ++ "];\n"

data NotationDecl
	= C_CNOTATION_1 !Name !EitherExtPubID 
	deriving (Show , Eq , Ord)

data NotationType
	= C_CNotationType2_1 !Names 
	deriving (Show , Eq , Ord)

data PEDecl
	= C_CPEDecl_1 !Name !PEDef 
	deriving (Show , Eq , Ord)

data PEDef
	= C_CPEDefEntityValue_1 !EntityValue 
	| C_CPEDefExternalID_1 !ExternalID 
	deriving (Show , Eq , Ord)

data PEReference
	= C_CPEReference_1 !Name 
	deriving (Show , Eq , Ord)

data PITarget
	= C_CPITarget_1 !Name 
	deriving (Show , Eq , Ord)

data ParIgnore
	= C_CParIgnore_1 !IgnoreSectContents !Ignore 
	deriving (Show , Eq , Ord)

data ProcInstr
	= C_CProcInstr_1 !PITarget !STR 
	deriving (Show , Eq , Ord)

data PubidLiteral
	= C_CPubidLiteral_1 !STR 
	deriving (Show , Eq , Ord)

data PublicID
	= C_CPUBLICID_1 !PubidLiteral 
	deriving (Show , Eq , Ord)

data RHSExp
	= C_CRHSExpNode_1 !EdgeOp !NodeId 
	| C_CRHSExpSubG_1 !EdgeOp !SubG 
	deriving (Eq , Ord)

instance Show RHSExp where
	show (C_CRHSExpNode_1 ed nd) = show ed ++ " \"" ++ show nd ++ "\""
	show (C_CRHSExpSubG_1 ed nd) = show ed ++ " \"" ++ show nd ++ "\""

data Stmt
	= C_CStmtAttr_1 !AttrStmt 
	| C_CStmtEdge_1 !EdgeStmt 
	| C_CStmtEq_1 !Attr 
	| C_CStmtNode_1 !NodeStmt 
	| C_CStmtSubG_1 !SubG 
	deriving (Eq , Ord)

instance Show Stmt where
	show (C_CStmtNode_1 nd) = show nd
	show (C_CStmtEq_1 attr) = show attr
	show (C_CStmtEdge_1 ed) = show ed

type Stmts = [Stmt]

data SubG
	= C_CSubG_1 !IdA !Stmts 
	| C_CSubGId_1 !IdA 
	| C_CSubGOmmited_1 !Stmts 
	deriving (Eq , Ord)

instance Show SubG where
	show (C_CSubGId_1 id) = show id


data SystemLiteral
	= C_CSystemLiteral_1 !STR 
	deriving (Show , Eq , Ord)

data TextDecl
	= C_CTextDecl_1 !VersionInfo !EncodingDecl 
	| C_NoTextDecl_1 
	deriving (Show , Eq , Ord)

data TokenizedType
	= C_CENTITIES_1 
	| C_CENTITY_1 
	| C_CID_1 
	| C_CIDREF_1 
	| C_CIDREFS_1 
	| C_CNMTOKEN_1 
	| C_CNMTOKENS_1 
	deriving (Show , Eq , Ord)

