-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- Online GUI for ChopaChops.
--
-----------------------------------------------------------------------------

module Main where

import CGI hiding (div, head, map, span)
import System
import CGIGraphics
import Language.Java.ChaseImports
import Language.Java.PackageGraph
import Data.List (isSuffixOf)
import Data.Set
import Data.Relation.SetOfPairs

import Control.ModelViewController
import Control.WashExtras

-----------------------------------------------------------------------------
-- * Main program.

main :: IO ()
main = run chopaChopsCGI 

chopaChopsCGI :: CGI ()
chopaChopsCGI = mvc chopaChopsController

-----------------------------------------------------------------------------
-- * Model View Controller

-- | Controller
chopaChopsController :: Controller PackageGraph
chopaChopsController = Controller {
  initModel = \fileRef -> do
    maybeCU <- parseJavaFile (fileReferenceName fileRef)
    java2pcg' id "" maybeCU
,      
  modelOperations = [
    ("Chop",chopPackageGraph)
  ]
,
  modelViews = [
    ("Package Graph (gif)",packageGraphView "gif"),
    ("Package Graph (jpg)",packageGraphView "jpg") --,
    -- ("Package Graph",packageGraphView')
  ]
}

{-
packageGraphView' = MV "Format" ["gif","jpg"] (\pg format -> do
  let dotString = pg2dot "pg" pg
  imageFileName <- ioHTML $ callDot dotString format "PackageGraph"
  let image = presentImage imageFileName
  <table><tr><td><% image %></td></tr></table>)
-}

packageGraphView format = MV (\pg -> do
  let dotString = pg2dot "pg" pg
  imageFileName <- ioHTML $ callDot dotString format "PackageGraph"
  let image = presentImage imageFileName
  <table><tr><td><% image %></td></tr></table>)

chopPackageGraph = MO ("Sources","Sinks") 
  (\(r1,r2,r3,r4) -> 
    let nodes 
         = setToList $ (ent r1)`union`(ent r2)`union`(ent r3)`union`(ent r4)
    in (nodes,nodes) 
  ) 
  (\pg (sources,sinks) -> 
    let pgChop = sliceOrChopWith intersect sources sinks (pgAsRel pg)
    in projectPG' pgChop pg
  )
  

-----------------------------------------------------------------------------

