

module Main where


import GetOpt
import System
import IO
import List --(partition)

import PrettyPrint.XML.Dot
import PrettyPrint.XML.Xml2Dot

import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.LR
import Language.ContextFree.LR_Semantics
import Language.ContextFree.G_SLR
import Language.ContextFree.G_LR
import Language.ContextFree.G_LR_Semantics
import Language.ContextFree.MemoInc_G_LR
import Language.ContextFree.G_LR_Inc_Semantics
import Language.ContextFree.Semantics

import GLRActionsTable
import LRActionsTable



import Language.ContextFree.Examples
--import Grammar --grammar under consideration

import qualified Text.XML.HaXml.Types

options     :: [OptDescr String]
options     =  [ Option ['N','n']
                 ["NDFA"]
                 (NoArg "N")
                 "Non-Deterministic LR_0 Automaton (graphviz)"
               , Option ['D','d']
                 ["DFA"]
                 (NoArg "D")
                 "Deterministic LR_0 Automaton (graphviz)"
               , Option ['T','t']
                 ["Action Table"]
                 (NoArg "T")
                 "SLR_1 Action Table"
               , Option ['G','g']
                 ["GLR"]
                 (NoArg "G")
                 "Use Generalised LR Parsing"
               , Option ['C','c']
                 ["Conflicts"]
                 (NoArg "C")
                 "Conflicts (SLR_1)"
               , Option ['M','m']
                 ["Memoisation"]
                 (NoArg "M")
                 "Incremental Parsing"
               , Option ['S','s']
                 ["Strict"]
                 (NoArg "S")
                 "\"Strict\" Parsing"
               , Option ['L','l']
                 ["Lazy"]
                 (NoArg "L")
                 "Lazy Parsing"
	       , Option ['X','x']
		 ["Xml Semantics"]
		 (NoArg "X")
		 "Xml Tree building and visualization"
	       , Option ['A','a']
		 ["ATerm Semantics"]
		 (NoArg "A")
		 "Aterm Tree building and its nodes counting"
               , Option ['I','i']
                 ["input"]
                 (ReqArg ('i':) "string")
                 "specify input sentence"
               , Option ['o']
                 ["output"]
                 (ReqArg ('o':) "file")
                 "specify output file"
               , Option ['h','?']
                 ["help"]
                 (NoArg "h")
                 "output a brief help message"
               ]

main :: IO ()
main = 
  do args <- getArgs
     let (o,n,errs)  = getOpt Permute options args
     let (inp,opts')  = partition ((=='i') . head) o
     let (fo,opts'') = partition ((=='o') . head) opts'
     let opts = opts''
     if (errs /= []) || ("h" `elem` opts)
        then putStr $ usageInfo usageheader options
        else if (inp == [] && n == [])
                then acceptFromStdIn fo opts
                else acceptSentence (tail $ head inp) fo opts 
  where
    usageheader = "HaGlr: an Incremental and Lazy Haskell-based\n" ++
                  "       Generalised LR Parser \n\n" ++
          "version 0.0 (by Joo Saraiva & Joo Fernandes)\n\n" ++
                  "Usage: HaGlr options [file] ...\n\nList of options:"


acceptSentence inp fileo opts
  = do let fw = if (fileo == []) 
                then putStrLn
                else writeFile (tail (head fileo))

       if ("N" `elem` opts) || ("D" `elem` opts) || ("T" `elem` opts) 
          then pretty_print_gram opts fw
          else if ("G" `elem` opts) then useGLRparsing inp opts fw 
                  else useLRparsing inp opts fw

pretty_print_gram opts fw = 
  do if ("N" `elem` opts) then fw (saveLR_0Ndfa g)
        else if ("D" `elem` opts) then fw (saveLR_0Dfa g)
                else if ("T" `elem` opts) && ("G" `elem` opts) then fw (gslrPPTable g)
                        else fw (slrPPTable g)


useGLRparsing inp opts fw = 
  do if ("L" `elem` opts) then  if ("M" `elem` opts) 
					then if ("X" `elem` opts) 
						then {-G L M X-} let res = glr_parser_inc_xml inp
								 in if ((or.(map fst)) res)
									then ((fw.show) True 
									      >> aux "lazyM" 0 (map (element2dot.snd) [head (filter (\(x,y) -> x==True) res)])
									      >>= aux' "lazyM"
									      >>= exitWith)
									else ((fw.show) False)
						else if ("A" `elem` opts) 
							then {-G L M A-} let res = glr_parser_inc_aterm inp
									 in if ((or.(map fst)) res)
										then ((fw.show) True
										      >> writeFile "./tmp/AlazyM" 
					(show ((count_Nodes.snd) (head ((filter (\(x,y) -> x == True) res))))))
										else ((fw.show) False)
							else {-G L M NoSemantics-} (fw.show.or) (glr_parser_inc inp)
                                      	else if ("X" `elem` opts) 
						then {-G L X-} let res = glr_parser_xml inp
								 in if ((or.(map fst)) res)
									then ((fw.show) True 
									      >> aux "lazy" 0 (map (element2dot.snd) [head (filter (\(x,y) -> x==True) res)])
									      >>= aux' "lazy"
									      >>= exitWith)
									else ((fw.show) False)

						else if ("A" `elem` opts) 
							then {-G L A-} let res = glr_parser_aterm inp
									 in if ((or.(map fst)) res)
										then ((fw.show) True
										      >> writeFile "./tmp/Alazy" 
					(show ((count_Nodes.snd) (head ((filter (\(x,y) -> x == True) res))))))
										else ((fw.show) False)

							else {-G L NoSemantics-} fw (show.or $ glr_parser inp)
        		  else if ("M" `elem` opts) then if ("X" `elem` opts) 
						then {-G S M X-} let res = glr_parser_inc_xml inp
								 in if ((or.(map fst)) res)
									then ((fw.show.length) res  
									      >> aux "strictM" 0 (map (element2dot.snd) (filter (\(x,y) -> x==True) res))
									      >>= aux' "strictM"
									      >>= exitWith)
									else ((fw.show.length) res)

						else if ("A" `elem` opts) 
							then {-G S M A-} let res = glr_parser_inc_aterm inp
									 in if ((or.(map fst)) res)
										then ((fw.show) True
										      >> writeFile "./tmp/AStrictM" 
					((concat.(intersperse "\n")) (map (show.count_Nodes.snd) ((filter (\(x,y) -> x == True) res)))))
										else ((fw.show) False)

							else {-G S M NoSemantics-} let res = glr_parser_inc inp 
										    in fw $ (show.or $ res) ++ "\n" ++ (show.length $ res)
                                      	else if ("X" `elem` opts) 
						then {-G S X-} let res = glr_parser_xml inp
								 in if ((or.(map fst)) res)
									then ((fw.show.length) res  
									      >> aux "strict" 0 (map (element2dot.snd) (filter (\(x,y) -> x==True) res))
									      >>= aux' "strict"
									      >>= exitWith)
									else ((fw.show.length) res)

						else if ("A" `elem` opts) 
							then {-G S A-} let res = glr_parser_aterm inp
									 in if ((or.(map fst)) res)
										then ((fw.show) True
										      >> writeFile "./tmp/AStrict" 
					((concat.(intersperse "\n")) (map (show.count_Nodes.snd) ((filter (\(x,y) -> x == True) res)))))
										else ((fw.show) False)

							else {-G S NoSemantics-} let res = glr_parser inp
										   in fw $ ((show.or) res)++"\n"++((show.length) res)

useLRparsing inp opts fw = 
	do if ("X" `elem` opts) then let (b,tree) = lr_parser_xml inp
			   	in if b then ((fw.show) True >> aux "strict" 0 [element2dot tree]
						       >>= aux' "strict"
						       >>= exitWith)
					else (fw.show) False
			        else if ("A" `elem` opts)  then let (b,tree) = lr_parser_aterm inp
									 in if b
									    then ((fw.show) True >> writeFile "./tmp/AStrict" ((concat.(intersperse "\n")) [(show.count_Nodes) tree]))
									    else ((fw.show) False)
							   else fw (show $ lr_parser inp)  



aux _ i [] = return (i-1)
aux x i (ele:eles) = writeFile ("./tmp/"++x++show i++".dot") (show ele) 
		 >> aux x (i+1) eles

aux' x i = system $ ("dotty " ++ (concat $ intersperse " " $ map ((("./tmp/"++x)++).(++".dot").show) [0..i]) ++ "&")

acceptFromStdIn fo opts = putStrLn "Not supported"
