module Main where

import System
import Data.List
import Data.Char

import Language.HaLex.Dfa
import Language.ContextFree.Examples

import Language.ContextFree.Cfg
import Language.ContextFree.LR_0
import Language.ContextFree.SLR
import Language.ContextFree.G_SLR

import Grammar --grammar under consideration

import GetOpt

import Debug.Trace


options     :: [OptDescr String]
options     =  [ Option ['g','G']
                 ["CFG Grammar"]
                 (ReqArg ('g':) "FilePath")
                 "Parsing from a List of Cfg Productions"
               , Option ['D','d']
                 ["SDF Grammar"]
                 (ReqArg ('d':) "FilePath")
                 "Parsing from a SDF Grammar definition"
	       , Option ['S','s']
		 ["CFG starting symbol"]
	         (ReqArg ('s':) "string")
		 "Specifying Grammar Starting Symbol"
               , Option ['h','?']
                 ["help"]
                 (NoArg "h")
                 "output a brief help message"]

--getGrammar :: [String] -> IO (Either (Cfg Char Char) (Cfg Char Char))
getGrammar ["-g",grammarFileName,"-s",startsymbol] = do
  content <- readFile grammarFileName
  let productions :: [(String,[Symb Char String])] = read (content)
  putStrLn $ "Done Reading Grammar\n\n" ++ show productions
  return $ Left (prods2cfg productions startsymbol)
getGrammar ["-d",sdfGrammarFileName] = do
  cfgGrammar <- normalizeSdf "" sdfGrammarFileName
  content <- readFile cfgGrammar
  let productions :: [(String,[Symb Char String])] = read (content)
  return $ Left (prods2cfg productions "<START>")  
getGrammar _ = do
  putStrLn "Using compiled-in grammar!"
  return $ Right g

normalizeSdf :: FilePath -> FilePath -> IO FilePath
normalizeSdf dir sdfGrammarFileName  
  | ".def" `isSuffixOf` sdfGrammarFileName 
  = normalizeSdf dir (take ((length sdfGrammarFileName) -4) sdfGrammarFileName)
  | otherwise 
  = do 
  let normalizedIn  = sdfGrammarFileName ++ "Normal.def"
  let normalizedOut = prefixDir dir sdfGrammarFileName ++ "_Normal.def"
  let cfgGrammar = prefixDir dir sdfGrammarFileName ++ "_Normal.cfg"
  let header     = "definition \n\n module Main \n\n exports \n\n"
  putStrLn "<<<Normalizing Grammar>>>"
  system $ "parse-sdf2 -i " ++ sdfGrammarFileName ++ " | parsetablegen -n -m Main | unparsePT -o " ++ normalizedIn
  content <- readFile normalizedIn
  writeFile normalizedOut (header ++ content)
  putStrLn "<<<Generating Cfg Grammar>>>"
  system $ "Sdf2Haskell -i " ++ normalizedOut ++ " -o " ++ cfgGrammar  ++" -c"
<<<<<<< tablegenerator.hs
  content <- readFile cfgGrammar
  let productions :: [(String,[Symb Char String])] = read (content)
  return $ Left (prods2cfg productions "<START>")  
getGrammar _ = do
  putStrLn "Using compiled-in grammar!"
  return $ Right g


=======
  return cfgGrammar

prefixDir "" fileName = fileName
prefixDir dir fileName = dir++"/"++fileName
 


>>>>>>> 1.5
main = do arg <- getArgs
	  --genTable "Char" g
	  let (o,n,errs)  = getOpt Permute options arg
	  if (errs /= []) || ("h" `elem` o)
		then putStr $ usageInfo usageheader options
		else do g <- getGrammar arg;
	  		either (genTable "String") (genTable "Char") g
       where usageheader = "Table Generator: a Haskell-based Parser Generator for HaGLR\n" ++
          		   "version 0.0 (by Joo Saraiva, Joo Fernandes & Joost Visser)\n\n" ++
                  	   "Usage: TableGenerator [options] [file] ...\n\nList of options:"

--genTable :: (Eq sy,Show sy,Ord sy) => (Cfg sy) -> IO ()
genTable str g = do
          let expanded@(Cfg t nt r p) = trace "<<expanding cfg grammar>>" $ expand_cfg g
	  let actionT                 = trace "<<computing lr action table>>" $ e_slr_at expanded
	  let dfa@(Dfa v q s z d)     = trace "<<computing LR_0 automata>>" $ ecfg2LR_0Dfa expanded
	  let actionT'                = trace "<<computing glr action table>>" $ e_gslr_at expanded
	  writeFile "./GLRActionsTable.hs" ("module GLRActionsTable where"
					      		++ "\n\n\n"
							++ "import List\n"
					      		++ "import Language.ContextFree.Cfg \n"
					      		++ "import Language.ContextFree.LR_0\n"
							++ "import Language.ContextFree.SLR\n"
							++ "import Language.ContextFree.G_LR\n"
					      		++ "import Language.ContextFree.G_SLR\n"
							++ "import Language.ContextFree.G_LR_Inc_Semantics\n"
							++ "import Language.ContextFree.G_LR_Semantics\n"
							++ "import Language.ContextFree.MemoInc_G_LR\n\n"
					      		++ "g = " ++ show g ++ "\n"
							++ "-- at' :: [[[Action Int [Symb Char Char]]]]\n"
							++ "at' = "
					      		++ "[" 
							++ uncurry show' (foldr (\(x,y) (l1,l2) -> (y:l1,x:l2)) ([],[]) actionT') 
							++ "]"
					      		++ "\n\n-- delta :: Int -> (Symb Char) -> Int"
					      		++ "\n\nlookup_TT = delta\n\n"
					      		++ showDfaDelta' actionT q v d "" 
							++ "\n\nstart_state = "
                                                        ++ show (aux (map fst actionT) s)
					      		++ "\n\nt = "
                                                        ++ show t
                                                        ++ "\n\n"
							++ "lookup_AT :: Int -> Symb Char " ++ str ++ " -> [Action Int [Symb Char " ++ str ++ "]]\n"
                                                        ++ "lookup_AT st sy = (at' !! st) !! col --  ????\n"
                                                        ++ "  where (Just col) = sy `elemIndex` t\n\n"
							++ "glr_parser inp = glr lookup_TT lookup_AT [start_state] inp\n"
							++ "glr_parser_inc_xml inp = glr_inc_sem xml ([],g) lookup_TT lookup_AT [start_state] inp\n" 
							++ "glr_parser_inc_aterm inp = glr_inc_sem aterm ([],g) lookup_TT lookup_AT [start_state] inp\n" 
							++ "glr_parser_xml inp = glr_sem xml ([],g) lookup_TT lookup_AT [start_state] inp\n" 
							++ "glr_parser_aterm inp = glr_sem aterm ([],g) lookup_TT lookup_AT [start_state] inp\n" 
							++ "glr_parser_inc inp = glr_inc lookup_TT lookup_AT [start_state] inp\n" 
							++ "\n\n-- atblIndWSts = False\n\n"
							++ "-- at = []\n"
							++ "-- d st _ = st\n")
	  writeFile "./LRActionsTable.hs" ("module LRActionsTable where"
						        ++ "\n\n\n"
							++ "import List \n"
							++ "import Language.ContextFree.Cfg \n"
							++ "import Language.ContextFree.LR\n"
							++ "import Language.ContextFree.LR_Semantics\n"
					      		++ "import Language.ContextFree.LR_0\n"
							++ "import Language.ContextFree.SLR\n\n"
							++ "g' = " ++ show g ++ "\n"
							++ "-- at' :: [[Action [Symb Char]]]\n"
							++ "at' = ["
							++ uncurry show'' (foldr (\(x,y) (l1,l2) -> (y:l1,x:l2)) ([],[]) actionT)
							++ "]"
							++ "\n\n-- d' :: Int -> (Symb Char) -> Int"
							++ "\n\nlookup_TT = delta\n\n"
							++ showDfaDelta' actionT q v d "" 
							++ "\n"
							++ "-- atblIndWSts = False\n\n"
							++ "-- at = []\n"
							++ "-- d st _ = st\n"
							++ "start_state = "
							++ show (aux (map fst actionT) s)
					      		++ "\n\nt = "
                                                        ++ show t
                                                        ++ "\n\n"
							++ "lookup_AT :: Int -> Symb Char " ++ str ++ " -> Action Int [Symb Char " ++ str ++ "]\n"
							++ "lookup_AT st sy = (at' !! st) !! col --  ????\n"
                                                        ++ "  where (Just col) = sy `elemIndex` t\n\n"
							++ "lr_parser inp = lr lookup_TT lookup_AT [start_state] inp\n"
							++ "lr_parser_xml inp = lr_sem xml ([],g') lookup_TT lookup_AT [start_state] inp\n" 
							++ "lr_parser_aterm inp = lr_sem aterm ([],g') lookup_TT lookup_AT [start_state] inp\n")


			 


show' [] _ = ""
show' [h]   sts = "[" ++ show'' h sts ++ "]"
show' (h:t) sts = "[" ++ (show'' h sts) ++ "]," ++ (show' t sts)

show'' []    _   = ""
show'' [h]   sts = "[" ++ show''' h sts ++ "]"
show'' (h:t) sts = "[" ++ (show''' h sts) ++ "]," ++ (show'' t sts)

show''' []    _   = ""
show''' [h]   sts =  show'''' h sts 
show''' (h:t) sts = (show'''' h sts) ++ "," ++ (show''' t sts)


show'''' (Shift st) sts = "Shift " ++ (show (aux sts st)) 
show'''' x          _   = show x

--showDfaDelta' :: (Show st, Show sy) => [st] -> [sy] -> (st -> sy -> st) -> [Char] -> [Char]
showDfaDelta' at q v d = foldr (.) (showChar '\n') f
  where
    f     = zipWith3 showF m n q'
    (m,n) = unzip l
    q'    = map (uncurry d) l
    l     = [(a,b) | a <- q , b <- v]
    showF st sy st' = showString("delta ") .
                      shows (aux sts st).
                      showString(" (") .
                      shows sy .
                      showString(") = ") .
                      shows (aux sts st') .
                      showChar('\n')
    sts   = map fst at

aux sts st = j
	where (Just j) =  elemIndex st sts


