
{
module Analex (Token(..), AlexPosn(..), alexScanTokens, token_posn) where 

}

%wrapper "posn"

$plainLetter = [a-zA-Z]
$keywordLetter = a-z
$digit = 0-9
$octalDigit = 0-7

$otherChar = [\,\:\;\=\(\)\|\-\[\]\{\}\+\/\\\<\>\.\*\&\^\_\#\~\@\'\`]

$nl = [\n\r]
$white = [\ \t\v]
$separator = [$nl $white]

@escapeSeq = "\\" | "\r" | "\n" | "\t" | "\f" | "\e" | "\a" 

@delimChar = 
	"," | ":" | ";" | "=" | "(" | ")" | "|" | "-" | "[" | "]" | 
	"{" | "}" | "+" | "/" | "<" | ">" | "<=" | ">=" | "<>" | "." |
	"*" | "->" | "+>" | "==>" | "||" | "=>" | "<=>" | "|->" | "<:" | ":>" |
	"<-:" | ":->" | "&" | "==" | "**" | "^" | "++" | "\" | "::" | ":=" |
	"..."

@character 	= $plainLetter 
		-- | $keywordLetter
		| $digit
		-- | $octalDigit
		-- | @delimChar
		| $otherChar
		| $separator

@identifier = $plainLetter ($plainLetter | $digit | "'" | "_")*
@typeVarId = "@" @identifier

@numeral = $digit+
@exponent = ( E | e ) ( "+" | "-" )? @numeral
@numericLit = @numeral ("." ($digit+))? ( @exponent )?

@charLit = \'( @character | @escapeSeq )\'

@stringLit = \"( '""' | @character | @escapeSeq )*\"

@quoteLit = "<"( $plainLetter ( $plainLetter | $digit | "_")* )">"

tokens :- 
				   
	$separator+ ;

	@quoteLit		{ \p s ->  TokenQuoteLit p ( (init.tail) s ) }


	"`"			;
	"," 			{ \p s ->  TokenComma p }
       	":"  			{ \p s ->  TokenColon p }
	":-"			{ \p s ->  TokenColonMinus p }
	";"  			{ \p s ->  TokenSemiColon p }
	"="  			{ \p s ->  TokenEqual p }
	"("  			{ \p s ->  TokenOBracket p }
	")"  			{ \p s ->  TokenCBracket p }
	"|"  			{ \p s ->  TokenVertBar p }
	"-"  			{ \p s ->  TokenMinus p }
	"["  			{ \p s ->  TokenOSqBracket p }
	"]"  			{ \p s ->  TokenCSqBracket p }
	"{"  			{ \p s ->  TokenOCurBracket p }
	"}"  			{ \p s ->  TokenCCurBracket p }
       	"+"  			{ \p s ->  TokenPlus p }
       	"/"  			{ \p s ->  TokenSlash p }
       	"<"  			{ \p s ->  TokenLess p }
       	">"  			{ \p s ->  TokenGreat p }
       	"<="  			{ \p s ->  TokenLessEq p }
       	">="  			{ \p s ->  TokenGreatEq p }
       	"<>"  			{ \p s ->  TokenDiff p }
       	"."  			{ \p s ->  TokenPeriod p }
	"*"  			{ \p s ->  TokenAsterisk p }
       	"->"  			{ \p s ->  TokenPFuncType p }
       	"+>"  			{ \p s ->  TokenTFuncType p }
	"==>"  			{ \p s ->  TokenQQ p } -- Descobrir o seu significado
       	"=>"  			{ \p s ->  TokenImplies p }
       	"<=>"  			{ \p s ->  TokenEquiv p }
       	"|->"  			{ \p s ->  TokenMapAp p }
       	"<:"  			{ \p s ->  TokenDomRestTo p }
       	":>"  			{ \p s ->  TokenRanRestTo p }
	"<-:"  			{ \p s ->  TokenDomRestBy p }
       	":->"  			{ \p s ->  TokenRanRestBy p }
       	"&"  			{ \p s ->  TokenAmpersand p }
       	"=="  			{ \p s ->  TokenDoubleEq p }
       	"**"  			{ \p s ->  TokenDoubleAsterisk p }
       	"^"  			{ \p s ->  TokenHat p }
       	"++" 			{ \p s ->  TokenDoublePlus p }
	"\"			{ \p s ->  TokenBackSlash p }
	"::"			{ \p s ->  TokenDoubleColon p }
	":="			{ \p s ->  TokenAtrib p }
	"..."			{ \p s ->  TokenEllipsis p }
	"~"			{ \p s ->  TokenOldName p }
	".#"			{ \p s ->  TokenTupleSelect p }

	-- Keywords --

	abs			{ \p s ->  TokenAbs p }
	and 			{ \p s ->  TokenAnd p }
	be 			{ \p s ->  TokenBe p }
	bool 			{ \p s ->  TokenBool p }
	card 			{ \p s ->  TokenCard p }
       	cases 			{ \p s ->  TokenCases p }
	char 			{ \p s ->  TokenChar p }
	comp  			{ \p s ->  TokenComp p }
	compose  		{ \p s ->  TokenCompose p }
	conc 			{ \p s ->  TokenConc p }
	def 			{ \p s ->  TokenDef p }
	dinter 			{ \p s ->  TokenDinter p }
	div 			{ \p s ->  TokenDiv p }
       	dom  			{ \p s ->  TokenDom p }
       	dunion  		{ \p s ->  TokenDunion p }
       	elems  			{ \p s ->  TokenElems p }
       	else  			{ \p s ->  TokenElse p }
       	elseif  		{ \p s ->  TokenElseif p }
       	end  			{ \p s ->  TokenEnd p }
       	errs  			{ \p s ->  TokenErrs p }
	exists  		{ \p s ->  TokenExists p }
       	exists1  		{ \p s ->  TokenExists1 p }
       	ext 			{ \p s ->  TokenExt p }
	false   		{ \p s ->  TokenFalse p }
	floor   		{ \p s ->  TokenFloor p }
       	forall  		{ \p s ->  TokenForall p }
       	functions  		{ \p s ->  TokenFunctions p }
	hd  			{ \p s ->  TokenHd p }
       	if  			{ \p s ->  TokenIf p }
	in\ set			{ \p s ->  TokenInSet p }
       	in  			{ \p s ->  TokenIn p }
       	inds  			{ \p s ->  TokenInds p }
	init  			{ \p s ->  TokenInit p }
       	inmap  			{ \p s ->  TokenInmap p }
       	int  			{ \p s ->  TokenInt p }
       	inter  			{ \p s ->  TokenInter p }
	inv  			{ \p s ->  TokenInv p }
       	inverse  		{ \p s ->  TokenInverse p }
       	iota  			{ \p s ->  TokenIota p }
	lambda  		{ \p s ->  TokenLambda p }
       	len  			{ \p s ->  TokenLen p }
       	let  			{ \p s ->  TokenLet p }
       	map  			{ \p s ->  TokenMap p }
       	merge  			{ \p s ->  TokenMerge p }
       	mod  			{ \p s ->  TokenMod p }
	mu  			{ \p s ->  TokenMu p }
       	munion 			{ \p s ->  TokenMunion p }
       	nat  			{ \p s ->  TokenNat p }
       	nat1  			{ \p s ->  TokenNat1 p }
       	nil  			{ \p s ->  TokenNil p }
       	not  			{ \p s ->  TokenNot p }
       	of  			{ \p s ->  TokenOf p }
	operations  		{ \p s ->  TokenOperations p }
       	or  			{ \p s ->  TokenOr p }
       	others  		{ \p s ->  TokenOthers p }
       	post  			{ \p s ->  TokenPost p }
	power  			{ \p s ->  TokenPower p }
       	pre  			{ \p s ->  TokenPre p }
       	psubset  		{ \p s ->  TokenPsubset p }
       	rat  			{ \p s ->  TokenRat p }
       	rd  			{ \p s ->  TokenRd p }
       	real  			{ \p s ->  TokenReal p }
       	rem  			{ \p s ->  TokenRem p }
       	rng  			{ \p s ->  TokenRng p }
       	seq  			{ \p s ->  TokenSeq p }
       	seq1  			{ \p s ->  TokenSeq1 p }
	set  			{ \p s ->  TokenSet p }
       	skip  			{ \p s ->  TokenSkip p }
       	st  			{ \p s ->  TokenSt p }
       	state 			{ \p s ->  TokenState p }
	subset   		{ \p s ->  TokenSubset p }
	then  			{ \p s ->  TokenThen p }
       	tl  			{ \p s ->  TokenTl p }
       	to  			{ \p s ->  TokenTo p }
       	token  			{ \p s ->  TokenToken p }
	true  			{ \p s ->  TokenTrue p }
       	types  			{ \p s ->  TokenTypes p }
       	union  			{ \p s ->  TokenUnion p }
	using  			{ \p s ->  TokenUsing p }
       	values 			{ \p s ->  TokenValues p }
       	wr  			{ \p s ->  TokenWr p }
	mk_			{ \p s ->  TokenMk p }

	-- Identifiers -- 

	mk_@identifier		{ \p s ->  TokenMkIdent p (drop 3 s) }
	is_@identifier		{ \p s ->  TokenIsIdent p (drop 3 s) }
	@identifier 		{ \p s ->  TokenIdentifier p s }	
	@typeVarId		{ \p s ->  TokenTypeVarId p (tail s) }
	@numeral		{ \p s ->  TokenNumeral p (read(s)) }
	@numericLit		{ \p s ->  TokenNumericLit p (read(s)) }
	@charLit		{ \p s ->  TokenCharLit p ( (init.tail) s ) }
	@stringLit		{ \p s ->  TokenStringLit p ( (init.tail) s ) }

	-- Error -- 

	. 			{ \p s -> lexerror p s }

{
data Token =      -- Keywords --
		  TokenAbs AlexPosn
		| TokenAnd AlexPosn
		| TokenBe AlexPosn
		| TokenBool AlexPosn
		| TokenCard AlexPosn
		| TokenCases AlexPosn
		| TokenChar AlexPosn
		| TokenComp AlexPosn
		| TokenCompose AlexPosn
		| TokenConc AlexPosn
		| TokenDef AlexPosn
		| TokenDinter AlexPosn
		| TokenDiv AlexPosn
		| TokenDom AlexPosn
		| TokenDunion AlexPosn
		| TokenElems AlexPosn
		| TokenElse AlexPosn
		| TokenElseif AlexPosn
		| TokenEnd AlexPosn
		| TokenErrs AlexPosn
		| TokenExists AlexPosn
		| TokenExists1 AlexPosn
		| TokenExt AlexPosn
		| TokenFalse AlexPosn
		| TokenFloor AlexPosn
		| TokenForall AlexPosn
		| TokenFunctions AlexPosn
		| TokenHd AlexPosn
		| TokenIf AlexPosn
		| TokenIn AlexPosn
		| TokenInSet AlexPosn
		| TokenInds AlexPosn
		| TokenInit AlexPosn
		| TokenInmap AlexPosn
		| TokenInt AlexPosn
		| TokenInter AlexPosn
		| TokenInv AlexPosn
		| TokenInverse AlexPosn
		| TokenIota AlexPosn
		| TokenLambda AlexPosn
		| TokenLen AlexPosn
		| TokenLet AlexPosn
		| TokenMap AlexPosn
		| TokenMerge AlexPosn
		| TokenMod AlexPosn
		| TokenMu AlexPosn
		| TokenMunion AlexPosn
		| TokenNat AlexPosn
		| TokenNat1 AlexPosn
		| TokenNil AlexPosn
		| TokenNot AlexPosn
		| TokenOf AlexPosn
		| TokenOperations AlexPosn
		| TokenOr AlexPosn
		| TokenOthers AlexPosn
		| TokenPost AlexPosn
		| TokenPower AlexPosn
		| TokenPre AlexPosn
		| TokenPsubset AlexPosn
		| TokenRat AlexPosn
		| TokenRd AlexPosn
		| TokenReal AlexPosn
		| TokenRem AlexPosn
		| TokenRng AlexPosn
		| TokenSeq AlexPosn
		| TokenSeq1 AlexPosn
		| TokenSet AlexPosn
		| TokenSkip AlexPosn
		| TokenSt AlexPosn
		| TokenState AlexPosn
		| TokenSubset AlexPosn
		| TokenThen AlexPosn
		| TokenTl AlexPosn
		| TokenTo AlexPosn
		| TokenToken AlexPosn
		| TokenTrue AlexPosn
		| TokenTypes AlexPosn
		| TokenUnion AlexPosn
		| TokenUsing AlexPosn
		| TokenValues AlexPosn
		| TokenWr AlexPosn
		| TokenMk AlexPosn
		| TokenIdentifier AlexPosn String
		| TokenTypeVarId AlexPosn String
		| TokenNumeral AlexPosn Int
		| TokenNumericLit AlexPosn Double
		| TokenMkIdent AlexPosn String
		| TokenIsIdent AlexPosn String
		| TokenCharLit AlexPosn String
		| TokenStringLit AlexPosn String
		| TokenQuoteLit AlexPosn String
		| TokenComma AlexPosn
       		| TokenColon AlexPosn
		| TokenColonMinus AlexPosn
		| TokenSemiColon AlexPosn
		| TokenEqual AlexPosn
		| TokenOBracket AlexPosn
		| TokenCBracket AlexPosn
		| TokenVertBar AlexPosn
		| TokenMinus AlexPosn
		| TokenOSqBracket AlexPosn
		| TokenCSqBracket AlexPosn
		| TokenOCurBracket AlexPosn
		| TokenCCurBracket AlexPosn
       		| TokenPlus AlexPosn
       		| TokenSlash AlexPosn
       		| TokenLess AlexPosn
       		| TokenGreat AlexPosn
       		| TokenLessEq AlexPosn
       		| TokenGreatEq AlexPosn
       		| TokenDiff AlexPosn
       		| TokenPeriod AlexPosn
		| TokenAsterisk AlexPosn
       		| TokenPFuncType AlexPosn
       		| TokenTFuncType AlexPosn
		| TokenQQ AlexPosn
       		| TokenImplies AlexPosn
       		| TokenEquiv AlexPosn
       		| TokenMapAp AlexPosn
       		| TokenDomRestTo AlexPosn
       		| TokenRanRestTo AlexPosn
		| TokenDomRestBy AlexPosn
       		| TokenRanRestBy AlexPosn
       		| TokenAmpersand AlexPosn
       		| TokenDoubleEq AlexPosn
       		| TokenDoubleAsterisk AlexPosn
       		| TokenHat AlexPosn
       		| TokenDoublePlus AlexPosn
		| TokenBackSlash AlexPosn
		| TokenDoubleColon AlexPosn
		| TokenAtrib AlexPosn
		| TokenEllipsis AlexPosn
		| TokenOldName AlexPosn
		| TokenTupleSelect AlexPosn
		deriving (Eq, Show)

token_posn :: Token -> AlexPosn
token_posn ( TokenAbs p ) = p
token_posn ( TokenAnd p ) = p
token_posn ( TokenBe p ) = p
token_posn ( TokenBool p ) = p
token_posn ( TokenCard p ) = p
token_posn ( TokenCases p ) = p
token_posn ( TokenChar p ) = p
token_posn ( TokenComp p ) = p
token_posn ( TokenCompose p ) = p
token_posn ( TokenConc p ) = p
token_posn ( TokenDef p ) = p
token_posn ( TokenDinter p ) = p
token_posn ( TokenDiv p ) = p
token_posn ( TokenDom p ) = p
token_posn ( TokenDunion p ) = p
token_posn ( TokenElems p ) = p
token_posn ( TokenElse p ) = p
token_posn ( TokenElseif p ) = p
token_posn ( TokenEnd p ) = p
token_posn ( TokenErrs p ) = p
token_posn ( TokenExists p ) = p
token_posn ( TokenExists1 p ) = p
token_posn ( TokenExt p ) = p
token_posn ( TokenFalse p ) = p
token_posn ( TokenFloor p ) = p
token_posn ( TokenForall p ) = p
token_posn ( TokenFunctions p ) = p
token_posn ( TokenHd p ) = p
token_posn ( TokenIf p ) = p
token_posn ( TokenInSet p ) = p
token_posn ( TokenIn p ) = p
token_posn ( TokenInds p ) = p
token_posn ( TokenInit p ) = p
token_posn ( TokenInmap p ) = p
token_posn ( TokenInt p ) = p
token_posn ( TokenInter p ) = p
token_posn ( TokenInv p ) = p
token_posn ( TokenInverse p ) = p
token_posn ( TokenIota p ) = p
token_posn ( TokenLambda p ) = p
token_posn ( TokenLen p ) = p
token_posn ( TokenLet p ) = p
token_posn ( TokenMap p ) = p
token_posn ( TokenMerge p ) = p
token_posn ( TokenMod p ) = p
token_posn ( TokenMu p ) = p
token_posn ( TokenMunion p ) = p
token_posn ( TokenNat p ) = p
token_posn ( TokenNat1 p ) = p
token_posn ( TokenNil p ) = p
token_posn ( TokenNot p ) = p
token_posn ( TokenOf p ) = p
token_posn ( TokenOperations p ) = p
token_posn ( TokenOr p ) = p
token_posn ( TokenOthers p ) = p
token_posn ( TokenPost p ) = p
token_posn ( TokenPower p ) = p
token_posn ( TokenPre p ) = p
token_posn ( TokenPsubset p ) = p
token_posn ( TokenRat p ) = p
token_posn ( TokenRd p ) = p
token_posn ( TokenReal p ) = p
token_posn ( TokenRem p ) = p
token_posn ( TokenRng p ) = p
token_posn ( TokenSeq p ) = p
token_posn ( TokenSeq1 p ) = p
token_posn ( TokenSet p ) = p
token_posn ( TokenSkip p ) = p
token_posn ( TokenSt p ) = p
token_posn ( TokenState p ) = p
token_posn ( TokenSubset p ) = p
token_posn ( TokenThen p ) = p
token_posn ( TokenTl p ) = p
token_posn ( TokenTo p ) = p
token_posn ( TokenToken p ) = p
token_posn ( TokenTrue p ) = p
token_posn ( TokenTypes p ) = p
token_posn ( TokenUnion p ) = p
token_posn ( TokenUsing p ) = p
token_posn ( TokenValues p ) = p
token_posn ( TokenWr p ) = p
token_posn ( TokenMk p ) = p
token_posn ( TokenIdentifier p _ ) = p
token_posn ( TokenTypeVarId p _ ) = p
token_posn ( TokenNumeral p _ ) = p
token_posn ( TokenNumericLit p _ ) = p
token_posn ( TokenCharLit p _ ) = p
token_posn ( TokenStringLit p _ ) = p
token_posn ( TokenQuoteLit p _ ) = p
token_posn ( TokenMkIdent p _ ) = p
token_posn ( TokenIsIdent p _ ) = p
token_posn ( TokenComma p ) = p
token_posn ( TokenColon p ) = p
token_posn ( TokenColonMinus p ) = p
token_posn ( TokenSemiColon p ) = p
token_posn ( TokenEqual p ) = p
token_posn ( TokenOBracket p ) = p
token_posn ( TokenCBracket p ) = p
token_posn ( TokenVertBar p ) = p
token_posn ( TokenMinus p ) = p
token_posn ( TokenOSqBracket p ) = p
token_posn ( TokenCSqBracket p ) = p
token_posn ( TokenOCurBracket p ) = p
token_posn ( TokenCCurBracket p ) = p
token_posn ( TokenPlus p ) = p
token_posn ( TokenSlash p ) = p
token_posn ( TokenLess p ) = p
token_posn ( TokenGreat p ) = p
token_posn ( TokenLessEq p ) = p
token_posn ( TokenGreatEq p ) = p
token_posn ( TokenDiff p ) = p
token_posn ( TokenPeriod p ) = p
token_posn ( TokenAsterisk p ) = p
token_posn ( TokenPFuncType p ) = p
token_posn ( TokenTFuncType p ) = p
token_posn ( TokenQQ p ) = p
token_posn ( TokenImplies p ) = p
token_posn ( TokenEquiv p ) = p
token_posn ( TokenMapAp p ) = p
token_posn ( TokenDomRestTo p ) = p
token_posn ( TokenRanRestTo p ) = p
token_posn ( TokenDomRestBy p ) = p
token_posn ( TokenRanRestBy p ) = p
token_posn ( TokenAmpersand p ) = p
token_posn ( TokenDoubleEq p ) = p
token_posn ( TokenDoubleAsterisk p ) = p
token_posn ( TokenHat p ) = p
token_posn ( TokenDoublePlus p ) = p
token_posn ( TokenBackSlash p ) = p
token_posn ( TokenDoubleColon p ) = p
token_posn ( TokenAtrib p ) = p
token_posn ( TokenEllipsis p ) = p
token_posn ( TokenOldName p ) = p
token_posn ( TokenTupleSelect p ) = p
--token_posn ( TokenUnder p ) = p

lexerror p s = 
	error("Lexical Error: \nOffset: " ++ show(o) ++ " Line: " ++ show(l) ++
		" Column: " ++ show(c))
		where AlexPn o l c = p

}
