
{
module Anasin(parser) where
import Analex
import Language.VDM_SL.Syntax
}

%name parser
%tokentype { Token }

%token	',' 			{ TokenComma _ }
       	':'  			{ TokenColon _  }
	';'  			{ TokenSemiColon _ }
	":-"			{ TokenColonMinus _ }
	'='  			{ TokenEqual _ }
	'('  			{ TokenOBracket _ }
	')'  			{ TokenCBracket _ }
	'|'  			{ TokenVertBar _ }
	'-'  			{ TokenMinus _ }
	'['  			{ TokenOSqBracket _ }
	']'  			{ TokenCSqBracket _ }
	'{'  			{ TokenOCurBracket _ }
	'}'  			{ TokenCCurBracket _ }
       	'+'  			{ TokenPlus _ }
       	'/'  			{ TokenSlash _ }
       	'<'  			{ TokenLess _ }
       	'>'  			{ TokenGreat _ }
       	"<="  			{ TokenLessEq _ }
       	">="  			{ TokenGreatEq _ }
       	"<>"  			{ TokenDiff _ }
       	'.'  			{ TokenPeriod _ }
	'*'  			{ TokenAsterisk _ }
       	"->"  			{ TokenPFuncType _ }
       	"+>"  			{ TokenTFuncType _ }
	"==>"  			{ TokenQQ _ } 
       	"=>"  			{ TokenImplies _ }
       	"<=>"  			{ TokenEquiv _ }
       	"|->"  			{ TokenMapAp _ }
       	"<:"  			{ TokenDomRestTo _ }
       	":>"  			{ TokenRanRestTo _ }
	"<-:"  			{ TokenDomRestBy _ }
       	":->"  			{ TokenRanRestBy _ }
       	'&'  			{ TokenAmpersand _ }
       	"=="  			{ TokenDoubleEq _ }
       	"**"  			{ TokenDoubleAsterisk _ }
       	'^'  			{ TokenHat _ }
       	"++" 			{ TokenDoublePlus _ }
	'\\'			{ TokenBackSlash _ }
	"::"			{ TokenDoubleColon _ }
	":="			{ TokenAtrib _ }
	"..."			{ TokenEllipsis _ }
	'~'			{ TokenOldName _ }
	".#"			{ TokenTupleSelect _ }
	
	-- Keywords --
	abs			{ TokenAbs _ }
	and 			{ TokenAnd _ }
	be 			{ TokenBe _ }
	bool 			{ TokenBool _ }
	card 			{ TokenCard _ }
       	cases 			{ TokenCases _ }
	char 			{ TokenChar _ }
	comp  			{ TokenComp _ }
	compose  		{ TokenCompose _ }
	conc 			{ TokenConc _ }
	def 			{ TokenDef _ }
	dinter 			{ TokenDinter _ }
	div 			{ TokenDiv _ }
       	dom  			{ TokenDom _ }
       	dunion  		{ TokenDunion _ }
       	elems  			{ TokenElems _ }
       	else  			{ TokenElse _ }
       	elseif  		{ TokenElseif _ }
       	end  			{ TokenEnd _ }
       	errs  			{ TokenErrs _ }
	exists  		{ TokenExists _ }
       	exists1  		{ TokenExists1 _ }
       	ext 			{ TokenExt _ }
	false   		{ TokenFalse _ }
	floor   		{ TokenFloor _ }
       	forall  		{ TokenForall _ }
       	functions  		{ TokenFunctions _ }
	hd  			{ TokenHd _ }
       	if  			{ TokenIf _ }
	"in set"		{ TokenInSet _ }
       	in  			{ TokenIn _ }
       	inds  			{ TokenInds _ }
	init  			{ TokenInit _ }
       	inmap  			{ TokenInmap _ }
       	int  			{ TokenInt _ }
       	inter  			{ TokenInter _ }
	inv  			{ TokenInv _ }
       	inverse  		{ TokenInverse _ }
       	iota  			{ TokenIota _ }
	lambda  		{ TokenLambda _ }
       	len  			{ TokenLen _ }
       	let  			{ TokenLet _ }
       	map  			{ TokenMap _ }
       	merge  			{ TokenMerge _ }
       	mod  			{ TokenMod _ }
	mu  			{ TokenMu _ }
       	munion 			{ TokenMunion _ }
       	nat  			{ TokenNat _ }
       	nat1  			{ TokenNat1 _ }
       	nil  			{ TokenNil _ }
       	not  			{ TokenNot _ }
       	of  			{ TokenOf _ }
	operations  		{ TokenOperations _ }
       	or  			{ TokenOr _ }
       	others  		{ TokenOthers _ }
       	post  			{ TokenPost _ }
	power  			{ TokenPower _ }
       	pre  			{ TokenPre _ }
       	psubset  		{ TokenPsubset _ }
       	rat  			{ TokenRat _ }
       	rd  			{ TokenRd _ }
       	real  			{ TokenReal _ }
       	rem  			{ TokenRem _ }
       	rng  			{ TokenRng _ }
       	seq  			{ TokenSeq _ }
       	seq1  			{ TokenSeq1 _ }
	set  			{ TokenSet _ }
       	skip  			{ TokenSkip _ }
       	st  			{ TokenSt _ }
       	state 			{ TokenState _ }
	subset   		{ TokenSubset _ }
	then  			{ TokenThen _ }
       	tl  			{ TokenTl _ }
       	to  			{ TokenTo _ }
       	token  			{ TokenToken _ }
	true  			{ TokenTrue _ }
       	types  			{ TokenTypes _ }
       	union  			{ TokenUnion _ }
	using  			{ TokenUsing _ }
       	values 			{ TokenValues _ }
       	wr  			{ TokenWr _ }
	mk_			{ TokenMk _ }

	-- Identifiers -- 
	mk_identifier		{ TokenMkIdent _ $$ }
	is_identifier		{ TokenIsIdent _ $$ }
	identifier 		{ TokenIdentifier _ $$ }	
	typeVarId		{ TokenTypeVarId _ $$ }
	numeral			{ TokenNumeral _ $$ }
	Lit_num			{ TokenNumericLit _ $$ }
	Lit_char		{ TokenCharLit _ $$ }
	Lit_string		{ TokenStringLit _ $$ }
	Lit_quote		{ TokenQuoteLit _ $$ }


%left EMPTY CLOSE_SCOPE
%left CTOR -- Contructor

%right "->" "+>" --INMAP_TO MAP_TO
%right INMAP_TO MAP_TO
%left '|'

--Connectives
%left "<=>"
%right "=>"
%left or
%left and
%left not

--Relations
%left "<=" '<' ">=" '>' '=' "<>" subset psubset "in set" NOT_IN_SET

--Evaluators
%left '+' '-' union merge '\\' munion "++" '^'
%left '*' '/' inter rem mod div
%left inverse
%left "<:" "<-:" 
%left ":>" ":->"
%left UNARY_PLUS UNARY_MINUS abs floor card power dinter dunion dom rng UNARY_MERGE len elems hd tl conc inds

%left SEQ_OF SEQ1_OF SET_OF

--Applicators
%left '(' '[' '.' ".#"

--Combinators
%right comp
%right "**"

%left ';'

%%

Document 			:: { Document }
Document : 
	  OpenScope Definition_block		
	  { DEF_BLOCK_S $2 }
	| Document Definition_block	
	  { DEF_BLOCK_L $1 $2 }

Definition_block 		:: { Definition_block }
Definition_block :	 
	  Type_definition_list
	  %prec EMPTY
	  { TYPES $1 }
	| State_definition
	  %prec EMPTY
	  { STATES $1 }
	| Value_definition_list
	  %prec EMPTY
	  { VALUES $1 }
	| Function_definition_list 
	  %prec EMPTY	
	  { FUNCTIONS $1 }
	| Operation_definition_list
	  %prec EMPTY	
	  { OPERATIONS $1 }

Type_definition_list 		:: { Type_definition_list }
Type_definition_list : 
	  types Type_definition	
	  { TYPE_DEF [$2] }
	| Type_definition_list ';' Type_definition	
	  { constTypeDef $1 $3 }


Type_definition 		:: { Type_definition }
Type_definition : 
	  Name '=' Type				
	  { TYPE_TYPE $1 $3 }
	| Name '=' Type Invariant		
	  { TYPE_TYPE_INV $1 $3 $4 }
	| Name "::" Field_list			
	  { TYPE_FIELD_L $1 $3 }
	| Name "::" Field_list Invariant	
	  { TYPE_FIELD_L_INV $1 $3 $4 }

Type_list 			:: { Type_list }
Type_list : 
	  Type		
	  { TYPE_S $1 } 
	| Type_list ',' Type
	  { TYPE_L $1 $3 }

Type 				:: { Type }
Type 	: 
	  '(' Type ')'				
	  { BRACK_TYPE $2 }
     	| bool					
	  { BOOL }
	| nat					
	  { NAT }
	| nat1					
	  { NAT1 }
	| int					
	  { INT }
	| rat					
	  { RAT }
	| real					
	  { REAL }
	| char					
	  { CHAR }
	| token					
	  { TOKEN }
	| Lit_quote				
	  { QUOTE_TYPE $1 }
	| compose Name of Field_list end	
	  { COMPOSE $2 $4 }
	| Type '|' Type				
	  { UNION_TYPE $1 $3 }
	| Type '*' Type	     			
	  { PROD_TYPE $1 $3 }
	| '[' Type ']'				
	  { OPT_TYPE $2 }
	| set of Type 	
	  %prec SET_OF	
	  { SET_OF $3 }
	| seq of Type	
	  %prec SEQ_OF	
	  { SEQ0_OF $3 }
	| seq1 of Type
	  %prec SEQ1_OF	
	  { SEQ1_OF $3 }
	| map Type to Type
	  %prec MAP_TO	
	  { MAP_TYPE $2 $4 }
	| inmap Type to Type 
	  %prec INMAP_TO	
	  { INMAP_TYPE $2 $4 }
	| Function_type				
	  { FUNC_TYPE $1 }
	| Type_variable_identifier		
	  { TYPE_VAR $1 }
	| Name  				
	  { NAME_TYPE $1 }

Function_type 			:: { Function_type }
Function_type : 
	  Type "->" Type		
	  { TOT_FUNC $1 $3 }
	| '(' ')' "->" Type		
	  { TOT_FUNC0 $4 }
	| Type "+>" Type		
	  { PART_FUNC $1 $3 }
	| '(' ')' "+>" Type  		
	  { PART_FUNC0 $4 }

Discretionary_type 		:: { Discretionary_type }
Discretionary_type : 
	  Type			
	  { SOME $1 }
	| '(' ')'  		
	  { NOTH }

Field 				:: { Field }
Field 	: 
	  Name ':' Type				
	  { FIELD_NAME $1 $3 }
	| Name ":-" Type
	  { FIELD_NAME_IGNORE $1 $3 }
	| Type  				
	  { FIELD_TYPE $1 }

Field_list 			:: { Field_list }
Field_list :
	  {-empty-}			
	  { FIELD_LIST [] }
	| Field_list Field  		
	  { constFieldList $1 $2 }

State_definition 		:: { State_definition }
State_definition : 
	  state Name of Field_list end	
	  { STATE $2 $4 }
	| state Name of Field_list Invariant end	
	  { STATE_INV $2 $4 $5 }
	| state Name of Field_list Initialization end	
	  { STATE_INI $2 $4 $5 }
	| state Name of Field_list Invariant Initialization end	
	  { STATE_INV_INI $2 $4 $5 $6 }
	| state Name of Field_list Initialization Invariant end  	
	  { STATE_INV_INI $2 $4 $6 $5 }

Invariant 			:: { Invariant }
Invariant : 
	  inv Pattern "==" Expression
	  %prec EMPTY 	
	  { INVARIANT $2 $4 }

Initialization 			:: { Initialization }
Initialization : 
	  init Pattern "==" Expression		
	  { INITIALIZATION $2 $4 }

Value_definition_list 		:: { Value_definition_list }
Value_definition_list : 
	  values Value_definition	
	  { VAL_DEF_S $2 }
	| Value_definition_list ';' Value_definition	
	  { VAL_DEF_L $1 $3}

Value_definition 		:: { Value_definition }
Value_definition : 
	  Pattern '=' Expression	
	  { PATTERN $1 $3 }
	| Pattern ':' Type '=' Expression  	
	  { PATTERN_TYPE $1 $3 $5 }

Function_definition_list 	:: { Function_definition_list }
Function_definition_list : 
	  functions OpenScope Function_definition CloseScope	
	  { FUNC_DEF_S $3 }
	| Function_definition_list ';' OpenScope Function_definition CloseScope	
	  { FUNC_DEF_L $1 $4 }

Function_definition 		:: { Function_definition }
Function_definition : 
	  Explicit_function_definition	
	  { EXPL_FUNC_DEF $1 }
	| Implicit_function_definition  
	  { IMPL_FUNC_DEF $1 }

Explicit_function_definition 	:: { Explicit_function_definition }
Explicit_function_definition : 
	  Name ':' Function_type Name Parameters_list "==" Expression Maybe_precondition	
	  { EXPL_FUNC $1 $3 $4 $5 $7 $8 }
	| Name '[' Type_variable_list ']' ':' Function_type Name Parameters_list "==" Expression Maybe_precondition	
	  { EXPL_FUNC_TYPE_VAR $1 $3 $6 $7 $8 $10 $11 }   
				
Implicit_function_definition 	:: { Implicit_function_definition }	
Implicit_function_definition : 
	  Name Parameter_type_list DeclName ':' Type Maybe_precondition post Expression	
	  { IMPL_FUNC $1 $2 $3 $5 $6 $8 }
	| Name '[' Type_variable_list ']' Parameter_type_list DeclName ':' Type Maybe_precondition post Expression	
	  { IMPL_FUNC_TYPE $1 $3 $5 $6 $8 $9 $11 }   
				
Type_variable_list 		:: { Type_variable_list }
Type_variable_list : 
	  Type_variable_identifier	
	  { TYPE_VAR_S $1 }
	| Type_variable_list ',' Type_variable_identifier	
	  { TYPE_VAR_L $1 $3 }

Type_variable_identifier 	:: { String }
Type_variable_identifier : 
	  typeVarId 		
	  { $1 }

Parameter_type_list 		:: { Parameter_type_list }
Parameter_type_list : 
	  '(' ')'	
	  { PARAM_TYPE_E }
	| '(' Pattern_type_pair_list ')'  	
	  { PARAM_TYPE_L $2 }

Pattern_type_pair_list 		:: { Pattern_type_pair_list }
Pattern_type_pair_list : 
	  Pattern_list ':' Type		
	  { PAT_TYPE_S $1 $3 }
	| Pattern_type_pair_list ',' Pattern_list ':' Type	
	  { PAT_TYPE_L $1 $3 $5 }

Parameters_list 		:: { Parameters_list }
Parameters_list : 
	  Brack_maybe_pattern_list	
	  { PARAM_S $1 }
	| Parameters_list Brack_maybe_pattern_list	
	  { PARAM_L $1 $2 }  

Maybe_precondition 		:: { Maybe_precondition }
Maybe_precondition :
	  {-empty-}					
	  { MAY_PRECOND_NOTH }
	| pre BeginPreCond Expression EndPreCond  	
	  { MAY_PRECOND_JUST $3 }

Brack_maybe_pattern_list 	:: { Brack_maybe_pattern_list }
Brack_maybe_pattern_list : 
	  '(' ')'	
	  { BRACK_MAY_NOTH }
	| '(' Pattern_list ')'		
	  { BRACK_MAY_JUST $2 }  

Operation_definition_list 	:: { Operation_definition_list }
Operation_definition_list : 
	  operations OpenScope Operation_definition CloseScope	
	  { OPER_DEF_S $3 }
	| Operation_definition_list ';' OpenScope Operation_definition CloseScope
	  { OPER_DEF_L $1 $4 }

Operation_definition 		:: { Operation_definition }
Operation_definition : 
	  Explicit_operation_definition		
	  { EXPL_OPER_DEF $1 }
	| Implicit_operation_definition  	
	  { IMPL_OPER_DEF $1 }

Explicit_operation_definition 	:: { Explicit_operation_definition }
Explicit_operation_definition : 
	  Name ':' Discretionary_type "==>" Discretionary_type Name Brack_maybe_pattern_list "==" Statement Maybe_precondition  	
	  { EXPL_OPER $1 $3 $5 $6 $7 $9 $10 }

Implicit_operation_definition 	:: { Implicit_operation_definition }
Implicit_operation_definition : 
	  Name Parameter_type_list Maybe_externals Maybe_precondition post Expression
	  { IMPL_OPER $1 $2 $3 $4 $6 }
	| Name Parameter_type_list Maybe_externals Maybe_precondition post Expression errs Exception_list	
	  { IMPL_OPER_EXCEP $1 $2 $3 $4 $6 $8 }
	| Name Parameter_type_list DeclName ':' Type Maybe_externals Maybe_precondition post Expression	
	  { IMPL_OPER_TYPE $1 $2 $3 $5 $6 $7 $9 }
	| Name Parameter_type_list DeclName ':' Type Maybe_externals Maybe_precondition post Expression errs Exception_list   
	  { IMPL_OPER_TYPE_EXCEP $1 $2 $3 $5 $6 $7 $9 $11 }
				
Maybe_externals 		:: { Maybe_externals }
Maybe_externals :
	  {-empty-}	
	  { MAY_EXTERN_NOTH }
	| Externals 	
	  { MAY_EXTERN_JUST $1 }

Externals 			:: { External_list }
Externals : 
	  ext Var_information			
	  { EXTERNAL_S $2 }
	| Externals Var_information		
	  { EXTERNAL_L $1 $2 }

Var_information 		:: { Var_information }
Var_information : 
	  rd Name_list			
	  { RD $2 }
	| wr Name_list			
	  { WR $2 }
	| rd Name_list ':' Type		
	  { RD_TYPE $2 $4 }
	| wr Name_list ':' Type  	
	  { WR_TYPE $2 $4 }

Exception_list 			:: { Exception_list }
Exception_list : 
	  Name ':' Expression "->" Expression	
	  { EXCEPT_S $1 $3 $5 }
	| Exception_list Name ':' Expression "->" Expression  
	  { EXCEPT_L $1 $2 $4 $6 }

Expression_list 		:: { Expression_list }
Expression_list : 
	  Expression 				
	  { EXPR_S $1 }
	| Expression_list ',' Expression	
	  { EXPR_L $1 $3 }

Addto_expression_list 		:: { Expression_list }
Addto_expression_list : 
	  Expression			
	  { EXPR_S $1 }
	| Addto_expression_list ',' Expression	
	  { EXPR_L $1 $3 }

Expression 			:: { Expression }
Expression : 
	  '(' Expression ')'
	  %prec CTOR
	  { BRACK_EXPR $2 }
	| let OpenScope Local_definition_list in Expression CloseScope	
	  { LET $3 $5 }
	| let OpenScope Pattern "in set" Expression in Expression CloseScope	 
	  %prec CTOR 
	  { LET_INSET $3 $5 $7 }
	| let OpenScope Name ':' Type in Expression CloseScope  
	  %prec CTOR	
	  { LET_VAR $3 $5 $7 }
	| let OpenScope Pattern2 ':' Type in Expression CloseScope 	
	  %prec CTOR
	  { LET_PAT $3 $5 $7 } 
	| let OpenScope Pattern "in set" Expression be st Expression in Expression CloseScope 
	  %prec CTOR	
	  { LETBE_INSET $3 $5 $8 $10 }
	| let OpenScope Name ':' Type be st Expression in Expression CloseScope 
	  %prec CTOR	
	  { LETBE_VAR $3 $5 $8 $10 }
	| let OpenScope Pattern2 ':' Type be st Expression in Expression CloseScope 
	  %prec CTOR	
	  { LETBE_PAT $3 $5 $8 $10 }
	| def OpenScope Patternbind_expr_list in Expression CloseScope 
	  %prec CTOR
	  { DEF $3 $5 }
	| If_expression	
	  { IF_EXPR $1 }
	| cases OpenScope Expression ':' Cases_alternatives end CloseScope	
	  { CASE $3 $5 }
	| cases OpenScope Expression ':' Cases_alternatives ',' others "->" Expression end CloseScope		
	  { CASE_OTHER $3 $5 $9 }
	| '+' Expression
	  %prec UNARY_PLUS
	  { PLUS $2 }
	| '-' Expression
	  %prec UNARY_MINUS
	  { MINUS $2 }
	| abs Expression			
	  { ABS $2 }
	| floor Expression			
	  { FLOOR $2 }
	| not Expression			
	  { NOT $2 }
	| card Expression			
	  { CARD $2 }
	| power Expression			
	  { POWER $2 }
	| dunion Expression			
	  { DUNION $2 }
	| dinter Expression			
	  { DINTER $2 }
	| hd Expression			
	  { HD $2 }
	| tl Expression			
	  { TL $2 }
	| len Expression			
	  { LEN $2 }
	| elems Expression			
	  { ELEMS $2 }
	| inds Expression			
	   { INDS $2 }
	| conc Expression			
	  { CONC $2 }
	| dom Expression			
	  { DOM $2 }
	| rng Expression			
	  { RANGE $2 }
	| merge Expression 
	  %prec UNARY_MERGE
	  { MERGE1 $2 }
	| inverse Expression			
	  { INVERSE $2 }
	| Expression '+' Expression		
	  { ADD $1 $3 }
	| Expression '-' Expression		
	  { SUBTR $1 $3 }
	| Expression '*' Expression		
	  { MULT $1 $3 }
	| Expression '/' Expression		
	  { DIVIDE $1 $3 }
	| Expression div Expression		
	  { DIV $1 $3 }
	| Expression rem Expression		
	  { REM $1 $3 }
	| Expression mod Expression		
	  { MOD $1 $3 }
	| Expression '<' Expression		
	  { LESS $1 $3 }
	| Expression "<=" Expression		
	  { LESS_EQ $1 $3 }
	| Expression '>' Expression		
	  { GREAT $1 $3 }
	| Expression ">=" Expression		
	  { GREAT_EQ $1 $3 }
	| Expression '=' Expression		
	  { EQUAL $1 $3 }
	| Expression "<>" Expression		
	  { NEQ $1 $3 }
	| Expression or Expression		
	  { OR $1 $3 }
	| Expression and Expression		
	  { AND $1 $3 }
	| Expression "=>" Expression		
	  { IMPLY $1 $3 }
	| Expression "<=>" Expression	
	  { EQUIV $1 $3 }
	| Expression "in set" Expression	
	  { IN_SET $1 $3 }
	| Expression not "in set" Expression
	  %prec NOT_IN_SET 
	  { NOT_IN_SET $1 $4 }
	| Expression subset Expression	
	  { SUBSET $1 $3 }
	| Expression psubset Expression	
	  { PSUBSET $1 $3 }
	| Expression union Expression	
	  { UNION $1 $3 }
	| Expression merge Expression	
	  { MERGE2 $1 $3 }
	| Expression '\\' Expression		
	  { SET_DIFF $1 $3 }
	| Expression inter Expression	
	  { INTER $1 $3 }
	| Expression '^' Expression		
	  { CONCAT $1 $3 }
	| Expression "++" Expression		
	  { MODIFY $1 $3 }
	| Expression munion Expression	
	  { MUNION $1 $3 }
	| Expression "<:" Expression		
	  { DOM_RESTR_TO $1 $3 }
	| Expression "<-:" Expression	
	  { DOM_RESTR_BY $1 $3 }
	| Expression ":>" Expression		
	  { RNG_RESTR_TO $1 $3 }
	| Expression ":->" Expression	
	  { RNG_RESTR_BY $1 $3 }
	| Expression comp Expression		
	  { COMP $1 $3 }
	| Expression "**" Expression		
	  { ITERATE $1 $3 }
	| forall OpenScope Bind_list '&' Expression CloseScope 	
	  %prec CTOR	
	  { FORALL $3 $5 }
	| exists OpenScope Bind_list '&' Expression CloseScope 	
	  %prec CTOR	
	  { EXISTS $3 $5 }
	| exists1 OpenScope Bind '&' Expression CloseScope 	
	  %prec CTOR	
	  { EXISTS1 $3 $5 }
	| iota OpenScope Bind '&' Expression CloseScope 	
	  %prec CTOR	
	  { IOTA $3 $5 }
	| '{' '}'				
	  { EMPTY_SET }
	| '{' Expression '}'			
	  { SET $2 }
	| '{' Expression ',' Expression '}'	
	  { SET2 $2 $4 }
	| '{' Expression ',' Expression ',' Addto_expression_list '}'	
	  { SET_ENUM $2 $4 $6 }
	| '{' Expression '|' OpenComprehensionScope Bind_list '}' CloseScope	
	  { COMPR_SET $2 $5 }
	| '{' Expression '|' OpenComprehensionScope Bind_list '&' Expression '}' CloseScope	
	  { COMPR_SET_COND $2 $5 $7 }
	| '{' Expression ',' "..." ',' Expression '}'	
	  { RANGE_SET $2 $6 }
	| '[' ']'				
	  { EMPTY_SEQ }
	| '[' Expression_list ']'		
	  { SEQ $2 }
	| '[' Expression '|' OpenComprehensionScope Bind_list ']' CloseScope	
	  { COMPR_SEQ $2 $5 }
	| '[' Expression '|' OpenComprehensionScope Bind_list '&' Expression ']' CloseScope	
	  { COMPR_SEQ_COND $2 $5 $7 }
	| Expression '(' Expression ',' "..." ',' Expression ')'	
	  { SUBSEQ $1 $3 $7 }
	| '{' "|->" '}'			
	  { EMPTY_MAP }
	| '{' Map_enumeration_list '}'	
	  { MAP $2 }
	| '{' Expression "|->" Expression '|' OpenComprehensionScope Bind_list '}' CloseScope	
	  { COMPR_MAP $2 $4 $7 }
	| '{' Expression "|->" Expression '|' OpenComprehensionScope Bind_list '&' Expression '}' CloseScope		
	  { COMPR_MAP_COND $2 $4 $7 $9 }
	| mk_'(' Expression_list ')' 		
	  { TUPLE $3 }
	| mk_identifier '(' ')'			
	  { RECORD_E $1 }
	| mk_identifier '(' Expression_list ')'	
	  { RECORD_S $1 $3 }
	| mu '(' Expression ',' Record_modification_list ')'	
	  { MU $3 $5 }
	| Expression '(' ')'				
	  { APPLY0 $1 }
	| Expression '(' Expression ')'		
	  { APPLY1  $1 $3 }
	| Expression '(' Expression ',' Addto_expression_list ')'	
	  { APPLYN $1 $3 $5 }
	| Expression '.' Name			
	  { FIELD_SEL $1 $3 }
  	| Expression ".#" numeral 			
    	  { TUPLE_SEL $1 $3 }
	| Name '[' Type_list ']'			
	  { FUNCT_TYPE_INST $1 $3 }
	| lambda Type_bind_list '&' Expression
	  %prec CTOR	
	  { LAMBDA $2 $4 }
	| is_identifier '(' Expression ')'		
	  { IS_IDENT $1 $3 }
	| identifier 				
	  { IDENT $1 }
	| Name '~'					
	  { OLD_NAME $1 }
	| Symbolic_literal  				
	  { SYMB_LIT $1 }

Symbolic_literal 		:: { Symbolic_literal }
Symbolic_literal : 
	  Lit_num		
	  { LIT_NUM $1 }
	| numeral
	  { NUMERAL $1 }
	| true 		
	  { TRUE }
	| false		
	  { FALSE }
	| nil 			
	  { NIL }
	| Lit_char		
	  { LIT_CHAR $1 }
	| Lit_string		
	  { LIT_STR $1 }
	| Lit_quote  		
	  { LIT_QUOTE $1 }

Patternbind_expr_list 		:: { Patternbind_expr_list }
Patternbind_expr_list : 
	  Pattern_bind '=' Expression	
	  { PATT_BIND_EXPR_S $1 $3 }
	| Patternbind_expr_list ';' Pattern_bind '=' Expression	
	  { PATT_BIND_EXPR_L $1 $3 $5 }

Local_definition_list 		:: { Local_definition_list }
Local_definition_list : 
	  Local_definition	
	  { LOCAL_DEF_S $1 }
	| Local_definition_list ',' Local_definition	
	  { LOCAL_DEF_L $1 $3 }

If_expression 			:: { If_expression }
If_expression : 
	  if Expression then Expression else Expression	 
	  %prec CTOR 
	  { IF_THEN_ELSE $2 $4 $6 }
	| if Expression then Expression Elseif_expression else Expression 
	  %prec CTOR
	  { IF_THEN_ELSEIF $2 $4 $5 $7 }

Elseif_expression 		:: { Elseif_expression }
Elseif_expression : 
	  elseif Expression then Expression	
	  { ELSEIF $2 $4 }
	| Elseif_expression elseif Expression then Expression 
	  { EXP_ELSEIF $1 $3 $5 }

Cases_alternatives 		:: { Cases_alternative_list }
Cases_alternatives : 
	  Pattern_list "->" Expression	
	  { CASE_ALT_S $1 $3 }
	| Cases_alternatives ',' Pattern_list "->" Expression	
	  { CASE_ALT_L $1 $3 $5 }

Name_list 			:: { Name_list }
Name_list : 
	  Name		
	  { NAME_S $1 }
	| Name_list ',' Name  
	  { NAME_L $1 $3 }

Map_enumeration_list 		:: { Map_enumeration_list }
Map_enumeration_list : 
	  Expression "|->" Expression	
	  { MAP_ENUM_S $1 $3 }
	| Map_enumeration_list ',' Expression "|->" Expression	
	  { MAP_ENUM_L $1 $3 $5 }

Record_modification_list 	:: { Record_modification_list }
Record_modification_list : 
	  Name "|->" Expression	
	  { RECORD_MOD_S $1 $3 }
	| Record_modification_list ',' Name "|->" Expression	
	  { RECORD_MOD_L $1 $3 $5 }

Name 				:: { Name }
Name 	: 
	  identifier		
	  { IDENTIFIER $1 }
	| mk_identifier		
	  { MK_ID $1 }
	| is_identifier   	
	  { IS_ID $1 }
	
DeclName 			:: { Name }
DeclName : 
	  Name   		
	  { $1 }

State_designator 		:: { State_designator }
State_designator : 
	  Name					
	  { STATE_S $1 }
	| State_designator '.' Name		
	  { STATE_L $1 $3 }
	| State_designator '(' Expression ')'  
	  { STATE_EXPR_L $1 $3 }

Statement 			:: { Statement }
Statement : 
	  let Local_definition_list in Statement		
	  { LET_STAT $2 $4 }
	| let Pattern "in set" Expression in Statement	
	  { LET_INSET_STAT $2 $4 $6 }
	| let Name ':' Type in Statement			
	  { LET_VAR_STAT $2 $4 $6 }
	| let Pattern2 ':' Type in Statement		
	  { LET_PAT_STAT $2 $4 $6 }
	| let Pattern "in set" Expression be st Expression in Statement	
	  { LETBE_INSET_STAT $2 $4 $7 $9 }
	| let Name ':' Type be st Expression in Statement	
	  { LETBE_VAR_STAT $2 $4 $7 $9 }
	| let Pattern2 ':' Type be st Expression in Statement	
	  { LETBE_PAT_STAT $2 $4 $7 $9 }
	| def Equals_definition_list in Statement		
	  { DEF_STAT $2 $4 }
	| '(' Maybe_dcl_statement_list Statement_list ')'	
	  { MAYBE_DCL_STAT $2 $3 }
	| Call_statement					
	  { CALL_STAT $1 }
	| skip  						
	  { SKIP }

Call_statement 			:: { Call_statement }
Call_statement : 
	  Name '(' ')'				
	  { CALL_ID_STAT $1 }
	| Name '(' Expression_list ')'		
	  { CALL_EXPR_STAT $1 $3 }
	| Name '(' ')' using State_designator	
	  { CALL_STATE_STAT $1 $5 }
	| Name '(' Expression_list ')' using State_designator 
	  { CALL_EXPR_STATE_STAT $1 $3 $6 } 

Equals_definition_list 		:: { Equals_definition_list }
Equals_definition_list : 
	  Pattern_bind '=' Expression	
	  { EQUALS_DEF_S $1 $3 }
	| Equals_definition_list ';' Pattern_bind '=' Expression  
	  { EQUALS_DEF_L $1 $3 $5 }

Maybe_dcl_statement_list 	:: { Maybe_dcl_statement_list }
Maybe_dcl_statement_list :
	  {-empty-}		
	  { MAYBE_DCL_STAT_E }
	| Maybe_dcl_statement_list Name ':' Type ';'	
	  { MAYBE_DCL_STAT_S $1 $2 $4 }
	| Maybe_dcl_statement_list Name ':' Type ":=" Expression ';'	
	  { MAYBE_DCL_STAT_L $1 $2 $4 $6 }
			 
Statement_list 			:: { Statement_list }
Statement_list : 
	  Statement			
	  { STATEMENT_S $1 }
	| Statement_list ';' Statement  
	  { STATEMENT_L $1 $3 }

Pattern 			:: { Pattern }
Pattern : 
	  Name		
	  { PAT_NAME $1 }
	| Pattern2  	
	  { PAT_PAT $1 }

Pattern2 			:: { Pattern2 }
Pattern2 : 
	  '-'					
	  { IDENT_PAT }
	| '(' Expression ')'			
	  { BRACK_PAT $2 }
	| Symbolic_literal			
	  { SYMB_LIT_PAT $1 }
	| '{' Pattern_list '}'			
	  { SET_ENUM_PAT $2 }
	| Pattern union Pattern		
	  { SET_UNION_PAT $1 $3 }
	| '[' Pattern_list ']'			
	  { SEQ_ENUM_PAT $2 }
	| Pattern '^' Pattern			
	  { SEQ_CONC_PAT $1 $3 }
	| mk_ '(' Pattern_list ')'		
	  { TUPLE_PAT $3 }
	| Name Brack_maybe_pattern_list  	
	  { LIST_PAT $1 $2 }

Pattern_list 			:: { Pattern_list }	
Pattern_list : 
	  Pattern			
	  { PATTERN_S $1 }
	| Pattern_list ',' Pattern	
	  { PATTERN_L $1 $3 }

Pattern_bind :: { Pattern_bind }
Pattern_bind : 
	  Pattern	
	  { PAT_BIND_PAT $1 }
	| Bind  	
	  { PAT_BIND_BIND $1 }

Bind 				:: { Bind }
Bind 	: 
	  Pattern "in set" Expression	
	  { BIND_EXPR $1 $3 }
	| Pattern ':' Type  		
	  { BIND_TYPE $1 $3 }

Bind_list 			:: { Bind_list }
Bind_list : 
	  Multiple_set_bind			
	  { BIND_S1 $1 }
	| Multiple_type_bind			
	  { BIND_S2 $1 }
	| Bind_list ',' Multiple_set_bind	
	  { BIND_L1 $1 $3 }
	| Bind_list ',' Multiple_type_bind	
	  { BIND_L2 $1 $3 }

Multiple_set_bind 		:: { Multiple_set_bind }
Multiple_set_bind : 
	  Pattern_list "in set" Expression   	
	  { MULT_SET_BIND $1 $3 }

Multiple_type_bind 		:: { Multiple_type_bind }
Multiple_type_bind : 
	  Pattern_list ':' Type  		
	  { MULT_TYPE_BIND $1 $3 }

Type_bind_list 			:: { Type_bind_list }
Type_bind_list : 
	  Pattern ':' Type			
	  { TYPE_BIND_S $1 $3 }
	| Type_bind_list ',' Pattern ':' Type	
	  { TYPE_BIND_L $1 $3 $5 }

Local_definition 		:: { Local_definition }
Local_definition : 
	  Pattern '=' Expression		
	  { L_PAT $1 $3 }
	| Name ':' Type '=' Expression		
	  { L_NAME $1 $3 $5 }
	| Pattern2 ':' Type '=' Expression	
	  { L_PAT_TYPE $1 $3 $5 }
	| Name ':' Function_type Name Parameters_list "==" Expression Maybe_precondition	
	  { L_EXPL_FUNC $1 $3 $4 $5 $7 $8 }
	| Name '[' Type_variable_list ']' ':' Function_type Name Parameters_list "==" Expression Maybe_precondition	
	  { L_EXPL_FUNC_TYPE $1 $3 $6 $7 $8 $10 $11 }
	| Name Parameter_type_list DeclName ':' Type Maybe_precondition post Expression	
	  { L_IMPL_FUNC $1 $2 $3 $5 $6 $8 }
	| Name '[' Type_variable_list ']' Parameter_type_list DeclName ':' Type Maybe_precondition post Expression   
	  { L_IMPL_FUNC_TYPE $1 $3 $5 $6 $8 $9 $11 }
		 
OpenScope 			:: { }
OpenScope :  			   { }  

CloseScope 			:: { }	
CloseScope :  %prec CLOSE_SCOPE    { }

OpenComprehensionScope 		:: { }
OpenComprehensionScope : 	   { }

BeginPreCond 			:: { }
BeginPreCond : 			   { }

EndPreCond 			:: { }
EndPreCond : 			   { }


{
constTypeDef :: Type_definition_list -> Type_definition -> Type_definition_list
constTypeDef (TYPE_DEF xs) x = TYPE_DEF (xs ++ [x])

constFieldList :: Field_list -> Field -> Field_list
constFieldList (FIELD_LIST xs) x = FIELD_LIST (xs ++ [x])

happyError :: [ Token ] -> a
happyError tks = error ( "Parse error at" ++ lcn ++ "\n" )
		 where 
			lcn = case tks of
			      [] -> " end of file"
			      tk:_ -> " line " ++ show l ++ ", column " ++ show c
					where
						AlexPn _ l c = token_posn tk
}
