
{ 
module Preprocessor(preprocessor) where
}

-- This preprocessor does not work if the begin and end are inside verbatim.
-- The preprocessor preserves the new line characters in order to 
-- give accurate error positions in the syntax analysis.

%wrapper "monad"

tokens :- 
	<0>"\begin{vdm_al}"	{ (\a b -> return ( Right [])) `andBegin` vdm }
	<0>"\end{vdm_al}"	{ endError }
	<0>"--"			{ begin genComment }
	<0>.			{ \a b -> return $ Left (pass a b) }
	<0>\n			{ \a b -> return $ Left (pass a b) }

	<vdm>"\end{vdm_al}"	{ begin disc }
	<vdm>"\begin{vdm_al}"	{ beginError }
	<vdm>"--"		{ begin vdmComment }
	<vdm>.			{ \a b -> return $ Right (pass a b) }
	<vdm>\n			{ \a b -> return $ Right (pass a b) }
	
	<genComment>.		{ skip }
	<genComment>\n		{ (\a b -> return (Left (pass a b))) `andBegin` 0 }

	<vdmComment>.		{ skip }
	<vdmComment>\n		{ (\a b -> return (Right (pass a b))) `andBegin` vdm }

	<disc>"\begin{vdm_al}"	{ begin vdm }
	<disc>"\end{vdm_al}"	{ endError }
	<disc>.			{ skip }
	<disc>\n		{ \a b -> return $ Right (pass a b) }

{

pass :: AlexInput -> Int -> String
pass (_, _, inp) _ = [head inp]

preprocessor :: String -> String 
preprocessor str = case pre str of
		Left s -> error s
		Right l -> case l of
				Left a -> a
				Right a -> a

pre :: String -> Either String (Either String String)
pre str = runAlex str func
  where func :: Alex (Either String String)
        func = do tok <- alexMonadScan
      		  if tok == Left [] 
		    then return $ Left []
		    else func2 tok
	       where func2 tok = do
			rest <- func
			case tok of 
			  Left a -> case rest of
			  	      Left rst -> return $ Left (a ++ rst)
				      Right rst -> 
				         return $ Right ( (leaveNewLine a ) ++ rst)
			  Right a -> case rest of
			  	      Left rst -> return $ Right a
				      Right rst -> return $ Right (a ++ rst)

leaveNewLine :: String -> String
leaveNewLine = filter (=='\n')

alexEOF :: Alex (Either String a)
alexEOF = do 
	s <- alexGetStartCode
	if s == 0 || s == disc || s == genComment 
	  then return $ Left []
	  else error ("Missing: \\end{vdm_al}" )

endError :: AlexInput -> Int -> Alex a 
endError ( (AlexPn _ ln col), _, _) _ = 
	alexError ("Unexpected \\end{vdm_al} found\n" ++ 
	       "Line: " ++ show ln ++ " Column: " ++ show col)

beginError :: AlexInput -> Int -> Alex a
beginError ( (AlexPn _ ln col), _,_) _ = 
	alexError ("Unexpected \\begin{vdm_al} found\n" ++
	       "Line: " ++ show ln ++ " Column: " ++ show col)
}
