
module Main where
import Preprocessor
import Analex
import Anasin
import Language.VDM_SL.TransfEngine
import VDM2PLSql 
import Language.PLSql.Syntax
import Language.PLSql.Pretty
import Language.VDM_SL.Pretty
import System.Console.GetOpt
import System.Environment

main :: IO ()
main = do 
   args <- getArgs
   cmd  <- cmdLine args
   fls  <- cmdProc cmd
   proc fls 
   where 
      proc (Nothing) = putStr cmdUsageInfo 
      proc (Just (inpFile, vdmFile, sqlFile)) =
         do
            code <- readFile inpFile
	    doc  <- (applyTransf . parser . alexScanTokens . preprocessor) code
	    saveVdm vdmFile doc
	    saveSql sqlFile doc
	    where 
	       saveVdm Nothing _ = return ()
	       saveVdm (Just file) doc = writeFile file (Language.VDM_SL.Pretty.prettyPrint doc)
	       saveSql Nothing _ = return ()
	       saveSql (Just file) doc = do
	          sql <- vdm2plsql doc 
		  writeFile file (toSqlStr sql)
	       toSqlStr = concatMap (\x -> (Language.PLSql.Pretty.prettyPrint x) ++ "\n" )  

data Flag = Input String 
	  | OutputVDM String
	  | OutputSQL String
	  | Help
	  deriving (Eq, Show)
	  
options :: [OptDescr Flag]
options = 
	[ Option ['i'] ["input"] (ReqArg Input "Input file") "VDM input file"
	, Option ['v'] ["vdm"] (ReqArg OutputVDM "VDM Output file") "VDM Output File"
	, Option ['s'] ["sql"] (ReqArg OutputSQL "SQL Output file") "SQL output file"
	, Option ['h'] ["help"] (NoArg Help) "Show this help screen"
	]

cmdLine :: [String] -> IO ([Flag], [String])
cmdLine argv = 
	case (getOpt RequireOrder options argv) of
		(o, n, []) -> return (o,n)
		(_,_,errs) -> cmdLineError errs

cmdLineError :: [String] -> IO a
cmdLineError errs = ioError $ userError $ concat errs ++ cmdUsageInfo

cmdUsageInfo :: String
cmdUsageInfo = usageInfo header options
	where header = "Usage: "
	
cmdProc :: ([Flag], [String]) -> IO (Maybe (String, Maybe String, Maybe String))
cmdProc (args, str) = 
	if (elem Help args) 
	then return Nothing
	else if ( null ( inpFilter )  )
	then cmdLineError (str ++ ["\nInput file expected\n"])
	else if (vdmFile == Nothing && sqlFile == Nothing )
	then cmdLineError (str ++ ["\nNo output file specified\n"])
	else return (Just (inpFile, vdmFile, sqlFile))
	where
		inpFile = let Input file = head ( inpFilter ) in file
		vdmFile = if ( null ( vdmFilter ) )
			  then Nothing
			  else let OutputVDM f = head ( vdmFilter ) in Just f 
		sqlFile = if ( null ( sqlFilter ) )
			  then Nothing
			  else let OutputSQL f = head ( sqlFilter ) in Just f
		inpFilter :: [Flag]
		inpFilter = filter isInput args
		isInput (Input _) = True
		isInput _ = False
		vdmFilter = filter isOutVDM args
		isOutVDM (OutputVDM _ ) = True
		isOutVDM _ = False
		sqlFilter = filter  isOutSQL args
		isOutSQL (OutputSQL _ ) = True
		isOutSQL _ = False
